-- 电站表
CREATE TABLE IF NOT EXISTS 
station(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    name TEXT(100) NOT NULL, --电站名称
    address TEXT(200) NOT NULL, --电站地址
    lng REAL NOT NULL, --经度
    lat REAL NOT NULL, --纬度
    time_zone TEXT(50) NOT NULL, --时区
    remark TEXT(1200), --备注
    UNIQUE(name) -- 电站名称唯一
);

-- -- 模板表
-- CREATE TABLE IF NOT EXISTS  
-- template(
--     id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
--     type TINYINT NOT NULL, --模板类别（0设备配置, 1北向配置）
--     protocol_type INTEGER NOT NULL, --协议类型
--     dev_type INTEGER, --设备类型
--     name TEXT(100) NOT NULL, --名称
--     remark TEXT(1200), --备注
--     UNIQUE(name) --模板名称唯一
-- ); 

-- -- 模板点位表
-- CREATE TABLE IF NOT EXISTS
-- template_point(
--     id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
--     template_id INTEGER NOT NULL, --模版Id
--     protocol_type INTEGER NOT NULL, --协议类型
--     dev_type INTEGER NOT NULL, --设备类型
--     group_type INTEGER NOT NULL, --分组类型
--     code INTEGER NOT NULL, --点位编号
--     name TEXT(100) NOT NULL, --点位名称
--     data_type INTEGER NOT NULL, --数据类型
--     attribute INTEGER NOT NULL, --点位属性
--     func INTEGER NOT NULL, --点位功能
--     reg_addr INTEGER NOT NULL, --点位地址
--     bit_pos INTEGER NOT NULL, --字节开始位
--     bit_num INTEGER NOT NULL, --字节数
--     endian INTEGER NOT NULL, --字节顺序
--     precision REAL NOT NULL, --精度
--     ratio REAL NOT NULL, --系数
--     offset REAL NOT NULL, --偏移量
--     is_persisted TINYINT NOT NULL, --是否持久化
--     storage_interval INTEGER NOT NULL, --存储周期
--     mutate_bound REAL, --突变范围
--     default_value REAL, --默认值
--     min_value REAL, --最小值
--     max_value REAL,  --最大值
--     unit TEXT(20), --计量单位
--     is_show TINYINT NOT NULL --是否展示
-- );

-- -- 北向上报下发设备配置模板表
-- CREATE TABLE IF NOT EXISTS
-- template_up_dis_dev(
--     id INTEGER PRIMARY KEY AUTOINCREMENT, -- 主键标识
--     template_id INTEGER NOT NULL, --模版Id
--     up_dis_config_type INTEGER, --上报下发类型    
--     dev_type INTEGER NOT NULL, --设备类型
--     UNIQUE(up_dis_config_type, dev_type) --上报下发类型和设备类型唯一唯一
-- );

-- -- 北向上报下发点位信息模板表
-- CREATE TABLE IF NOT EXISTS
-- template_up_dis_point(
--     id INTEGER PRIMARY KEY AUTOINCREMENT, -- 主键标识
--     template_up_dis_id INTEGER NOT NULL, -- 外键，北向上报下发设备配置模板表
--     cloud_code TEXT(50) NOT NULL, -- 第三方云平台编号
--     cloud_name TEXT(100) NOT NULL, -- 第三方云平台名称
--     point_code INTEGER NOT NULL, -- 点位编号
--     point_name TEXT(100) NOT NULL, -- 点位名称
--     remark TEXT(1200) -- 备注信息
-- );

-- -- 模板协议表
-- CREATE TABLE IF NOT EXISTS
-- template_protocol(
--     id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
--     template_id INTEGER NOT NULL, --模版Id
--     template_type TINYINT NOT NULL, --模板类别（0设备配置, 1北向配置）
--     type INTEGER NOT NULL, --协议类型
--     name TEXT(100) NOT NULL, --协议名称
--     content TEXT(3000) NOT NULL, --协议内容
--     remark TEXT(1200) --备注
-- ); 

-- 设备表
CREATE TABLE IF NOT EXISTS
device(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    sn TEXT(50) NOT NULL, --设备sn
    type INTEGER NOT NULL, --设备类型
    code INTEGER NOT NULL, --设备编号
    name TEXT(100) NOT NULL, --设备名称
    protocol_type INTEGER NOT NULL, --协议类型, 参考： 主站通讯协议类型枚举
    template_id INTEGER NOT NULL, --模版Id
    sort_order INTEGER, --序号
    remark TEXT(1200), --备注
    status TINYINT, --状态
    UNIQUE(name), -- 设备名称唯一
    UNIQUE(type, code) -- 设备类型和设备编号唯一
);

-- 设备/北向配置协议表
CREATE TABLE IF NOT EXISTS 
protocol(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    related_id INTEGER NOT NULL, --关联Id
    template_type TINYINT NOT NULL, --模板类别（0设备配置, 1北向配置）
    type INTEGER NOT NULL, --协议类型
    name TEXT(100) NOT NULL, --协议名称
    content TEXT(1200) NOT NULL, --协议内容
    remark TEXT(1200), --备注
    UNIQUE(name) -- 协议名称唯一
);

-- 点位表
CREATE TABLE IF NOT EXISTS 
point(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    template_id INTEGER NOT NULL, --模版Id
    protocol_type INTEGER NOT NULL, --协议类型
    dev_type INTEGER NOT NULL, --设备类型
    group_type INTEGER NOT NULL, --分组类型
    code INTEGER NOT NULL, --点位编号
    name TEXT(100) NOT NULL, --点位名称
    data_type INTEGER NOT NULL, --数据类型
    attribute INTEGER NOT NULL, --点位属性
    func INTEGER NOT NULL, --点位功能
    reg_addr INTEGER NOT NULL, --点位地址
    bit_pos INTEGER NOT NULL, --字节开始位
    bit_num INTEGER NOT NULL, --字节数
    endian INTEGER NOT NULL, --字节顺序
    precision REAL NOT NULL, --精度
    ratio REAL NOT NULL, --系数
    offset REAL NOT NULL, --偏移量
    is_persisted TINYINT NOT NULL, --是否持久化
    storage_interval INTEGER NOT NULL, --存储周期
    mutate_bound REAL, --突变范围
    default_value REAL, --默认值
    min_value REAL, --最小值
    max_value REAL, --最大值
    unit TEXT(20), --计量单位
    is_show TINYINT NOT NULL --是否展示
);

-- 数据监视器
CREATE TABLE IF NOT EXISTS data_monitor (
    id INTEGER PRIMARY KEY AUTOINCREMENT,  -- 主键标识
    dev_type INTEGER NOT NULL,             -- 设备类型
    dev_id INTEGER NOT NULL,               -- 设备Id
    point_id INTEGER NOT NULL,             -- 点位Id
    interval INTEGER NOT NULL,             -- 处理周期
    level TINYINT NOT NULL,                -- 处理等级
    handle_method TINYINT NOT NULL         -- 处理方式
);

-- 拓扑结构表
CREATE TABLE IF NOT EXISTS
topology(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    parent_id INTEGER, --父Id 
    menu_tree TINYINT NOT NULL, --类型:0-虚拟节点，1-设备节点
    name TEXT(100), --层级名称
    dev_id INTEGER, --设备Id
    sort_order INTEGER NOT NULL, --序号
    remark TEXT(1200) --备注
);

-- EMS设备配置表
CREATE TABLE IF NOT EXISTS
ems_device_config(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    type INTEGER NOT NULL, --配置类型
    name TEXT(100) NOT NULL, --配置名称
    content TEXT(1200) NOT NULL, --配置内容
    UNIQUE(name) -- 配置名称唯一
);

-- 北向配置表
CREATE TABLE IF NOT EXISTS
north_config(
    id INTEGER PRIMARY KEY AUTOINCREMENT, -- 主键标识
    name TEXT(100) NOT NULL, --配置名称
    protocol_type INTEGER NOT NULL, --协议类型，参考： 从站通讯协议类型枚举
    config_type TINYINT NOT NULL, --北向配置类型：0默认全部，1配置上传
    template_id INTEGER, --模版Id
    remark TEXT(1200), -- 备注信息
    UNIQUE(name) -- 配置名称唯一
);

-- 北向上报下发配置表
CREATE TABLE IF NOT EXISTS
up_dis_dev(
    id INTEGER PRIMARY KEY AUTOINCREMENT, -- 主键标识
    north_config_id INTEGER NOT NULL, -- 外键，关联北向配置表
    up_dis_config_type INTEGER, --上报下发类型    
    dev_type INTEGER NOT NULL, --设备类型
    UNIQUE(dev_type) --设备类型和设备编号唯一
);

-- 北向上报下发点位信息表
CREATE TABLE IF NOT EXISTS
up_dis_point(
    id INTEGER PRIMARY KEY AUTOINCREMENT, -- 主键标识
    up_dis_id INTEGER NOT NULL, -- 外键，上报设备信息表
    cloud_code TEXT(50) NOT NULL, -- 云平台编号
    cloud_name TEXT(100) NOT NULL, -- 云平台名称
    point_code INTEGER NOT NULL, -- 点位编号
    point_name TEXT(100) NOT NULL, -- 点位名称
    remark TEXT(1200), -- 备注信息
    UNIQUE(cloud_code) -- 云平台编号唯一
);

-- 削峰填谷日期配置表
CREATE TABLE IF NOT EXISTS
pv_date_config(
    id INTEGER PRIMARY KEY AUTOINCREMENT, -- 主键标识
    pv_time_config_id INTEGER NOT NULL, -- 外键，关联区间时间配置表
    start_date TEXT(20) NOT NULL, -- 开始日期
    end_date TEXT(20) NOT NULL, -- 结束日期
    remark TEXT(1200) -- 备注信息
);

-- 削峰填谷时间配置表
CREATE TABLE IF NOT EXISTS
pv_time_config(
    id INTEGER PRIMARY KEY AUTOINCREMENT, -- 主键标识
    name TEXT(100) NOT NULL, -- 配置名称
    remark TEXT(1200), -- 备注信息
    UNIQUE(name) -- 配置名称唯一
);

--  削峰填谷业务时间执行表
CREATE TABLE IF NOT EXISTS
pv_time_exe(
    id INTEGER PRIMARY KEY AUTOINCREMENT, -- 主键标识
    time_section_config_id INTEGER NOT NULL, -- 外键，关联区间时间配置表
    start_time INTEGER NOT NULL, -- 当天云平台编号开始秒数
    end_time INTEGER NOT NULL, -- 当天云平台编号结束秒数
    power REAL NOT NULL, --执行功率
    remark TEXT(1200) -- 备注信息
);

-- 保护算法表
CREATE TABLE IF NOT EXISTS
protect_algorithm(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    transformer_capacity REAL NOT NULL, --变压器总容量(KW)
    ol_warn_limit_value REAL NOT NULL, --过载预警限流值(KW)
    ol_shutdown_value REAL NOT NULL, --过载关机值(KW)
    max_power REAL NOT NULL, --充放电功率上限
    demand_switch TINYINT, --防需量开关
    target_demand REAL, --目标需量(KW)
    de_warn_limit_value REAL, --需量预警限流值(KW)
    de_shutdown_value REAL, --需量关机值(KW)
    backflow_switch TINYINT, --防逆流开关
    bf_warn_limit_value REAL, --防逆流预警限流值(KW)
    bf_shutdown_value REAL, --防逆流关机值(KW)
    soc_forbid_chg TINYINT, --禁充SOC [55%-100%]
    soc_forbid_dischg TINYINT, --禁放SOC [0%-45%]
    remark TEXT(1200) --备注
);

-- 调试模式算法表
CREATE TABLE IF NOT EXISTS
debug_algorithm(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    active_power INTEGER NOT NULL, --目标有功功率(KW)
    reactive_power TINYINT NOT NULL, --目标无功功率(KW)
    pcs_switch TINYINT, --充放电开关 1 开 0 关
    protect_switch TINYINT, --保护算法开关
    remark TEXT(1200) --备注
);

-- 功率分配表
CREATE TABLE IF NOT EXISTS
allocate_power(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    pcs_num INTEGER NOT NULL, --pcs数量 
    chg_direction TINYINT NOT NULL, --充放电方向 1(正放 负充) -1(正充 负放)
    allocate_mode TINYINT, --下发功率模式：总功率（0），分相功率（1）
    remark TEXT(1200) --备注
);

-- 工作模式设置表
CREATE TABLE IF NOT EXISTS
work_mode_set(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    work_mode TINYINT NOT NULL, --工作模式:0关机模式、1待机模式、2调试模式、3削峰填谷模式、4需求响应模式、5负载跟踪模式
    remark TEXT(1200) --备注
);

-- DIDO逻辑设置表
CREATE TABLE IF NOT EXISTS
di_do_set(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    seq TINYINT NOT NULL, --DIDO序列号
    type TINYINT NOT NULL, --DIDO类型: 0DI，1DO
    cabinet_code TINYINT NOT NULL, --柜子编号
    strategy_seq TINYINT NOT NULL, --策略序号
    remark TEXT(1200) --备注
);

-- 温控算法表
CREATE TABLE IF NOT EXISTS
temp_ctrl_algorithm(
    id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
    mod INTEGER NOT NULL, --温度控制模式, 参考：温控模式
    work_cool_trig_temp REAL NOT NULL, --制冷触发温度(0-60℃)
    work_cool_hys REAL NOT NULL, --制冷回差(1-10℃)
    work_heat_trig_temp REAL NOT NULL, --制热触发温度(0-30℃)
    work_heat_hys REAL NOT NULL, --制热回差(1-10℃)
    standby_cool_trig_temp REAL, --制冷触发温度(0-60℃)
    standby_cool_hys REAL, --制冷回差(1-10℃)
    standby_heat_trig_temp REAL, --制热触发温度(0-30℃)
    standby_heat_hys REAL, --制热回差(1-10℃)
    standby_max_limit REAL, --开启自循环高温(0-60℃)
    standby_min_limit REAL, --开启自循环低温(0-30℃)
    standby_maxback_limit REAL, --自循环高温回温(1-10℃)
    standby_minback_limit REAL, --自循环低温回温(1-10℃)
	temp_cmd_normal INTEGER NOT NULL, --循环控制字
	temp_cmd_heat INTEGER NOT NULL, --制热控制字
	temp_cmd_cold INTEGER NOT NULL, --制冷控制字
    temp_cmd_open INTEGER NOT NULL, --开控制字
	temp_cmd_close INTEGER NOT NULL, --关控制字
    remark TEXT(1200) --备注
);

-- 高级设置
CREATE TABLE IF NOT EXISTS
advanced_setting (
  id INTEGER PRIMARY KEY AUTOINCREMENT, --主键标识
  key TEXT NOT NULL, --设置项
  value TEXT NOT NULL --设置值
);

-- 升级记录
CREATE TABLE IF NOT EXISTS
upgrade_record(
   id INTEGER PRIMARY KEY AUTOINCREMENT, -- 主键标识
   version TEXT NOT NULL, -- 升级版本号
   path TEXT, -- 升级包位置
   err_msg TEXT, -- 升级错误信息
   status TINYINT NOT NULL DEFAULT 1, -- 升级状态 (0：待升级, 1: 升级成功, 2:升级失败)
   start_time TIMESTAMP, -- 升级开始时间
   end_time TIMESTAMP -- 升级结束时间
);

-- -------------------------------------------------------------初始化数据------------------------------------------------------------------


-- ----------------------------
-- Records of allocate_power
-- ----------------------------
INSERT INTO "allocate_power" VALUES (1, 1, 1, 0, '分配算法配置');

-- ----------------------------
-- Records of device
-- ----------------------------
INSERT INTO "device" VALUES (0, 'ems001', 0, 1, '1#EMS', 0, 0, 0, '0', 0);
INSERT INTO "device" VALUES (1, 'sn_test_1', 3, 1, '1#PCS', 1, 1, 1, '1', 0);
INSERT INTO "device" VALUES (2, 'bms001', 4, 1, '2#BMS', 0, 0, 2, '2', 0);
INSERT INTO "device" VALUES (3, 'pcc001', 1, 1, '1#PCC电表', 0, 4, 3, '3', 0);
INSERT INTO "device" VALUES (4, 'bcu001', 5, 1, '1#bcu', 1, 3, 4, '4', 0);
INSERT INTO "device" VALUES (5, 'meterbsm001', 2, 1, '1#储能电表', 0, 5, 5, '5', 0);
INSERT INTO "device" VALUES (6, 'wsd001', 6, 1, '1#温湿度传感器', 2, 2, 6, '6', 0);
INSERT INTO "device" VALUES (7, 'shuijin001', 10, 1, '1#水浸烟感', 2, 6, 7, '7', 0);
INSERT INTO "device" VALUES (8, 'yangan001', 10, 2, '2#水浸烟感', 2, 7, 8, '8', 0);
INSERT INTO "device" VALUES (9, 'kongtiao001', 9, 1, '1#空调水冷机', 2, 8, 8, '8', 0);

-- ----------------------------
-- Records of ems_device_config
-- ----------------------------
INSERT INTO "ems_device_config" VALUES (1, 0, '串口1', '{"uartId":1,"data":3,"stop":0,"baud":0,"parity":0,"address":"/dev/ttyS9"}');
INSERT INTO "ems_device_config" VALUES (2, 0, '串口2', '{"uartId":2,"data":3,"stop":0,"baud":0,"parity":0,"address":"/dev/ttyS5"}');
INSERT INTO "ems_device_config" VALUES (3, 0, '串口3', '{"uartId":3,"data":3,"stop":0,"baud":5,"parity":0,"address":"/dev/ttyS3"}');
INSERT INTO "ems_device_config" VALUES (4, 0, '串口4', '{"uartId":4,"data":3,"stop":0,"baud":5,"parity":0,"address":"/dev/ttyS4"}');
INSERT INTO "ems_device_config" VALUES (5, 0, '串口5', '{"uartId":5,"data":3,"stop":0,"baud":5,"parity":0,"address":"/dev/ttyS7"}');
INSERT INTO "ems_device_config" VALUES (6, 0, '串口6', '{"uartId":6,"data":3,"stop":0,"baud":5,"parity":0,"address":"/dev/ttyS8"}');
INSERT INTO "ems_device_config" VALUES (7, 0, '串口7', '{"uartId":7,"data":3,"stop":0,"baud":5,"parity":0,"address":"/dev/ttyS0"}');
INSERT INTO "ems_device_config" VALUES (8, 0, '串口8', '{"uartId":8,"data":3,"stop":0,"baud":5,"parity":0,"address":"/dev/ttyXRUSB1"}');
INSERT INTO "ems_device_config" VALUES (9, 0, '串口9', '{"uartId":9,"data":3,"stop":0,"baud":5,"parity":0,"address":"/dev/ttyXRUSB2"}');
INSERT INTO "ems_device_config" VALUES (10, 1, '网口1', '{"netId":0,"isDHCP":0, "ip":"192.168.0.123", "mask":"255.255.255.0", "gateway":"", "address":""}');
INSERT INTO "ems_device_config" VALUES (11, 1, '网口2', '{"netId":1,"isDHCP":0, "ip":"172.18.39.44", "mask":"255.255.255.0", "gateway":"", "address":""}');
INSERT INTO "ems_device_config" VALUES (12, 1, '网口3', '{"netId":2,"isDHCP":0, "ip":"192.168.2.123", "mask":"255.255.255.0", "gateway":"", "address":""}');
INSERT INTO "ems_device_config" VALUES (13, 1, '网口4', '{"netId":3,"isDHCP":0, "ip":"192.168.3.123", "mask":"255.255.255.0", "gateway":"", "address":""}');
INSERT INTO "ems_device_config" VALUES (14, 4, 'Wifi配置', '{"wifiName": "company", "wifiPassword": "12345678", "enable": 0}');
INSERT INTO "ems_device_config" VALUES (15, 5, '4G配置', '{"cmdContent": "quectel-CM &", "enable": 0}');
INSERT INTO "ems_device_config" VALUES (16, 6, '对时配置', '{"isNtp": 0, "port": 0, "address": "", "manualTime": "2024/10/12 18:20:20"}');

-- ----------------------------
-- Records of north_config
-- ----------------------------
INSERT INTO "north_config" VALUES (1, 'MQTT转发', 10, 1, '1', 0);

-- ----------------------------
-- Records of point
-- ----------------------------
INSERT INTO "point" VALUES (1, 1, 1, 3, 0, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (2, 1, 1, 3, 0, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (3, 1, 1, 3, 0, 2, '总故障', 0, 0, 2, 84, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (4, 1, 1, 3, 0, 3, '总告警', 0, 0, 2, 85, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (5, 1, 1, 3, 0, 4, 'A相硬件过流', 0, 0, 4, 272, 1, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (6, 1, 1, 3, 0, 5, 'B相硬件过流', 0, 0, 4, 272, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (7, 1, 1, 3, 0, 6, 'C相硬件过流', 0, 0, 4, 272, 3, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (8, 1, 1, 3, 0, 7, 'N相硬件过流', 0, 0, 4, 272, 4, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (9, 1, 1, 3, 0, 8, '单元直压', 0, 0, 4, 272, 7, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (10, 1, 1, 3, 0, 9, '开关电源欠压', 0, 0, 4, 272, 10, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (11, 1, 1, 3, 0, 10, 'A相IGBT故障', 0, 0, 4, 272, 11, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (12, 1, 1, 3, 0, 11, 'B相IGBT故障', 0, 0, 4, 272, 12, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (13, 1, 1, 3, 0, 12, 'C相IGBT故障', 0, 0, 4, 272, 13, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (14, 1, 1, 3, 0, 13, 'N相IGBT故障', 0, 0, 4, 272, 14, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (15, 1, 1, 3, 0, 14, '过温故障', 0, 0, 4, 272, 15, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (16, 1, 1, 3, 0, 15, 'A相输出过流', 0, 0, 4, 273, 1, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (17, 1, 1, 3, 0, 16, 'A相输出速断', 0, 0, 4, 273, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (18, 1, 1, 3, 0, 17, 'B相输出过流', 0, 0, 4, 273, 3, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (19, 1, 1, 3, 0, 18, 'B相输出速断', 0, 0, 4, 273, 4, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (20, 1, 1, 3, 0, 19, 'C相输出过流', 0, 0, 4, 273, 5, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (21, 1, 1, 3, 0, 20, 'C相输出速断', 0, 0, 4, 273, 6, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (22, 1, 1, 3, 0, 21, 'N相输出过流', 0, 0, 4, 273, 7, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (23, 1, 1, 3, 0, 22, 'N相输出速断', 0, 0, 4, 273, 8, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (24, 1, 1, 3, 0, 23, '交流过压', 0, 0, 4, 273, 9, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (25, 1, 1, 3, 0, 24, '交流欠压', 0, 0, 4, 273, 10, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (26, 1, 1, 3, 0, 25, '交流过频', 0, 0, 4, 273, 11, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (27, 1, 1, 3, 0, 26, '交流欠频', 0, 0, 4, 273, 12, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (28, 1, 1, 3, 0, 27, '电压THDU超限', 0, 0, 4, 273, 13, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (29, 1, 1, 3, 0, 28, '系统缺相', 0, 0, 4, 273, 14, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (30, 1, 1, 3, 0, 29, '系统相序错误', 0, 0, 4, 273, 15, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (31, 1, 1, 3, 0, 30, '直流极性反接', 0, 0, 4, 273, 16, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (32, 1, 1, 3, 0, 31, '直流母线欠压', 0, 0, 4, 274, 1, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (33, 1, 1, 3, 0, 32, '直流母线过压', 0, 0, 4, 274, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (34, 1, 1, 3, 0, 33, '系统过频', 0, 0, 4, 274, 3, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (35, 1, 1, 3, 0, 34, '系统欠频', 0, 0, 4, 274, 4, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (36, 1, 1, 3, 0, 35, '直流充电过流', 0, 0, 4, 274, 5, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (37, 1, 1, 3, 0, 36, '直流放电过流', 0, 0, 4, 274, 6, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (38, 1, 1, 3, 0, 37, '孤岛保护', 0, 0, 4, 274, 7, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (39, 1, 1, 3, 0, 38, '交流主接合闸故障', 0, 0, 4, 274, 9, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (40, 1, 1, 3, 0, 39, '交流主接分闸故障', 0, 0, 4, 274, 10, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (41, 1, 1, 3, 0, 40, '交流软启合闸故障', 0, 0, 4, 274, 11, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (42, 1, 1, 3, 0, 41, '交流软启分闸故障', 0, 0, 4, 274, 12, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (43, 1, 1, 3, 0, 42, '直流主接合闸故障', 0, 0, 4, 274, 13, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (44, 1, 1, 3, 0, 43, '直流主接分闸故障', 0, 0, 4, 274, 14, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (45, 1, 1, 3, 0, 44, '直流软启合闸故障', 0, 0, 4, 274, 15, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (46, 1, 1, 3, 0, 45, '直流软启分闸故障', 0, 0, 4, 274, 16, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (47, 1, 1, 3, 0, 46, '铁电参数存储错误', 0, 0, 4, 275, 1, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (48, 1, 1, 3, 0, 47, '直流软起失败', 0, 0, 4, 275, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (49, 1, 1, 3, 0, 48, '起机条件不满足', 0, 0, 4, 275, 5, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (50, 1, 1, 3, 0, 49, '运行中开关故障', 0, 0, 4, 275, 6, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (51, 1, 1, 3, 0, 50, '逆变器启动超时', 0, 0, 4, 275, 7, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (52, 1, 1, 3, 0, 51, '参数下发设置错误', 0, 0, 4, 275, 8, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (53, 1, 1, 3, 0, 52, 'BMS通讯故障', 0, 0, 4, 275, 9, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (54, 1, 1, 3, 0, 53, 'BMS温度异常', 0, 0, 4, 275, 10, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (55, 1, 1, 3, 0, 54, 'BMS跳机', 0, 0, 4, 275, 11, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (56, 1, 1, 3, 0, 55, 'BMS电池告警', 0, 0, 4, 275, 12, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (57, 1, 1, 3, 0, 56, 'DCDC通讯故障', 0, 0, 4, 275, 13, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (58, 1, 1, 3, 0, 57, 'EMS通讯故障', 0, 0, 4, 275, 14, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (59, 1, 1, 3, 0, 58, '急停故障', 0, 0, 4, 275, 15, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (60, 1, 1, 3, 0, 59, '电池软件欠压', 0, 0, 4, 256, 2, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (61, 1, 1, 3, 0, 60, '电池软件过压', 0, 0, 4, 257, 3, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (62, 1, 1, 3, 0, 61, '母线不平衡异常', 0, 0, 4, 258, 4, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (63, 1, 1, 3, 0, 62, '母线半直压过压', 0, 0, 4, 259, 5, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (64, 1, 1, 3, 1, 150, '停机状态', 0, 0, 2, 81, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (65, 1, 1, 3, 1, 151, '待机状态', 0, 0, 2, 82, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (66, 1, 1, 3, 1, 152, '运行状态', 0, 0, 2, 83, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (67, 1, 1, 3, 1, 153, '并网状态', 0, 0, 2, 88, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (68, 1, 1, 3, 1, 154, '离网状态', 0, 0, 2, 89, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (69, 1, 1, 3, 1, 155, 'BMS干节点信号', 0, 0, 2, 94, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (70, 1, 1, 3, 1, 156, '本地/远程', 0, 0, 1, 7, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (71, 1, 1, 3, 1, 157, 'A相电压', 2, 0, 4, 201, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "point" VALUES (72, 1, 1, 3, 1, 158, 'B相电压', 2, 0, 4, 202, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "point" VALUES (73, 1, 1, 3, 1, 159, 'C相电压', 2, 0, 4, 203, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "point" VALUES (74, 1, 1, 3, 1, 160, 'A相电流', 3, 0, 4, 204, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (75, 1, 1, 3, 1, 161, 'B相电流', 3, 0, 4, 205, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (76, 1, 1, 3, 1, 162, 'C相电流', 3, 0, 4, 206, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (77, 1, 1, 3, 1, 163, '电网频率', 2, 0, 4, 207, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'Hz', 1);
INSERT INTO "point" VALUES (78, 1, 1, 3, 1, 164, 'A相有功功率', 3, 0, 4, 208, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (79, 1, 1, 3, 1, 165, 'B相有功功率', 3, 0, 4, 209, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (80, 1, 1, 3, 1, 166, 'C相有功功率', 3, 0, 4, 210, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (81, 1, 1, 3, 1, 167, '总有功功率', 3, 0, 4, 211, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (82, 1, 1, 3, 1, 168, 'A相无功功率', 3, 0, 4, 212, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "point" VALUES (83, 1, 1, 3, 1, 169, 'B相无功功率', 3, 0, 4, 213, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "point" VALUES (84, 1, 1, 3, 1, 170, 'C相无功功率', 3, 0, 4, 214, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "point" VALUES (85, 1, 1, 3, 1, 171, '总无功功率', 3, 0, 4, 215, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "point" VALUES (86, 1, 1, 3, 1, 172, 'A相视在功率', 2, 0, 4, 216, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVA', 1);
INSERT INTO "point" VALUES (87, 1, 1, 3, 1, 173, 'B相视在功率', 2, 0, 4, 217, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVA', 1);
INSERT INTO "point" VALUES (88, 1, 1, 3, 1, 174, 'C相视在功率', 2, 0, 4, 218, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVA', 1);
INSERT INTO "point" VALUES (89, 1, 1, 3, 1, 175, '总视在功率', 2, 0, 4, 219, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVA', 1);
INSERT INTO "point" VALUES (90, 1, 1, 3, 1, 176, 'A相功率因数', 2, 0, 4, 220, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (91, 1, 1, 3, 1, 177, 'B相功率因数', 2, 0, 4, 221, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (92, 1, 1, 3, 1, 178, 'C相功率因数', 2, 0, 4, 222, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (93, 1, 1, 3, 1, 179, '总功率因数', 2, 0, 4, 223, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (94, 1, 1, 3, 1, 180, 'PCS输入功率', 3, 0, 4, 224, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (95, 1, 1, 3, 1, 181, 'PCS输入电压', 3, 0, 4, 225, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "point" VALUES (96, 1, 1, 3, 1, 182, 'PCS输入电流', 3, 0, 4, 226, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (97, 1, 1, 3, 1, 183, '交流累计充电量', 4, 0, 4, 230, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kWh', 1);
INSERT INTO "point" VALUES (98, 1, 1, 3, 1, 184, '交流累计放电量', 4, 0, 4, 232, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kWh', 1);
INSERT INTO "point" VALUES (99, 1, 1, 3, 1, 185, '直流累计充电量', 4, 0, 4, 234, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kWh', 1);
INSERT INTO "point" VALUES (100, 1, 1, 3, 1, 186, '直流累计放电量', 4, 0, 4, 236, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kWh', 1);
INSERT INTO "point" VALUES (101, 1, 1, 3, 1, 187, '系统时钟-秒', 2, 0, 4, 239, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (102, 1, 1, 3, 1, 188, '系统时钟-分', 2, 0, 4, 240, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (103, 1, 1, 3, 1, 189, '系统时钟-时', 2, 0, 4, 241, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (104, 1, 1, 3, 1, 190, '系统时钟-日', 2, 0, 4, 242, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (105, 1, 1, 3, 1, 191, '系统时钟-月', 2, 0, 4, 243, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (106, 1, 1, 3, 1, 192, '系统时钟-年', 2, 0, 4, 244, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (107, 1, 1, 3, 1, 193, 'PCS故障字1', 2, 0, 4, 272, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (108, 1, 1, 3, 1, 194, 'PCS故障字2', 2, 0, 4, 273, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (109, 1, 1, 3, 1, 195, 'PCS故障字3', 2, 0, 4, 274, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (110, 1, 1, 3, 1, 196, 'PCS故障字4', 2, 0, 4, 275, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (111, 1, 1, 3, 1, 197, 'PCS故障字5', 2, 0, 4, 256, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (112, 1, 1, 3, 1, 198, 'PCS故障字6', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (113, 1, 1, 3, 1, 199, '最大允许充电电流', 2, 0, 4, 281, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (114, 1, 1, 3, 1, 200, '最大允许放电电流', 2, 0, 4, 282, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (115, 1, 1, 3, 1, 201, '最大允许充电功率', 2, 0, 4, 294, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (116, 1, 1, 3, 1, 202, '最大允许放电功率', 2, 0, 4, 295, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (117, 1, 1, 3, 1, 203, 'IGBT温度', 3, 0, 4, 227, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, '℃', 1);
INSERT INTO "point" VALUES (118, 1, 1, 3, 2, 250, '远程本地设置', 2, 1, 5, 7, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (119, 1, 1, 3, 2, 251, '运行模式设置', 2, 1, 6, 301, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (120, 1, 1, 3, 2, 252, '并网离网设置', 2, 1, 6, 306, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (121, 1, 1, 3, 2, 253, '设备开机', 0, 0, 5, 2, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (122, 1, 1, 3, 2, 254, '设备停机', 0, 0, 5, 3, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (123, 1, 1, 3, 2, 255, '设备开关机指令', 2, 1, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (124, 1, 1, 3, 2, 256, '远程急停', 0, 0, 1, 4, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (125, 1, 1, 3, 2, 257, '功率因数设置', 2, 1, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (126, 1, 1, 3, 2, 258, '恒压充电电压设置值', 2, 1, 6, 302, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "point" VALUES (127, 1, 1, 3, 2, 259, '恒流充电电流设置值', 3, 1, 6, 303, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (128, 1, 1, 3, 2, 260, '恒功率有功功率设置', 3, 1, 6, 304, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (129, 1, 1, 3, 2, 261, '恒功率无功功率设置', 3, 1, 6, 305, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "point" VALUES (130, 1, 1, 3, 2, 262, '恒功率A相有功功率设置', 3, 1, 6, 309, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (131, 1, 1, 3, 2, 263, '恒功率B相有功功率设置', 3, 1, 6, 310, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (132, 1, 1, 3, 2, 264, '恒功率C相有功功率设置', 3, 1, 6, 311, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (133, 2, 2, 6, 0, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (134, 2, 2, 6, 0, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, NULL, 1);
INSERT INTO "point" VALUES (135, 2, 2, 6, 1, 2, '温度', 3, 0, 4, 0, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "point" VALUES (136, 2, 2, 6, 1, 3, '湿度', 2, 0, 4, 1, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, 0.0, 1.0, '%RH', 1);
INSERT INTO "point" VALUES (137, 3, 2, 5, 0, 0, '设备启用', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (138, 3, 2, 5, 0, 1, '设备在线', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (139, 3, 2, 5, 0, 2, '与电池簇通信告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (140, 3, 2, 5, 0, 3, '主从通讯告警', 2, 0, 2, 54, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (141, 3, 2, 5, 0, 4, '总告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (142, 3, 2, 5, 0, 5, '总故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (143, 3, 2, 5, 0, 6, '单体过压告警一级', 0, 0, 2, 22, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (144, 3, 2, 5, 0, 7, '单体欠压告警一级', 0, 0, 2, 25, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (145, 3, 2, 5, 0, 8, '单体过温告警一级', 0, 0, 2, 16, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (146, 3, 2, 5, 0, 9, '单体低温告警一级', 0, 0, 2, 19, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (147, 3, 2, 5, 0, 10, '单体压差告警一级', 0, 0, 2, 28, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (148, 3, 2, 5, 0, 11, '充电过流告警一级', 0, 0, 2, 10, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (149, 3, 2, 5, 0, 12, '放电过流告警一级', 0, 0, 2, 7, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (150, 3, 2, 5, 0, 13, 'SOC过低告警一级', 0, 0, 2, 34, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (151, 3, 2, 5, 0, 14, 'SOC差异过大告警一级', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (152, 3, 2, 5, 0, 15, '绝缘过低告警一级', 0, 0, 2, 13, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (153, 3, 2, 5, 0, 16, '单体过压告警二级', 0, 0, 2, 23, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (154, 3, 2, 5, 0, 17, '单体欠压告警二级', 0, 0, 2, 26, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (155, 3, 2, 5, 0, 18, '单体过温告警二级', 0, 0, 2, 17, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (156, 3, 2, 5, 0, 19, '单体低温告警二级', 0, 0, 2, 20, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (157, 3, 2, 5, 0, 20, '单体压差告警二级', 0, 0, 2, 29, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (158, 3, 2, 5, 0, 21, '充电过流告警二级', 0, 0, 2, 11, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (159, 3, 2, 5, 0, 22, '放电过流告警二级', 0, 0, 2, 8, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (160, 3, 2, 5, 0, 23, 'SOC过低告警二级', 0, 0, 2, 35, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (161, 3, 2, 5, 0, 24, 'SOC差异过大告警二级', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (162, 3, 2, 5, 0, 25, '绝缘过低告警二级', 0, 0, 2, 14, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (163, 3, 2, 5, 0, 26, '单体过压告警三级', 0, 0, 2, 24, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (164, 3, 2, 5, 0, 27, '单体欠压告警三级', 0, 0, 2, 27, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (165, 3, 2, 5, 0, 28, '单体过温告警三级', 0, 0, 2, 18, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (166, 3, 2, 5, 0, 29, '单体低温告警三级', 0, 0, 2, 21, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (167, 3, 2, 5, 0, 30, '单体压差告警三级', 0, 0, 2, 30, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (168, 3, 2, 5, 0, 31, '充电过流告警三级', 0, 0, 2, 12, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (169, 3, 2, 5, 0, 32, '放电过流告警三级', 0, 0, 2, 9, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (170, 3, 2, 5, 0, 33, 'SOC过低告警三级', 0, 0, 2, 36, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (171, 3, 2, 5, 0, 34, 'SOC差异过大告警三级', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (172, 3, 2, 5, 0, 35, '绝缘过低告警三级', 0, 0, 2, 15, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (173, 3, 2, 5, 0, 36, '电芯温度极限告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (174, 3, 2, 5, 0, 37, '电芯电压极限告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (175, 3, 2, 5, 0, 38, '簇间环流1级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (176, 3, 2, 5, 0, 39, '簇间环流2级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (177, 3, 2, 5, 0, 40, '簇间环流3级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (178, 3, 2, 5, 0, 41, '簇间电流差1级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (179, 3, 2, 5, 0, 42, '簇间电流差2级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (180, 3, 2, 5, 0, 43, '簇间电流差3级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (181, 3, 2, 5, 0, 44, '组端过压1级告警', 0, 0, 2, 1, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (182, 3, 2, 5, 0, 45, '组端过压2级告警', 0, 0, 2, 2, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (183, 3, 2, 5, 0, 46, '组端过压3级告警', 0, 0, 2, 3, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (184, 3, 2, 5, 0, 47, '组端欠压1级告警', 0, 0, 2, 4, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (185, 3, 2, 5, 0, 48, '组端欠压2级告警', 0, 0, 2, 5, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (186, 3, 2, 5, 0, 49, '组端欠压3级告警', 0, 0, 2, 6, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (187, 3, 2, 5, 0, 50, '极柱过温1级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (188, 3, 2, 5, 0, 51, '极柱过温2级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (189, 3, 2, 5, 0, 52, '极柱过温3级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (190, 3, 2, 5, 0, 53, 'AFE温感排线异常', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (191, 3, 2, 5, 0, 54, 'AFE电压排线异常', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (192, 3, 2, 5, 0, 55, 'AFE故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (193, 3, 2, 5, 0, 56, '高压异常', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (194, 3, 2, 5, 0, 57, '预充告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (195, 3, 2, 5, 0, 58, '开路故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (196, 3, 2, 5, 0, 59, '继电器粘连告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (197, 3, 2, 5, 0, 60, '电池极限故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (198, 3, 2, 5, 0, 61, '熔丝故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (199, 3, 2, 5, 0, 62, '断路器故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (200, 3, 2, 5, 0, 63, '空调故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (201, 3, 2, 5, 0, 64, '消防喷洒', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (202, 3, 2, 5, 0, 65, '消防设备故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (203, 3, 2, 5, 0, 66, '消防火警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (204, 3, 2, 5, 0, 67, '急停故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (205, 3, 2, 5, 0, 68, 'MSD故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (206, 3, 2, 5, 0, 69, '控制指令超时', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (207, 3, 2, 5, 1, 101, 'SOC', 2, 0, 4, 3, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, '%', 1);
INSERT INTO "point" VALUES (208, 3, 2, 5, 1, 102, 'SOH', 2, 0, 4, 4, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, '%', 1);
INSERT INTO "point" VALUES (209, 3, 2, 5, 1, 103, '总电压', 2, 0, 4, 1, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "point" VALUES (210, 3, 2, 5, 1, 104, '总电流', 3, 0, 4, 2, 0, 0, 0, 0.1, 0.1, -1600.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (211, 3, 2, 5, 1, 105, '系统SOH', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, '%', 1);
INSERT INTO "point" VALUES (212, 3, 2, 5, 1, 106, '最大单体电压', 2, 0, 4, 20, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'mv', 1);
INSERT INTO "point" VALUES (213, 3, 2, 5, 1, 107, '最小单体电压', 2, 0, 4, 23, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'mv', 1);
INSERT INTO "point" VALUES (214, 3, 2, 5, 1, 108, '平均单体电压', 2, 0, 4, 19, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'mv', 1);
INSERT INTO "point" VALUES (215, 3, 2, 5, 1, 109, '最高单体温度', 2, 0, 4, 11, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (216, 3, 2, 5, 1, 110, '最低单体温度', 2, 0, 4, 14, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (217, 3, 2, 5, 1, 111, '平均单体温度', 2, 0, 4, 17, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (218, 3, 2, 5, 1, 112, '充电限制电流', 2, 0, 4, 150, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (219, 3, 2, 5, 1, 113, '放电限制电流', 2, 0, 4, 151, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "point" VALUES (220, 3, 2, 5, 1, 114, '充电限制功率', 2, 0, 4, 152, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (221, 3, 2, 5, 1, 115, '放电限制功率', 2, 0, 4, 153, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "point" VALUES (222, 3, 2, 5, 1, 151, 'PACK均衡状态字1', 2, 0, 4, 7000, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (223, 3, 2, 5, 1, 152, 'PACK均衡状态字2', 2, 0, 4, 7001, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (224, 3, 2, 5, 1, 153, 'PACK均衡状态字3', 2, 0, 4, 7002, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (225, 3, 2, 5, 1, 154, 'PACK均衡状态字4', 2, 0, 4, 7003, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (226, 3, 2, 5, 1, 155, 'PACK均衡状态字5', 2, 0, 4, 7004, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (227, 3, 2, 5, 1, 156, 'PACK均衡状态字6', 2, 0, 4, 7005, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (228, 3, 2, 5, 1, 157, 'PACK均衡状态字7', 2, 0, 4, 7006, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (229, 3, 2, 5, 1, 158, 'PACK均衡状态字8', 2, 0, 4, 7007, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (230, 3, 2, 5, 1, 159, 'PACK均衡状态字9', 2, 0, 4, 7008, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (231, 3, 2, 5, 1, 160, 'PACK均衡状态字10', 2, 0, 4, 7009, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (232, 3, 2, 5, 1, 161, 'PACK均衡状态字11', 2, 0, 4, 7010, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (233, 3, 2, 5, 1, 162, 'PACK均衡状态字12', 2, 0, 4, 7011, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (234, 3, 2, 5, 1, 163, 'PACK均衡状态字13', 2, 0, 4, 7012, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (235, 3, 2, 5, 1, 164, 'PACK均衡状态字14', 2, 0, 4, 7013, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (236, 3, 2, 5, 1, 165, 'PACK均衡状态字15', 2, 0, 4, 7014, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (237, 3, 2, 5, 1, 166, 'PACK均衡状态字16', 2, 0, 4, 7015, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (238, 3, 2, 5, 1, 167, 'PACK均衡状态字17', 2, 0, 4, 7016, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (239, 3, 2, 5, 1, 168, 'PACK均衡状态字18', 2, 0, 4, 7017, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (240, 3, 2, 5, 1, 169, 'PACK均衡状态字19', 2, 0, 4, 7018, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (241, 3, 2, 5, 1, 170, 'PACK均衡状态字20', 2, 0, 4, 7019, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (242, 3, 2, 5, 1, 171, 'PACK均衡状态字21', 2, 0, 4, 7020, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (243, 3, 2, 5, 1, 172, 'PACK均衡状态字22', 2, 0, 4, 7021, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (244, 3, 2, 5, 1, 173, 'PACK均衡状态字23', 2, 0, 4, 7022, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (245, 3, 2, 5, 1, 174, 'PACK均衡状态字24', 2, 0, 4, 7023, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (246, 3, 2, 5, 1, 175, 'PACK均衡状态字25', 2, 0, 4, 7024, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (247, 3, 2, 5, 1, 176, 'PACK均衡状态字26', 2, 0, 4, 7025, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (248, 3, 2, 5, 1, 177, 'PACK均衡状态字27', 2, 0, 4, 7026, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (249, 3, 2, 5, 1, 178, 'PACK均衡状态字28', 2, 0, 4, 7027, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (250, 3, 2, 5, 1, 179, 'PACK均衡状态字29', 2, 0, 4, 7028, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (251, 3, 2, 5, 1, 180, 'PACK均衡状态字30', 2, 0, 4, 7029, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (252, 3, 2, 5, 1, 201, '单体电压1', 2, 0, 4, 100, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (253, 3, 2, 5, 1, 202, '单体电压2', 2, 0, 4, 101, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (254, 3, 2, 5, 1, 203, '单体电压3', 2, 0, 4, 102, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (255, 3, 2, 5, 1, 204, '单体电压4', 2, 0, 4, 103, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (256, 3, 2, 5, 1, 205, '单体电压5', 2, 0, 4, 104, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (257, 3, 2, 5, 1, 206, '单体电压6', 2, 0, 4, 105, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (258, 3, 2, 5, 1, 207, '单体电压7', 2, 0, 4, 106, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (259, 3, 2, 5, 1, 208, '单体电压8', 2, 0, 4, 107, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (260, 3, 2, 5, 1, 209, '单体电压9', 2, 0, 4, 108, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (261, 3, 2, 5, 1, 210, '单体电压10', 2, 0, 4, 109, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (262, 3, 2, 5, 1, 211, '单体电压11', 2, 0, 4, 110, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (263, 3, 2, 5, 1, 212, '单体电压12', 2, 0, 4, 111, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (264, 3, 2, 5, 1, 213, '单体电压13', 2, 0, 4, 112, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (265, 3, 2, 5, 1, 214, '单体电压14', 2, 0, 4, 113, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (266, 3, 2, 5, 1, 215, '单体电压15', 2, 0, 4, 114, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (267, 3, 2, 5, 1, 216, '单体电压16', 2, 0, 4, 115, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (268, 3, 2, 5, 1, 217, '单体电压17', 2, 0, 4, 116, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (269, 3, 2, 5, 1, 218, '单体电压18', 2, 0, 4, 117, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (270, 3, 2, 5, 1, 219, '单体电压19', 2, 0, 4, 118, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (271, 3, 2, 5, 1, 220, '单体电压20', 2, 0, 4, 119, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (272, 3, 2, 5, 1, 221, '单体电压21', 2, 0, 4, 120, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (273, 3, 2, 5, 1, 222, '单体电压22', 2, 0, 4, 121, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (274, 3, 2, 5, 1, 223, '单体电压23', 2, 0, 4, 122, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (275, 3, 2, 5, 1, 224, '单体电压24', 2, 0, 4, 123, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (276, 3, 2, 5, 1, 225, '单体电压25', 2, 0, 4, 124, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (277, 3, 2, 5, 1, 226, '单体电压26', 2, 0, 4, 125, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (278, 3, 2, 5, 1, 227, '单体电压27', 2, 0, 4, 126, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (279, 3, 2, 5, 1, 228, '单体电压28', 2, 0, 4, 127, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (280, 3, 2, 5, 1, 229, '单体电压29', 2, 0, 4, 128, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (281, 3, 2, 5, 1, 230, '单体电压30', 2, 0, 4, 129, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (282, 3, 2, 5, 1, 231, '单体电压31', 2, 0, 4, 130, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (283, 3, 2, 5, 1, 232, '单体电压32', 2, 0, 4, 131, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (284, 3, 2, 5, 1, 233, '单体电压33', 2, 0, 4, 132, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (285, 3, 2, 5, 1, 234, '单体电压34', 2, 0, 4, 133, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (286, 3, 2, 5, 1, 235, '单体电压35', 2, 0, 4, 134, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (287, 3, 2, 5, 1, 236, '单体电压36', 2, 0, 4, 135, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (288, 3, 2, 5, 1, 237, '单体电压37', 2, 0, 4, 136, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (289, 3, 2, 5, 1, 238, '单体电压38', 2, 0, 4, 137, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (290, 3, 2, 5, 1, 239, '单体电压39', 2, 0, 4, 138, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (291, 3, 2, 5, 1, 240, '单体电压40', 2, 0, 4, 139, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (292, 3, 2, 5, 1, 241, '单体电压41', 2, 0, 4, 140, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (293, 3, 2, 5, 1, 242, '单体电压42', 2, 0, 4, 141, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (294, 3, 2, 5, 1, 243, '单体电压43', 2, 0, 4, 142, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (295, 3, 2, 5, 1, 244, '单体电压44', 2, 0, 4, 143, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (296, 3, 2, 5, 1, 245, '单体电压45', 2, 0, 4, 144, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (297, 3, 2, 5, 1, 246, '单体电压46', 2, 0, 4, 145, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (298, 3, 2, 5, 1, 247, '单体电压47', 2, 0, 4, 146, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (299, 3, 2, 5, 1, 248, '单体电压48', 2, 0, 4, 147, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (300, 3, 2, 5, 1, 249, '单体电压49', 2, 0, 4, 148, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (301, 3, 2, 5, 1, 250, '单体电压50', 2, 0, 4, 149, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (302, 3, 2, 5, 1, 251, '单体电压51', 2, 0, 4, 150, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (303, 3, 2, 5, 1, 252, '单体电压52', 2, 0, 4, 151, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (304, 3, 2, 5, 1, 253, '单体电压53', 2, 0, 4, 152, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (305, 3, 2, 5, 1, 254, '单体电压54', 2, 0, 4, 153, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (306, 3, 2, 5, 1, 255, '单体电压55', 2, 0, 4, 154, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (307, 3, 2, 5, 1, 256, '单体电压56', 2, 0, 4, 155, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (308, 3, 2, 5, 1, 257, '单体电压57', 2, 0, 4, 156, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (309, 3, 2, 5, 1, 258, '单体电压58', 2, 0, 4, 157, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (310, 3, 2, 5, 1, 259, '单体电压59', 2, 0, 4, 158, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (311, 3, 2, 5, 1, 260, '单体电压60', 2, 0, 4, 159, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (312, 3, 2, 5, 1, 261, '单体电压61', 2, 0, 4, 160, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (313, 3, 2, 5, 1, 262, '单体电压62', 2, 0, 4, 161, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (314, 3, 2, 5, 1, 263, '单体电压63', 2, 0, 4, 162, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (315, 3, 2, 5, 1, 264, '单体电压64', 2, 0, 4, 163, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (316, 3, 2, 5, 1, 265, '单体电压65', 2, 0, 4, 164, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (317, 3, 2, 5, 1, 266, '单体电压66', 2, 0, 4, 165, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (318, 3, 2, 5, 1, 267, '单体电压67', 2, 0, 4, 166, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (319, 3, 2, 5, 1, 268, '单体电压68', 2, 0, 4, 167, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (320, 3, 2, 5, 1, 269, '单体电压69', 2, 0, 4, 168, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (321, 3, 2, 5, 1, 270, '单体电压70', 2, 0, 4, 169, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (322, 3, 2, 5, 1, 271, '单体电压71', 2, 0, 4, 170, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (323, 3, 2, 5, 1, 272, '单体电压72', 2, 0, 4, 171, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (324, 3, 2, 5, 1, 273, '单体电压73', 2, 0, 4, 172, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (325, 3, 2, 5, 1, 274, '单体电压74', 2, 0, 4, 173, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (326, 3, 2, 5, 1, 275, '单体电压75', 2, 0, 4, 174, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (327, 3, 2, 5, 1, 276, '单体电压76', 2, 0, 4, 175, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (328, 3, 2, 5, 1, 277, '单体电压77', 2, 0, 4, 176, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (329, 3, 2, 5, 1, 278, '单体电压78', 2, 0, 4, 177, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (330, 3, 2, 5, 1, 279, '单体电压79', 2, 0, 4, 178, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (331, 3, 2, 5, 1, 280, '单体电压80', 2, 0, 4, 179, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (332, 3, 2, 5, 1, 281, '单体电压81', 2, 0, 4, 180, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (333, 3, 2, 5, 1, 282, '单体电压82', 2, 0, 4, 181, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (334, 3, 2, 5, 1, 283, '单体电压83', 2, 0, 4, 182, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (335, 3, 2, 5, 1, 284, '单体电压84', 2, 0, 4, 183, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (336, 3, 2, 5, 1, 285, '单体电压85', 2, 0, 4, 184, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (337, 3, 2, 5, 1, 286, '单体电压86', 2, 0, 4, 185, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (338, 3, 2, 5, 1, 287, '单体电压87', 2, 0, 4, 186, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (339, 3, 2, 5, 1, 288, '单体电压88', 2, 0, 4, 187, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (340, 3, 2, 5, 1, 289, '单体电压89', 2, 0, 4, 188, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (341, 3, 2, 5, 1, 290, '单体电压90', 2, 0, 4, 189, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (342, 3, 2, 5, 1, 291, '单体电压91', 2, 0, 4, 190, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (343, 3, 2, 5, 1, 292, '单体电压92', 2, 0, 4, 191, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (344, 3, 2, 5, 1, 293, '单体电压93', 2, 0, 4, 192, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (345, 3, 2, 5, 1, 294, '单体电压94', 2, 0, 4, 193, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (346, 3, 2, 5, 1, 295, '单体电压95', 2, 0, 4, 194, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (347, 3, 2, 5, 1, 296, '单体电压96', 2, 0, 4, 195, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (348, 3, 2, 5, 1, 297, '单体电压97', 2, 0, 4, 196, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (349, 3, 2, 5, 1, 298, '单体电压98', 2, 0, 4, 197, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (350, 3, 2, 5, 1, 299, '单体电压99', 2, 0, 4, 198, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (351, 3, 2, 5, 1, 300, '单体电压100', 2, 0, 4, 199, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (352, 3, 2, 5, 1, 301, '单体电压101', 2, 0, 4, 200, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (353, 3, 2, 5, 1, 302, '单体电压102', 2, 0, 4, 201, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (354, 3, 2, 5, 1, 303, '单体电压103', 2, 0, 4, 202, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (355, 3, 2, 5, 1, 304, '单体电压104', 2, 0, 4, 203, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (356, 3, 2, 5, 1, 305, '单体电压105', 2, 0, 4, 204, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (357, 3, 2, 5, 1, 306, '单体电压106', 2, 0, 4, 205, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (358, 3, 2, 5, 1, 307, '单体电压107', 2, 0, 4, 206, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (359, 3, 2, 5, 1, 308, '单体电压108', 2, 0, 4, 207, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (360, 3, 2, 5, 1, 309, '单体电压109', 2, 0, 4, 208, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (361, 3, 2, 5, 1, 310, '单体电压110', 2, 0, 4, 209, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (362, 3, 2, 5, 1, 311, '单体电压111', 2, 0, 4, 210, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (363, 3, 2, 5, 1, 312, '单体电压112', 2, 0, 4, 211, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (364, 3, 2, 5, 1, 313, '单体电压113', 2, 0, 4, 212, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (365, 3, 2, 5, 1, 314, '单体电压114', 2, 0, 4, 213, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (366, 3, 2, 5, 1, 315, '单体电压115', 2, 0, 4, 214, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (367, 3, 2, 5, 1, 316, '单体电压116', 2, 0, 4, 215, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (368, 3, 2, 5, 1, 317, '单体电压117', 2, 0, 4, 216, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (369, 3, 2, 5, 1, 318, '单体电压118', 2, 0, 4, 217, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (370, 3, 2, 5, 1, 319, '单体电压119', 2, 0, 4, 218, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (371, 3, 2, 5, 1, 320, '单体电压120', 2, 0, 4, 219, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (372, 3, 2, 5, 1, 321, '单体电压121', 2, 0, 4, 220, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (373, 3, 2, 5, 1, 322, '单体电压122', 2, 0, 4, 221, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (374, 3, 2, 5, 1, 323, '单体电压123', 2, 0, 4, 222, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (375, 3, 2, 5, 1, 324, '单体电压124', 2, 0, 4, 223, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (376, 3, 2, 5, 1, 325, '单体电压125', 2, 0, 4, 224, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (377, 3, 2, 5, 1, 326, '单体电压126', 2, 0, 4, 225, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (378, 3, 2, 5, 1, 327, '单体电压127', 2, 0, 4, 226, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (379, 3, 2, 5, 1, 328, '单体电压128', 2, 0, 4, 227, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (380, 3, 2, 5, 1, 329, '单体电压129', 2, 0, 4, 228, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (381, 3, 2, 5, 1, 330, '单体电压130', 2, 0, 4, 229, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (382, 3, 2, 5, 1, 331, '单体电压131', 2, 0, 4, 230, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (383, 3, 2, 5, 1, 332, '单体电压132', 2, 0, 4, 231, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (384, 3, 2, 5, 1, 333, '单体电压133', 2, 0, 4, 232, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (385, 3, 2, 5, 1, 334, '单体电压134', 2, 0, 4, 233, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (386, 3, 2, 5, 1, 335, '单体电压135', 2, 0, 4, 234, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (387, 3, 2, 5, 1, 336, '单体电压136', 2, 0, 4, 235, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (388, 3, 2, 5, 1, 337, '单体电压137', 2, 0, 4, 236, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (389, 3, 2, 5, 1, 338, '单体电压138', 2, 0, 4, 237, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (390, 3, 2, 5, 1, 339, '单体电压139', 2, 0, 4, 238, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (391, 3, 2, 5, 1, 340, '单体电压140', 2, 0, 4, 239, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (392, 3, 2, 5, 1, 341, '单体电压141', 2, 0, 4, 240, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (393, 3, 2, 5, 1, 342, '单体电压142', 2, 0, 4, 241, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (394, 3, 2, 5, 1, 343, '单体电压143', 2, 0, 4, 242, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (395, 3, 2, 5, 1, 344, '单体电压144', 2, 0, 4, 243, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (396, 3, 2, 5, 1, 345, '单体电压145', 2, 0, 4, 244, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (397, 3, 2, 5, 1, 346, '单体电压146', 2, 0, 4, 245, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (398, 3, 2, 5, 1, 347, '单体电压147', 2, 0, 4, 246, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (399, 3, 2, 5, 1, 348, '单体电压148', 2, 0, 4, 247, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (400, 3, 2, 5, 1, 349, '单体电压149', 2, 0, 4, 248, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (401, 3, 2, 5, 1, 350, '单体电压150', 2, 0, 4, 249, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (402, 3, 2, 5, 1, 351, '单体电压151', 2, 0, 4, 250, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (403, 3, 2, 5, 1, 352, '单体电压152', 2, 0, 4, 251, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (404, 3, 2, 5, 1, 353, '单体电压153', 2, 0, 4, 252, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (405, 3, 2, 5, 1, 354, '单体电压154', 2, 0, 4, 253, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (406, 3, 2, 5, 1, 355, '单体电压155', 2, 0, 4, 254, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (407, 3, 2, 5, 1, 356, '单体电压156', 2, 0, 4, 255, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (408, 3, 2, 5, 1, 357, '单体电压157', 2, 0, 4, 256, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (409, 3, 2, 5, 1, 358, '单体电压158', 2, 0, 4, 257, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (410, 3, 2, 5, 1, 359, '单体电压159', 2, 0, 4, 258, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (411, 3, 2, 5, 1, 360, '单体电压160', 2, 0, 4, 259, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (412, 3, 2, 5, 1, 361, '单体电压161', 2, 0, 4, 260, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (413, 3, 2, 5, 1, 362, '单体电压162', 2, 0, 4, 261, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (414, 3, 2, 5, 1, 363, '单体电压163', 2, 0, 4, 262, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (415, 3, 2, 5, 1, 364, '单体电压164', 2, 0, 4, 263, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (416, 3, 2, 5, 1, 365, '单体电压165', 2, 0, 4, 264, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (417, 3, 2, 5, 1, 366, '单体电压166', 2, 0, 4, 265, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (418, 3, 2, 5, 1, 367, '单体电压167', 2, 0, 4, 266, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (419, 3, 2, 5, 1, 368, '单体电压168', 2, 0, 4, 267, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (420, 3, 2, 5, 1, 369, '单体电压169', 2, 0, 4, 268, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (421, 3, 2, 5, 1, 370, '单体电压170', 2, 0, 4, 269, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (422, 3, 2, 5, 1, 371, '单体电压171', 2, 0, 4, 270, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (423, 3, 2, 5, 1, 372, '单体电压172', 2, 0, 4, 271, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (424, 3, 2, 5, 1, 373, '单体电压173', 2, 0, 4, 272, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (425, 3, 2, 5, 1, 374, '单体电压174', 2, 0, 4, 273, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (426, 3, 2, 5, 1, 375, '单体电压175', 2, 0, 4, 274, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (427, 3, 2, 5, 1, 376, '单体电压176', 2, 0, 4, 275, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (428, 3, 2, 5, 1, 377, '单体电压177', 2, 0, 4, 276, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (429, 3, 2, 5, 1, 378, '单体电压178', 2, 0, 4, 277, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (430, 3, 2, 5, 1, 379, '单体电压179', 2, 0, 4, 278, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (431, 3, 2, 5, 1, 380, '单体电压180', 2, 0, 4, 279, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (432, 3, 2, 5, 1, 381, '单体电压181', 2, 0, 4, 280, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (433, 3, 2, 5, 1, 382, '单体电压182', 2, 0, 4, 281, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (434, 3, 2, 5, 1, 383, '单体电压183', 2, 0, 4, 282, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (435, 3, 2, 5, 1, 384, '单体电压184', 2, 0, 4, 283, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (436, 3, 2, 5, 1, 385, '单体电压185', 2, 0, 4, 284, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (437, 3, 2, 5, 1, 386, '单体电压186', 2, 0, 4, 285, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (438, 3, 2, 5, 1, 387, '单体电压187', 2, 0, 4, 286, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (439, 3, 2, 5, 1, 388, '单体电压188', 2, 0, 4, 287, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (440, 3, 2, 5, 1, 389, '单体电压189', 2, 0, 4, 288, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (441, 3, 2, 5, 1, 390, '单体电压190', 2, 0, 4, 289, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (442, 3, 2, 5, 1, 391, '单体电压191', 2, 0, 4, 290, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (443, 3, 2, 5, 1, 392, '单体电压192', 2, 0, 4, 291, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (444, 3, 2, 5, 1, 393, '单体电压193', 2, 0, 4, 292, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (445, 3, 2, 5, 1, 394, '单体电压194', 2, 0, 4, 293, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (446, 3, 2, 5, 1, 395, '单体电压195', 2, 0, 4, 294, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (447, 3, 2, 5, 1, 396, '单体电压196', 2, 0, 4, 295, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (448, 3, 2, 5, 1, 397, '单体电压197', 2, 0, 4, 296, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (449, 3, 2, 5, 1, 398, '单体电压198', 2, 0, 4, 297, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (450, 3, 2, 5, 1, 399, '单体电压199', 2, 0, 4, 298, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (451, 3, 2, 5, 1, 400, '单体电压200', 2, 0, 4, 299, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'mv', 1);
INSERT INTO "point" VALUES (452, 3, 2, 5, 1, 651, '单体SOC1', 2, 0, 4, 1100, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (453, 3, 2, 5, 1, 652, '单体SOC2', 2, 0, 4, 1101, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (454, 3, 2, 5, 1, 653, '单体SOC3', 2, 0, 4, 1102, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (455, 3, 2, 5, 1, 654, '单体SOC4', 2, 0, 4, 1103, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (456, 3, 2, 5, 1, 655, '单体SOC5', 2, 0, 4, 1104, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (457, 3, 2, 5, 1, 656, '单体SOC6', 2, 0, 4, 1105, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (458, 3, 2, 5, 1, 657, '单体SOC7', 2, 0, 4, 1106, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (459, 3, 2, 5, 1, 658, '单体SOC8', 2, 0, 4, 1107, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (460, 3, 2, 5, 1, 659, '单体SOC9', 2, 0, 4, 1108, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (461, 3, 2, 5, 1, 660, '单体SOC10', 2, 0, 4, 1109, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (462, 3, 2, 5, 1, 661, '单体SOC11', 2, 0, 4, 1110, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (463, 3, 2, 5, 1, 662, '单体SOC12', 2, 0, 4, 1111, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (464, 3, 2, 5, 1, 663, '单体SOC13', 2, 0, 4, 1112, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (465, 3, 2, 5, 1, 664, '单体SOC14', 2, 0, 4, 1113, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (466, 3, 2, 5, 1, 665, '单体SOC15', 2, 0, 4, 1114, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (467, 3, 2, 5, 1, 666, '单体SOC16', 2, 0, 4, 1115, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (468, 3, 2, 5, 1, 667, '单体SOC17', 2, 0, 4, 1116, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (469, 3, 2, 5, 1, 668, '单体SOC18', 2, 0, 4, 1117, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (470, 3, 2, 5, 1, 669, '单体SOC19', 2, 0, 4, 1118, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (471, 3, 2, 5, 1, 670, '单体SOC20', 2, 0, 4, 1119, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (472, 3, 2, 5, 1, 671, '单体SOC21', 2, 0, 4, 1120, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (473, 3, 2, 5, 1, 672, '单体SOC22', 2, 0, 4, 1121, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (474, 3, 2, 5, 1, 673, '单体SOC23', 2, 0, 4, 1122, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (475, 3, 2, 5, 1, 674, '单体SOC24', 2, 0, 4, 1123, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (476, 3, 2, 5, 1, 675, '单体SOC25', 2, 0, 4, 1124, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (477, 3, 2, 5, 1, 676, '单体SOC26', 2, 0, 4, 1125, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (478, 3, 2, 5, 1, 677, '单体SOC27', 2, 0, 4, 1126, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (479, 3, 2, 5, 1, 678, '单体SOC28', 2, 0, 4, 1127, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (480, 3, 2, 5, 1, 679, '单体SOC29', 2, 0, 4, 1128, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (481, 3, 2, 5, 1, 680, '单体SOC30', 2, 0, 4, 1129, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (482, 3, 2, 5, 1, 681, '单体SOC31', 2, 0, 4, 1130, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (483, 3, 2, 5, 1, 682, '单体SOC32', 2, 0, 4, 1131, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (484, 3, 2, 5, 1, 683, '单体SOC33', 2, 0, 4, 1132, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (485, 3, 2, 5, 1, 684, '单体SOC34', 2, 0, 4, 1133, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (486, 3, 2, 5, 1, 685, '单体SOC35', 2, 0, 4, 1134, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (487, 3, 2, 5, 1, 686, '单体SOC36', 2, 0, 4, 1135, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (488, 3, 2, 5, 1, 687, '单体SOC37', 2, 0, 4, 1136, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (489, 3, 2, 5, 1, 688, '单体SOC38', 2, 0, 4, 1137, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (490, 3, 2, 5, 1, 689, '单体SOC39', 2, 0, 4, 1138, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (491, 3, 2, 5, 1, 690, '单体SOC40', 2, 0, 4, 1139, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (492, 3, 2, 5, 1, 691, '单体SOC41', 2, 0, 4, 1140, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (493, 3, 2, 5, 1, 692, '单体SOC42', 2, 0, 4, 1141, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (494, 3, 2, 5, 1, 693, '单体SOC43', 2, 0, 4, 1142, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (495, 3, 2, 5, 1, 694, '单体SOC44', 2, 0, 4, 1143, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (496, 3, 2, 5, 1, 695, '单体SOC45', 2, 0, 4, 1144, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (497, 3, 2, 5, 1, 696, '单体SOC46', 2, 0, 4, 1145, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (498, 3, 2, 5, 1, 697, '单体SOC47', 2, 0, 4, 1146, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (499, 3, 2, 5, 1, 698, '单体SOC48', 2, 0, 4, 1147, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (500, 3, 2, 5, 1, 699, '单体SOC49', 2, 0, 4, 1148, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (501, 3, 2, 5, 1, 700, '单体SOC50', 2, 0, 4, 1149, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (502, 3, 2, 5, 1, 701, '单体SOC51', 2, 0, 4, 1150, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (503, 3, 2, 5, 1, 702, '单体SOC52', 2, 0, 4, 1151, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (504, 3, 2, 5, 1, 703, '单体SOC53', 2, 0, 4, 1152, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (505, 3, 2, 5, 1, 704, '单体SOC54', 2, 0, 4, 1153, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (506, 3, 2, 5, 1, 705, '单体SOC55', 2, 0, 4, 1154, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (507, 3, 2, 5, 1, 706, '单体SOC56', 2, 0, 4, 1155, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (508, 3, 2, 5, 1, 707, '单体SOC57', 2, 0, 4, 1156, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (509, 3, 2, 5, 1, 708, '单体SOC58', 2, 0, 4, 1157, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (510, 3, 2, 5, 1, 709, '单体SOC59', 2, 0, 4, 1158, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (511, 3, 2, 5, 1, 710, '单体SOC60', 2, 0, 4, 1159, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (512, 3, 2, 5, 1, 711, '单体SOC61', 2, 0, 4, 1160, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (513, 3, 2, 5, 1, 712, '单体SOC62', 2, 0, 4, 1161, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (514, 3, 2, 5, 1, 713, '单体SOC63', 2, 0, 4, 1162, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (515, 3, 2, 5, 1, 714, '单体SOC64', 2, 0, 4, 1163, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (516, 3, 2, 5, 1, 715, '单体SOC65', 2, 0, 4, 1164, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (517, 3, 2, 5, 1, 716, '单体SOC66', 2, 0, 4, 1165, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (518, 3, 2, 5, 1, 717, '单体SOC67', 2, 0, 4, 1166, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (519, 3, 2, 5, 1, 718, '单体SOC68', 2, 0, 4, 1167, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (520, 3, 2, 5, 1, 719, '单体SOC69', 2, 0, 4, 1168, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (521, 3, 2, 5, 1, 720, '单体SOC70', 2, 0, 4, 1169, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (522, 3, 2, 5, 1, 721, '单体SOC71', 2, 0, 4, 1170, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (523, 3, 2, 5, 1, 722, '单体SOC72', 2, 0, 4, 1171, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (524, 3, 2, 5, 1, 723, '单体SOC73', 2, 0, 4, 1172, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (525, 3, 2, 5, 1, 724, '单体SOC74', 2, 0, 4, 1173, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (526, 3, 2, 5, 1, 725, '单体SOC75', 2, 0, 4, 1174, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (527, 3, 2, 5, 1, 726, '单体SOC76', 2, 0, 4, 1175, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (528, 3, 2, 5, 1, 727, '单体SOC77', 2, 0, 4, 1176, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (529, 3, 2, 5, 1, 728, '单体SOC78', 2, 0, 4, 1177, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (530, 3, 2, 5, 1, 729, '单体SOC79', 2, 0, 4, 1178, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (531, 3, 2, 5, 1, 730, '单体SOC80', 2, 0, 4, 1179, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (532, 3, 2, 5, 1, 731, '单体SOC81', 2, 0, 4, 1180, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (533, 3, 2, 5, 1, 732, '单体SOC82', 2, 0, 4, 1181, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (534, 3, 2, 5, 1, 733, '单体SOC83', 2, 0, 4, 1182, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (535, 3, 2, 5, 1, 734, '单体SOC84', 2, 0, 4, 1183, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (536, 3, 2, 5, 1, 735, '单体SOC85', 2, 0, 4, 1184, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (537, 3, 2, 5, 1, 736, '单体SOC86', 2, 0, 4, 1185, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (538, 3, 2, 5, 1, 737, '单体SOC87', 2, 0, 4, 1186, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (539, 3, 2, 5, 1, 738, '单体SOC88', 2, 0, 4, 1187, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (540, 3, 2, 5, 1, 739, '单体SOC89', 2, 0, 4, 1188, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (541, 3, 2, 5, 1, 740, '单体SOC90', 2, 0, 4, 1189, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (542, 3, 2, 5, 1, 741, '单体SOC91', 2, 0, 4, 1190, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (543, 3, 2, 5, 1, 742, '单体SOC92', 2, 0, 4, 1191, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (544, 3, 2, 5, 1, 743, '单体SOC93', 2, 0, 4, 1192, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (545, 3, 2, 5, 1, 744, '单体SOC94', 2, 0, 4, 1193, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (546, 3, 2, 5, 1, 745, '单体SOC95', 2, 0, 4, 1194, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (547, 3, 2, 5, 1, 746, '单体SOC96', 2, 0, 4, 1195, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (548, 3, 2, 5, 1, 747, '单体SOC97', 2, 0, 4, 1196, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (549, 3, 2, 5, 1, 748, '单体SOC98', 2, 0, 4, 1197, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (550, 3, 2, 5, 1, 749, '单体SOC99', 2, 0, 4, 1198, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (551, 3, 2, 5, 1, 750, '单体SOC100', 2, 0, 4, 1199, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (552, 3, 2, 5, 1, 751, '单体SOC101', 2, 0, 4, 1200, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (553, 3, 2, 5, 1, 752, '单体SOC102', 2, 0, 4, 1201, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (554, 3, 2, 5, 1, 753, '单体SOC103', 2, 0, 4, 1202, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (555, 3, 2, 5, 1, 754, '单体SOC104', 2, 0, 4, 1203, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (556, 3, 2, 5, 1, 755, '单体SOC105', 2, 0, 4, 1204, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (557, 3, 2, 5, 1, 756, '单体SOC106', 2, 0, 4, 1205, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (558, 3, 2, 5, 1, 757, '单体SOC107', 2, 0, 4, 1206, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (559, 3, 2, 5, 1, 758, '单体SOC108', 2, 0, 4, 1207, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (560, 3, 2, 5, 1, 759, '单体SOC109', 2, 0, 4, 1208, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (561, 3, 2, 5, 1, 760, '单体SOC110', 2, 0, 4, 1209, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (562, 3, 2, 5, 1, 761, '单体SOC111', 2, 0, 4, 1210, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (563, 3, 2, 5, 1, 762, '单体SOC112', 2, 0, 4, 1211, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (564, 3, 2, 5, 1, 763, '单体SOC113', 2, 0, 4, 1212, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (565, 3, 2, 5, 1, 764, '单体SOC114', 2, 0, 4, 1213, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (566, 3, 2, 5, 1, 765, '单体SOC115', 2, 0, 4, 1214, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (567, 3, 2, 5, 1, 766, '单体SOC116', 2, 0, 4, 1215, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (568, 3, 2, 5, 1, 767, '单体SOC117', 2, 0, 4, 1216, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (569, 3, 2, 5, 1, 768, '单体SOC118', 2, 0, 4, 1217, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (570, 3, 2, 5, 1, 769, '单体SOC119', 2, 0, 4, 1218, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (571, 3, 2, 5, 1, 770, '单体SOC120', 2, 0, 4, 1219, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (572, 3, 2, 5, 1, 771, '单体SOC121', 2, 0, 4, 1220, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (573, 3, 2, 5, 1, 772, '单体SOC122', 2, 0, 4, 1221, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (574, 3, 2, 5, 1, 773, '单体SOC123', 2, 0, 4, 1222, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (575, 3, 2, 5, 1, 774, '单体SOC124', 2, 0, 4, 1223, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (576, 3, 2, 5, 1, 775, '单体SOC125', 2, 0, 4, 1224, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (577, 3, 2, 5, 1, 776, '单体SOC126', 2, 0, 4, 1225, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (578, 3, 2, 5, 1, 777, '单体SOC127', 2, 0, 4, 1226, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (579, 3, 2, 5, 1, 778, '单体SOC128', 2, 0, 4, 1227, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (580, 3, 2, 5, 1, 779, '单体SOC129', 2, 0, 4, 1228, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (581, 3, 2, 5, 1, 780, '单体SOC130', 2, 0, 4, 1229, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (582, 3, 2, 5, 1, 781, '单体SOC131', 2, 0, 4, 1230, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (583, 3, 2, 5, 1, 782, '单体SOC132', 2, 0, 4, 1231, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (584, 3, 2, 5, 1, 783, '单体SOC133', 2, 0, 4, 1232, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (585, 3, 2, 5, 1, 784, '单体SOC134', 2, 0, 4, 1233, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (586, 3, 2, 5, 1, 785, '单体SOC135', 2, 0, 4, 1234, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (587, 3, 2, 5, 1, 786, '单体SOC136', 2, 0, 4, 1235, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (588, 3, 2, 5, 1, 787, '单体SOC137', 2, 0, 4, 1236, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (589, 3, 2, 5, 1, 788, '单体SOC138', 2, 0, 4, 1237, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (590, 3, 2, 5, 1, 789, '单体SOC139', 2, 0, 4, 1238, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (591, 3, 2, 5, 1, 790, '单体SOC140', 2, 0, 4, 1239, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (592, 3, 2, 5, 1, 791, '单体SOC141', 2, 0, 4, 1240, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (593, 3, 2, 5, 1, 792, '单体SOC142', 2, 0, 4, 1241, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (594, 3, 2, 5, 1, 793, '单体SOC143', 2, 0, 4, 1242, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (595, 3, 2, 5, 1, 794, '单体SOC144', 2, 0, 4, 1243, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (596, 3, 2, 5, 1, 795, '单体SOC145', 2, 0, 4, 1244, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (597, 3, 2, 5, 1, 796, '单体SOC146', 2, 0, 4, 1245, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (598, 3, 2, 5, 1, 797, '单体SOC147', 2, 0, 4, 1246, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (599, 3, 2, 5, 1, 798, '单体SOC148', 2, 0, 4, 1247, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (600, 3, 2, 5, 1, 799, '单体SOC149', 2, 0, 4, 1248, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (601, 3, 2, 5, 1, 800, '单体SOC150', 2, 0, 4, 1249, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (602, 3, 2, 5, 1, 801, '单体SOC151', 2, 0, 4, 1250, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (603, 3, 2, 5, 1, 802, '单体SOC152', 2, 0, 4, 1251, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (604, 3, 2, 5, 1, 803, '单体SOC153', 2, 0, 4, 1252, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (605, 3, 2, 5, 1, 804, '单体SOC154', 2, 0, 4, 1253, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (606, 3, 2, 5, 1, 805, '单体SOC155', 2, 0, 4, 1254, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (607, 3, 2, 5, 1, 806, '单体SOC156', 2, 0, 4, 1255, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (608, 3, 2, 5, 1, 807, '单体SOC157', 2, 0, 4, 1256, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (609, 3, 2, 5, 1, 808, '单体SOC158', 2, 0, 4, 1257, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (610, 3, 2, 5, 1, 809, '单体SOC159', 2, 0, 4, 1258, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (611, 3, 2, 5, 1, 810, '单体SOC160', 2, 0, 4, 1259, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (612, 3, 2, 5, 1, 811, '单体SOC161', 2, 0, 4, 1260, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (613, 3, 2, 5, 1, 812, '单体SOC162', 2, 0, 4, 1261, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (614, 3, 2, 5, 1, 813, '单体SOC163', 2, 0, 4, 1262, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (615, 3, 2, 5, 1, 814, '单体SOC164', 2, 0, 4, 1263, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (616, 3, 2, 5, 1, 815, '单体SOC165', 2, 0, 4, 1264, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (617, 3, 2, 5, 1, 816, '单体SOC166', 2, 0, 4, 1265, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (618, 3, 2, 5, 1, 817, '单体SOC167', 2, 0, 4, 1266, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (619, 3, 2, 5, 1, 818, '单体SOC168', 2, 0, 4, 1267, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (620, 3, 2, 5, 1, 819, '单体SOC169', 2, 0, 4, 1268, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (621, 3, 2, 5, 1, 820, '单体SOC170', 2, 0, 4, 1269, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (622, 3, 2, 5, 1, 821, '单体SOC171', 2, 0, 4, 1270, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (623, 3, 2, 5, 1, 822, '单体SOC172', 2, 0, 4, 1271, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (624, 3, 2, 5, 1, 823, '单体SOC173', 2, 0, 4, 1272, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (625, 3, 2, 5, 1, 824, '单体SOC174', 2, 0, 4, 1273, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (626, 3, 2, 5, 1, 825, '单体SOC175', 2, 0, 4, 1274, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (627, 3, 2, 5, 1, 826, '单体SOC176', 2, 0, 4, 1275, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (628, 3, 2, 5, 1, 827, '单体SOC177', 2, 0, 4, 1276, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (629, 3, 2, 5, 1, 828, '单体SOC178', 2, 0, 4, 1277, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (630, 3, 2, 5, 1, 829, '单体SOC179', 2, 0, 4, 1278, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (631, 3, 2, 5, 1, 830, '单体SOC180', 2, 0, 4, 1279, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (632, 3, 2, 5, 1, 831, '单体SOC181', 2, 0, 4, 1280, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (633, 3, 2, 5, 1, 832, '单体SOC182', 2, 0, 4, 1281, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (634, 3, 2, 5, 1, 833, '单体SOC183', 2, 0, 4, 1282, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (635, 3, 2, 5, 1, 834, '单体SOC184', 2, 0, 4, 1283, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (636, 3, 2, 5, 1, 835, '单体SOC185', 2, 0, 4, 1284, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (637, 3, 2, 5, 1, 836, '单体SOC186', 2, 0, 4, 1285, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (638, 3, 2, 5, 1, 837, '单体SOC187', 2, 0, 4, 1286, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (639, 3, 2, 5, 1, 838, '单体SOC188', 2, 0, 4, 1287, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (640, 3, 2, 5, 1, 839, '单体SOC189', 2, 0, 4, 1288, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (641, 3, 2, 5, 1, 840, '单体SOC190', 2, 0, 4, 1289, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (642, 3, 2, 5, 1, 841, '单体SOC191', 2, 0, 4, 1290, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (643, 3, 2, 5, 1, 842, '单体SOC192', 2, 0, 4, 1291, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (644, 3, 2, 5, 1, 843, '单体SOC193', 2, 0, 4, 1292, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (645, 3, 2, 5, 1, 844, '单体SOC194', 2, 0, 4, 1293, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (646, 3, 2, 5, 1, 845, '单体SOC195', 2, 0, 4, 1294, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (647, 3, 2, 5, 1, 846, '单体SOC196', 2, 0, 4, 1295, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (648, 3, 2, 5, 1, 847, '单体SOC197', 2, 0, 4, 1296, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (649, 3, 2, 5, 1, 848, '单体SOC198', 2, 0, 4, 1297, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (650, 3, 2, 5, 1, 849, '单体SOC199', 2, 0, 4, 1298, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (651, 3, 2, 5, 1, 850, '单体SOC200', 2, 0, 4, 1299, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (652, 3, 2, 5, 1, 1101, '单体SOH1', 2, 0, 4, 1600, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (653, 3, 2, 5, 1, 1102, '单体SOH2', 2, 0, 4, 1601, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (654, 3, 2, 5, 1, 1103, '单体SOH3', 2, 0, 4, 1602, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (655, 3, 2, 5, 1, 1104, '单体SOH4', 2, 0, 4, 1603, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (656, 3, 2, 5, 1, 1105, '单体SOH5', 2, 0, 4, 1604, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (657, 3, 2, 5, 1, 1106, '单体SOH6', 2, 0, 4, 1605, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (658, 3, 2, 5, 1, 1107, '单体SOH7', 2, 0, 4, 1606, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (659, 3, 2, 5, 1, 1108, '单体SOH8', 2, 0, 4, 1607, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (660, 3, 2, 5, 1, 1109, '单体SOH9', 2, 0, 4, 1608, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (661, 3, 2, 5, 1, 1110, '单体SOH10', 2, 0, 4, 1609, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (662, 3, 2, 5, 1, 1111, '单体SOH11', 2, 0, 4, 1610, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (663, 3, 2, 5, 1, 1112, '单体SOH12', 2, 0, 4, 1611, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (664, 3, 2, 5, 1, 1113, '单体SOH13', 2, 0, 4, 1612, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (665, 3, 2, 5, 1, 1114, '单体SOH14', 2, 0, 4, 1613, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (666, 3, 2, 5, 1, 1115, '单体SOH15', 2, 0, 4, 1614, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (667, 3, 2, 5, 1, 1116, '单体SOH16', 2, 0, 4, 1615, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (668, 3, 2, 5, 1, 1117, '单体SOH17', 2, 0, 4, 1616, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (669, 3, 2, 5, 1, 1118, '单体SOH18', 2, 0, 4, 1617, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (670, 3, 2, 5, 1, 1119, '单体SOH19', 2, 0, 4, 1618, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (671, 3, 2, 5, 1, 1120, '单体SOH20', 2, 0, 4, 1619, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (672, 3, 2, 5, 1, 1121, '单体SOH21', 2, 0, 4, 1620, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (673, 3, 2, 5, 1, 1122, '单体SOH22', 2, 0, 4, 1621, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (674, 3, 2, 5, 1, 1123, '单体SOH23', 2, 0, 4, 1622, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (675, 3, 2, 5, 1, 1124, '单体SOH24', 2, 0, 4, 1623, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (676, 3, 2, 5, 1, 1125, '单体SOH25', 2, 0, 4, 1624, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (677, 3, 2, 5, 1, 1126, '单体SOH26', 2, 0, 4, 1625, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (678, 3, 2, 5, 1, 1127, '单体SOH27', 2, 0, 4, 1626, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (679, 3, 2, 5, 1, 1128, '单体SOH28', 2, 0, 4, 1627, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (680, 3, 2, 5, 1, 1129, '单体SOH29', 2, 0, 4, 1628, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (681, 3, 2, 5, 1, 1130, '单体SOH30', 2, 0, 4, 1629, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (682, 3, 2, 5, 1, 1131, '单体SOH31', 2, 0, 4, 1630, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (683, 3, 2, 5, 1, 1132, '单体SOH32', 2, 0, 4, 1631, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (684, 3, 2, 5, 1, 1133, '单体SOH33', 2, 0, 4, 1632, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (685, 3, 2, 5, 1, 1134, '单体SOH34', 2, 0, 4, 1633, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (686, 3, 2, 5, 1, 1135, '单体SOH35', 2, 0, 4, 1634, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (687, 3, 2, 5, 1, 1136, '单体SOH36', 2, 0, 4, 1635, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (688, 3, 2, 5, 1, 1137, '单体SOH37', 2, 0, 4, 1636, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (689, 3, 2, 5, 1, 1138, '单体SOH38', 2, 0, 4, 1637, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (690, 3, 2, 5, 1, 1139, '单体SOH39', 2, 0, 4, 1638, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (691, 3, 2, 5, 1, 1140, '单体SOH40', 2, 0, 4, 1639, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (692, 3, 2, 5, 1, 1141, '单体SOH41', 2, 0, 4, 1640, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (693, 3, 2, 5, 1, 1142, '单体SOH42', 2, 0, 4, 1641, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (694, 3, 2, 5, 1, 1143, '单体SOH43', 2, 0, 4, 1642, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (695, 3, 2, 5, 1, 1144, '单体SOH44', 2, 0, 4, 1643, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (696, 3, 2, 5, 1, 1145, '单体SOH45', 2, 0, 4, 1644, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (697, 3, 2, 5, 1, 1146, '单体SOH46', 2, 0, 4, 1645, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (698, 3, 2, 5, 1, 1147, '单体SOH47', 2, 0, 4, 1646, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (699, 3, 2, 5, 1, 1148, '单体SOH48', 2, 0, 4, 1647, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (700, 3, 2, 5, 1, 1149, '单体SOH49', 2, 0, 4, 1648, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (701, 3, 2, 5, 1, 1150, '单体SOH50', 2, 0, 4, 1649, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (702, 3, 2, 5, 1, 1151, '单体SOH51', 2, 0, 4, 1650, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (703, 3, 2, 5, 1, 1152, '单体SOH52', 2, 0, 4, 1651, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (704, 3, 2, 5, 1, 1153, '单体SOH53', 2, 0, 4, 1652, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (705, 3, 2, 5, 1, 1154, '单体SOH54', 2, 0, 4, 1653, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (706, 3, 2, 5, 1, 1155, '单体SOH55', 2, 0, 4, 1654, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (707, 3, 2, 5, 1, 1156, '单体SOH56', 2, 0, 4, 1655, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (708, 3, 2, 5, 1, 1157, '单体SOH57', 2, 0, 4, 1656, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (709, 3, 2, 5, 1, 1158, '单体SOH58', 2, 0, 4, 1657, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (710, 3, 2, 5, 1, 1159, '单体SOH59', 2, 0, 4, 1658, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (711, 3, 2, 5, 1, 1160, '单体SOH60', 2, 0, 4, 1659, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (712, 3, 2, 5, 1, 1161, '单体SOH61', 2, 0, 4, 1660, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (713, 3, 2, 5, 1, 1162, '单体SOH62', 2, 0, 4, 1661, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (714, 3, 2, 5, 1, 1163, '单体SOH63', 2, 0, 4, 1662, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (715, 3, 2, 5, 1, 1164, '单体SOH64', 2, 0, 4, 1663, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (716, 3, 2, 5, 1, 1165, '单体SOH65', 2, 0, 4, 1664, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (717, 3, 2, 5, 1, 1166, '单体SOH66', 2, 0, 4, 1665, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (718, 3, 2, 5, 1, 1167, '单体SOH67', 2, 0, 4, 1666, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (719, 3, 2, 5, 1, 1168, '单体SOH68', 2, 0, 4, 1667, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (720, 3, 2, 5, 1, 1169, '单体SOH69', 2, 0, 4, 1668, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (721, 3, 2, 5, 1, 1170, '单体SOH70', 2, 0, 4, 1669, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (722, 3, 2, 5, 1, 1171, '单体SOH71', 2, 0, 4, 1670, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (723, 3, 2, 5, 1, 1172, '单体SOH72', 2, 0, 4, 1671, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (724, 3, 2, 5, 1, 1173, '单体SOH73', 2, 0, 4, 1672, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (725, 3, 2, 5, 1, 1174, '单体SOH74', 2, 0, 4, 1673, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (726, 3, 2, 5, 1, 1175, '单体SOH75', 2, 0, 4, 1674, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (727, 3, 2, 5, 1, 1176, '单体SOH76', 2, 0, 4, 1675, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (728, 3, 2, 5, 1, 1177, '单体SOH77', 2, 0, 4, 1676, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (729, 3, 2, 5, 1, 1178, '单体SOH78', 2, 0, 4, 1677, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (730, 3, 2, 5, 1, 1179, '单体SOH79', 2, 0, 4, 1678, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (731, 3, 2, 5, 1, 1180, '单体SOH80', 2, 0, 4, 1679, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (732, 3, 2, 5, 1, 1181, '单体SOH81', 2, 0, 4, 1680, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (733, 3, 2, 5, 1, 1182, '单体SOH82', 2, 0, 4, 1681, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (734, 3, 2, 5, 1, 1183, '单体SOH83', 2, 0, 4, 1682, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (735, 3, 2, 5, 1, 1184, '单体SOH84', 2, 0, 4, 1683, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (736, 3, 2, 5, 1, 1185, '单体SOH85', 2, 0, 4, 1684, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (737, 3, 2, 5, 1, 1186, '单体SOH86', 2, 0, 4, 1685, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (738, 3, 2, 5, 1, 1187, '单体SOH87', 2, 0, 4, 1686, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (739, 3, 2, 5, 1, 1188, '单体SOH88', 2, 0, 4, 1687, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (740, 3, 2, 5, 1, 1189, '单体SOH89', 2, 0, 4, 1688, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (741, 3, 2, 5, 1, 1190, '单体SOH90', 2, 0, 4, 1689, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (742, 3, 2, 5, 1, 1191, '单体SOH91', 2, 0, 4, 1690, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (743, 3, 2, 5, 1, 1192, '单体SOH92', 2, 0, 4, 1691, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (744, 3, 2, 5, 1, 1193, '单体SOH93', 2, 0, 4, 1692, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (745, 3, 2, 5, 1, 1194, '单体SOH94', 2, 0, 4, 1693, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (746, 3, 2, 5, 1, 1195, '单体SOH95', 2, 0, 4, 1694, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (747, 3, 2, 5, 1, 1196, '单体SOH96', 2, 0, 4, 1695, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (748, 3, 2, 5, 1, 1197, '单体SOH97', 2, 0, 4, 1696, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (749, 3, 2, 5, 1, 1198, '单体SOH98', 2, 0, 4, 1697, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (750, 3, 2, 5, 1, 1199, '单体SOH99', 2, 0, 4, 1698, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (751, 3, 2, 5, 1, 1200, '单体SOH100', 2, 0, 4, 1699, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (752, 3, 2, 5, 1, 1201, '单体SOH101', 2, 0, 4, 1700, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (753, 3, 2, 5, 1, 1202, '单体SOH102', 2, 0, 4, 1701, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (754, 3, 2, 5, 1, 1203, '单体SOH103', 2, 0, 4, 1702, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (755, 3, 2, 5, 1, 1204, '单体SOH104', 2, 0, 4, 1703, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (756, 3, 2, 5, 1, 1205, '单体SOH105', 2, 0, 4, 1704, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (757, 3, 2, 5, 1, 1206, '单体SOH106', 2, 0, 4, 1705, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (758, 3, 2, 5, 1, 1207, '单体SOH107', 2, 0, 4, 1706, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (759, 3, 2, 5, 1, 1208, '单体SOH108', 2, 0, 4, 1707, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (760, 3, 2, 5, 1, 1209, '单体SOH109', 2, 0, 4, 1708, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (761, 3, 2, 5, 1, 1210, '单体SOH110', 2, 0, 4, 1709, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (762, 3, 2, 5, 1, 1211, '单体SOH111', 2, 0, 4, 1710, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (763, 3, 2, 5, 1, 1212, '单体SOH112', 2, 0, 4, 1711, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (764, 3, 2, 5, 1, 1213, '单体SOH113', 2, 0, 4, 1712, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (765, 3, 2, 5, 1, 1214, '单体SOH114', 2, 0, 4, 1713, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (766, 3, 2, 5, 1, 1215, '单体SOH115', 2, 0, 4, 1714, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (767, 3, 2, 5, 1, 1216, '单体SOH116', 2, 0, 4, 1715, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (768, 3, 2, 5, 1, 1217, '单体SOH117', 2, 0, 4, 1716, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (769, 3, 2, 5, 1, 1218, '单体SOH118', 2, 0, 4, 1717, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (770, 3, 2, 5, 1, 1219, '单体SOH119', 2, 0, 4, 1718, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (771, 3, 2, 5, 1, 1220, '单体SOH120', 2, 0, 4, 1719, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (772, 3, 2, 5, 1, 1221, '单体SOH121', 2, 0, 4, 1720, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (773, 3, 2, 5, 1, 1222, '单体SOH122', 2, 0, 4, 1721, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (774, 3, 2, 5, 1, 1223, '单体SOH123', 2, 0, 4, 1722, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (775, 3, 2, 5, 1, 1224, '单体SOH124', 2, 0, 4, 1723, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (776, 3, 2, 5, 1, 1225, '单体SOH125', 2, 0, 4, 1724, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (777, 3, 2, 5, 1, 1226, '单体SOH126', 2, 0, 4, 1725, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (778, 3, 2, 5, 1, 1227, '单体SOH127', 2, 0, 4, 1726, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (779, 3, 2, 5, 1, 1228, '单体SOH128', 2, 0, 4, 1727, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (780, 3, 2, 5, 1, 1229, '单体SOH129', 2, 0, 4, 1728, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (781, 3, 2, 5, 1, 1230, '单体SOH130', 2, 0, 4, 1729, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (782, 3, 2, 5, 1, 1231, '单体SOH131', 2, 0, 4, 1730, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (783, 3, 2, 5, 1, 1232, '单体SOH132', 2, 0, 4, 1731, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (784, 3, 2, 5, 1, 1233, '单体SOH133', 2, 0, 4, 1732, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (785, 3, 2, 5, 1, 1234, '单体SOH134', 2, 0, 4, 1733, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (786, 3, 2, 5, 1, 1235, '单体SOH135', 2, 0, 4, 1734, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (787, 3, 2, 5, 1, 1236, '单体SOH136', 2, 0, 4, 1735, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (788, 3, 2, 5, 1, 1237, '单体SOH137', 2, 0, 4, 1736, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (789, 3, 2, 5, 1, 1238, '单体SOH138', 2, 0, 4, 1737, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (790, 3, 2, 5, 1, 1239, '单体SOH139', 2, 0, 4, 1738, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (791, 3, 2, 5, 1, 1240, '单体SOH140', 2, 0, 4, 1739, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (792, 3, 2, 5, 1, 1241, '单体SOH141', 2, 0, 4, 1740, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (793, 3, 2, 5, 1, 1242, '单体SOH142', 2, 0, 4, 1741, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (794, 3, 2, 5, 1, 1243, '单体SOH143', 2, 0, 4, 1742, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (795, 3, 2, 5, 1, 1244, '单体SOH144', 2, 0, 4, 1743, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (796, 3, 2, 5, 1, 1245, '单体SOH145', 2, 0, 4, 1744, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (797, 3, 2, 5, 1, 1246, '单体SOH146', 2, 0, 4, 1745, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (798, 3, 2, 5, 1, 1247, '单体SOH147', 2, 0, 4, 1746, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (799, 3, 2, 5, 1, 1248, '单体SOH148', 2, 0, 4, 1747, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (800, 3, 2, 5, 1, 1249, '单体SOH149', 2, 0, 4, 1748, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (801, 3, 2, 5, 1, 1250, '单体SOH150', 2, 0, 4, 1749, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (802, 3, 2, 5, 1, 1251, '单体SOH151', 2, 0, 4, 1750, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (803, 3, 2, 5, 1, 1252, '单体SOH152', 2, 0, 4, 1751, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (804, 3, 2, 5, 1, 1253, '单体SOH153', 2, 0, 4, 1752, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (805, 3, 2, 5, 1, 1254, '单体SOH154', 2, 0, 4, 1753, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (806, 3, 2, 5, 1, 1255, '单体SOH155', 2, 0, 4, 1754, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (807, 3, 2, 5, 1, 1256, '单体SOH156', 2, 0, 4, 1755, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (808, 3, 2, 5, 1, 1257, '单体SOH157', 2, 0, 4, 1756, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (809, 3, 2, 5, 1, 1258, '单体SOH158', 2, 0, 4, 1757, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (810, 3, 2, 5, 1, 1259, '单体SOH159', 2, 0, 4, 1758, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (811, 3, 2, 5, 1, 1260, '单体SOH160', 2, 0, 4, 1759, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (812, 3, 2, 5, 1, 1261, '单体SOH161', 2, 0, 4, 1760, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (813, 3, 2, 5, 1, 1262, '单体SOH162', 2, 0, 4, 1761, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (814, 3, 2, 5, 1, 1263, '单体SOH163', 2, 0, 4, 1762, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (815, 3, 2, 5, 1, 1264, '单体SOH164', 2, 0, 4, 1763, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (816, 3, 2, 5, 1, 1265, '单体SOH165', 2, 0, 4, 1764, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (817, 3, 2, 5, 1, 1266, '单体SOH166', 2, 0, 4, 1765, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (818, 3, 2, 5, 1, 1267, '单体SOH167', 2, 0, 4, 1766, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (819, 3, 2, 5, 1, 1268, '单体SOH168', 2, 0, 4, 1767, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (820, 3, 2, 5, 1, 1269, '单体SOH169', 2, 0, 4, 1768, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (821, 3, 2, 5, 1, 1270, '单体SOH170', 2, 0, 4, 1769, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (822, 3, 2, 5, 1, 1271, '单体SOH171', 2, 0, 4, 1770, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (823, 3, 2, 5, 1, 1272, '单体SOH172', 2, 0, 4, 1771, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (824, 3, 2, 5, 1, 1273, '单体SOH173', 2, 0, 4, 1772, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (825, 3, 2, 5, 1, 1274, '单体SOH174', 2, 0, 4, 1773, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (826, 3, 2, 5, 1, 1275, '单体SOH175', 2, 0, 4, 1774, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (827, 3, 2, 5, 1, 1276, '单体SOH176', 2, 0, 4, 1775, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (828, 3, 2, 5, 1, 1277, '单体SOH177', 2, 0, 4, 1776, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (829, 3, 2, 5, 1, 1278, '单体SOH178', 2, 0, 4, 1777, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (830, 3, 2, 5, 1, 1279, '单体SOH179', 2, 0, 4, 1778, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (831, 3, 2, 5, 1, 1280, '单体SOH180', 2, 0, 4, 1779, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (832, 3, 2, 5, 1, 1281, '单体SOH181', 2, 0, 4, 1780, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (833, 3, 2, 5, 1, 1282, '单体SOH182', 2, 0, 4, 1781, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (834, 3, 2, 5, 1, 1283, '单体SOH183', 2, 0, 4, 1782, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (835, 3, 2, 5, 1, 1284, '单体SOH184', 2, 0, 4, 1783, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (836, 3, 2, 5, 1, 1285, '单体SOH185', 2, 0, 4, 1784, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (837, 3, 2, 5, 1, 1286, '单体SOH186', 2, 0, 4, 1785, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (838, 3, 2, 5, 1, 1287, '单体SOH187', 2, 0, 4, 1786, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (839, 3, 2, 5, 1, 1288, '单体SOH188', 2, 0, 4, 1787, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (840, 3, 2, 5, 1, 1289, '单体SOH189', 2, 0, 4, 1788, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (841, 3, 2, 5, 1, 1290, '单体SOH190', 2, 0, 4, 1789, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (842, 3, 2, 5, 1, 1291, '单体SOH191', 2, 0, 4, 1790, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (843, 3, 2, 5, 1, 1292, '单体SOH192', 2, 0, 4, 1791, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (844, 3, 2, 5, 1, 1293, '单体SOH193', 2, 0, 4, 1792, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (845, 3, 2, 5, 1, 1294, '单体SOH194', 2, 0, 4, 1793, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (846, 3, 2, 5, 1, 1295, '单体SOH195', 2, 0, 4, 1794, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (847, 3, 2, 5, 1, 1296, '单体SOH196', 2, 0, 4, 1795, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (848, 3, 2, 5, 1, 1297, '单体SOH197', 2, 0, 4, 1796, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (849, 3, 2, 5, 1, 1298, '单体SOH198', 2, 0, 4, 1797, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (850, 3, 2, 5, 1, 1299, '单体SOH199', 2, 0, 4, 1798, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (851, 3, 2, 5, 1, 1300, '单体SOH200', 2, 0, 4, 1799, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "point" VALUES (852, 3, 2, 5, 1, 1551, '单体温度1', 2, 0, 4, 600, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (853, 3, 2, 5, 1, 1552, '单体温度2', 2, 0, 4, 601, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (854, 3, 2, 5, 1, 1553, '单体温度3', 2, 0, 4, 602, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (855, 3, 2, 5, 1, 1554, '单体温度4', 2, 0, 4, 603, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (856, 3, 2, 5, 1, 1555, '单体温度5', 2, 0, 4, 604, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (857, 3, 2, 5, 1, 1556, '单体温度6', 2, 0, 4, 605, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (858, 3, 2, 5, 1, 1557, '单体温度7', 2, 0, 4, 606, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (859, 3, 2, 5, 1, 1558, '单体温度8', 2, 0, 4, 607, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (860, 3, 2, 5, 1, 1559, '单体温度9', 2, 0, 4, 608, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (861, 3, 2, 5, 1, 1560, '单体温度10', 2, 0, 4, 609, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (862, 3, 2, 5, 1, 1561, '单体温度11', 2, 0, 4, 610, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (863, 3, 2, 5, 1, 1562, '单体温度12', 2, 0, 4, 611, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (864, 3, 2, 5, 1, 1563, '单体温度13', 2, 0, 4, 612, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (865, 3, 2, 5, 1, 1564, '单体温度14', 2, 0, 4, 613, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (866, 3, 2, 5, 1, 1565, '单体温度15', 2, 0, 4, 614, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (867, 3, 2, 5, 1, 1566, '单体温度16', 2, 0, 4, 615, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (868, 3, 2, 5, 1, 1567, '单体温度17', 2, 0, 4, 616, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (869, 3, 2, 5, 1, 1568, '单体温度18', 2, 0, 4, 617, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (870, 3, 2, 5, 1, 1569, '单体温度19', 2, 0, 4, 618, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (871, 3, 2, 5, 1, 1570, '单体温度20', 2, 0, 4, 619, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (872, 3, 2, 5, 1, 1571, '单体温度21', 2, 0, 4, 620, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (873, 3, 2, 5, 1, 1572, '单体温度22', 2, 0, 4, 621, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (874, 3, 2, 5, 1, 1573, '单体温度23', 2, 0, 4, 622, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (875, 3, 2, 5, 1, 1574, '单体温度24', 2, 0, 4, 623, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (876, 3, 2, 5, 1, 1575, '单体温度25', 2, 0, 4, 624, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (877, 3, 2, 5, 1, 1576, '单体温度26', 2, 0, 4, 625, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (878, 3, 2, 5, 1, 1577, '单体温度27', 2, 0, 4, 626, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (879, 3, 2, 5, 1, 1578, '单体温度28', 2, 0, 4, 627, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (880, 3, 2, 5, 1, 1579, '单体温度29', 2, 0, 4, 628, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (881, 3, 2, 5, 1, 1580, '单体温度30', 2, 0, 4, 629, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (882, 3, 2, 5, 1, 1581, '单体温度31', 2, 0, 4, 630, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (883, 3, 2, 5, 1, 1582, '单体温度32', 2, 0, 4, 631, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (884, 3, 2, 5, 1, 1583, '单体温度33', 2, 0, 4, 632, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (885, 3, 2, 5, 1, 1584, '单体温度34', 2, 0, 4, 633, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (886, 3, 2, 5, 1, 1585, '单体温度35', 2, 0, 4, 634, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (887, 3, 2, 5, 1, 1586, '单体温度36', 2, 0, 4, 635, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (888, 3, 2, 5, 1, 1587, '单体温度37', 2, 0, 4, 636, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (889, 3, 2, 5, 1, 1588, '单体温度38', 2, 0, 4, 637, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (890, 3, 2, 5, 1, 1589, '单体温度39', 2, 0, 4, 638, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (891, 3, 2, 5, 1, 1590, '单体温度40', 2, 0, 4, 639, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (892, 3, 2, 5, 1, 1591, '单体温度41', 2, 0, 4, 640, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (893, 3, 2, 5, 1, 1592, '单体温度42', 2, 0, 4, 641, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (894, 3, 2, 5, 1, 1593, '单体温度43', 2, 0, 4, 642, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (895, 3, 2, 5, 1, 1594, '单体温度44', 2, 0, 4, 643, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (896, 3, 2, 5, 1, 1595, '单体温度45', 2, 0, 4, 644, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (897, 3, 2, 5, 1, 1596, '单体温度46', 2, 0, 4, 645, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (898, 3, 2, 5, 1, 1597, '单体温度47', 2, 0, 4, 646, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (899, 3, 2, 5, 1, 1598, '单体温度48', 2, 0, 4, 647, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (900, 3, 2, 5, 1, 1599, '单体温度49', 2, 0, 4, 648, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (901, 3, 2, 5, 1, 1600, '单体温度50', 2, 0, 4, 649, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (902, 3, 2, 5, 1, 1601, '单体温度51', 2, 0, 4, 650, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (903, 3, 2, 5, 1, 1602, '单体温度52', 2, 0, 4, 651, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (904, 3, 2, 5, 1, 1603, '单体温度53', 2, 0, 4, 652, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (905, 3, 2, 5, 1, 1604, '单体温度54', 2, 0, 4, 653, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (906, 3, 2, 5, 1, 1605, '单体温度55', 2, 0, 4, 654, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (907, 3, 2, 5, 1, 1606, '单体温度56', 2, 0, 4, 655, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (908, 3, 2, 5, 1, 1607, '单体温度57', 2, 0, 4, 656, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (909, 3, 2, 5, 1, 1608, '单体温度58', 2, 0, 4, 657, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (910, 3, 2, 5, 1, 1609, '单体温度59', 2, 0, 4, 658, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (911, 3, 2, 5, 1, 1610, '单体温度60', 2, 0, 4, 659, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (912, 3, 2, 5, 1, 1611, '单体温度61', 2, 0, 4, 660, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (913, 3, 2, 5, 1, 1612, '单体温度62', 2, 0, 4, 661, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (914, 3, 2, 5, 1, 1613, '单体温度63', 2, 0, 4, 662, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (915, 3, 2, 5, 1, 1614, '单体温度64', 2, 0, 4, 663, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (916, 3, 2, 5, 1, 1615, '单体温度65', 2, 0, 4, 664, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (917, 3, 2, 5, 1, 1616, '单体温度66', 2, 0, 4, 665, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (918, 3, 2, 5, 1, 1617, '单体温度67', 2, 0, 4, 666, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (919, 3, 2, 5, 1, 1618, '单体温度68', 2, 0, 4, 667, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (920, 3, 2, 5, 1, 1619, '单体温度69', 2, 0, 4, 668, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (921, 3, 2, 5, 1, 1620, '单体温度70', 2, 0, 4, 669, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (922, 3, 2, 5, 1, 1621, '单体温度71', 2, 0, 4, 670, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (923, 3, 2, 5, 1, 1622, '单体温度72', 2, 0, 4, 671, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (924, 3, 2, 5, 1, 1623, '单体温度73', 2, 0, 4, 672, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (925, 3, 2, 5, 1, 1624, '单体温度74', 2, 0, 4, 673, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (926, 3, 2, 5, 1, 1625, '单体温度75', 2, 0, 4, 674, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (927, 3, 2, 5, 1, 1626, '单体温度76', 2, 0, 4, 675, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (928, 3, 2, 5, 1, 1627, '单体温度77', 2, 0, 4, 676, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (929, 3, 2, 5, 1, 1628, '单体温度78', 2, 0, 4, 677, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (930, 3, 2, 5, 1, 1629, '单体温度79', 2, 0, 4, 678, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (931, 3, 2, 5, 1, 1630, '单体温度80', 2, 0, 4, 679, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (932, 3, 2, 5, 1, 1631, '单体温度81', 2, 0, 4, 680, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (933, 3, 2, 5, 1, 1632, '单体温度82', 2, 0, 4, 681, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (934, 3, 2, 5, 1, 1633, '单体温度83', 2, 0, 4, 682, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (935, 3, 2, 5, 1, 1634, '单体温度84', 2, 0, 4, 683, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (936, 3, 2, 5, 1, 1635, '单体温度85', 2, 0, 4, 684, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (937, 3, 2, 5, 1, 1636, '单体温度86', 2, 0, 4, 685, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (938, 3, 2, 5, 1, 1637, '单体温度87', 2, 0, 4, 686, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (939, 3, 2, 5, 1, 1638, '单体温度88', 2, 0, 4, 687, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (940, 3, 2, 5, 1, 1639, '单体温度89', 2, 0, 4, 688, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (941, 3, 2, 5, 1, 1640, '单体温度90', 2, 0, 4, 689, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (942, 3, 2, 5, 1, 1641, '单体温度91', 2, 0, 4, 690, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (943, 3, 2, 5, 1, 1642, '单体温度92', 2, 0, 4, 691, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (944, 3, 2, 5, 1, 1643, '单体温度93', 2, 0, 4, 692, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (945, 3, 2, 5, 1, 1644, '单体温度94', 2, 0, 4, 693, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (946, 3, 2, 5, 1, 1645, '单体温度95', 2, 0, 4, 694, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (947, 3, 2, 5, 1, 1646, '单体温度96', 2, 0, 4, 695, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (948, 3, 2, 5, 1, 1647, '单体温度97', 2, 0, 4, 696, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (949, 3, 2, 5, 1, 1648, '单体温度98', 2, 0, 4, 697, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (950, 3, 2, 5, 1, 1649, '单体温度99', 2, 0, 4, 698, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (951, 3, 2, 5, 1, 1650, '单体温度100', 2, 0, 4, 699, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (952, 3, 2, 5, 1, 1651, '单体温度101', 2, 0, 4, 700, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (953, 3, 2, 5, 1, 1652, '单体温度102', 2, 0, 4, 701, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (954, 3, 2, 5, 1, 1653, '单体温度103', 2, 0, 4, 702, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (955, 3, 2, 5, 1, 1654, '单体温度104', 2, 0, 4, 703, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (956, 3, 2, 5, 1, 1655, '单体温度105', 2, 0, 4, 704, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (957, 3, 2, 5, 1, 1656, '单体温度106', 2, 0, 4, 705, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (958, 3, 2, 5, 1, 1657, '单体温度107', 2, 0, 4, 706, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (959, 3, 2, 5, 1, 1658, '单体温度108', 2, 0, 4, 707, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (960, 3, 2, 5, 1, 1659, '单体温度109', 2, 0, 4, 708, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (961, 3, 2, 5, 1, 1660, '单体温度110', 2, 0, 4, 709, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (962, 3, 2, 5, 1, 1661, '单体温度111', 2, 0, 4, 710, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (963, 3, 2, 5, 1, 1662, '单体温度112', 2, 0, 4, 711, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (964, 3, 2, 5, 1, 1663, '单体温度113', 2, 0, 4, 712, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (965, 3, 2, 5, 1, 1664, '单体温度114', 2, 0, 4, 713, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (966, 3, 2, 5, 1, 1665, '单体温度115', 2, 0, 4, 714, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (967, 3, 2, 5, 1, 1666, '单体温度116', 2, 0, 4, 715, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (968, 3, 2, 5, 1, 1667, '单体温度117', 2, 0, 4, 716, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (969, 3, 2, 5, 1, 1668, '单体温度118', 2, 0, 4, 717, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (970, 3, 2, 5, 1, 1669, '单体温度119', 2, 0, 4, 718, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (971, 3, 2, 5, 1, 1670, '单体温度120', 2, 0, 4, 719, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (972, 3, 2, 5, 1, 1671, '单体温度121', 2, 0, 4, 720, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (973, 3, 2, 5, 1, 1672, '单体温度122', 2, 0, 4, 721, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (974, 3, 2, 5, 1, 1673, '单体温度123', 2, 0, 4, 722, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (975, 3, 2, 5, 1, 1674, '单体温度124', 2, 0, 4, 723, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (976, 3, 2, 5, 1, 1675, '单体温度125', 2, 0, 4, 724, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (977, 3, 2, 5, 1, 1676, '单体温度126', 2, 0, 4, 725, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (978, 3, 2, 5, 1, 1677, '单体温度127', 2, 0, 4, 726, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (979, 3, 2, 5, 1, 1678, '单体温度128', 2, 0, 4, 727, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (980, 3, 2, 5, 1, 1679, '单体温度129', 2, 0, 4, 728, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (981, 3, 2, 5, 1, 1680, '单体温度130', 2, 0, 4, 729, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (982, 3, 2, 5, 1, 1681, '单体温度131', 2, 0, 4, 730, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (983, 3, 2, 5, 1, 1682, '单体温度132', 2, 0, 4, 731, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (984, 3, 2, 5, 1, 1683, '单体温度133', 2, 0, 4, 732, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (985, 3, 2, 5, 1, 1684, '单体温度134', 2, 0, 4, 733, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (986, 3, 2, 5, 1, 1685, '单体温度135', 2, 0, 4, 734, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (987, 3, 2, 5, 1, 1686, '单体温度136', 2, 0, 4, 735, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (988, 3, 2, 5, 1, 1687, '单体温度137', 2, 0, 4, 736, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (989, 3, 2, 5, 1, 1688, '单体温度138', 2, 0, 4, 737, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (990, 3, 2, 5, 1, 1689, '单体温度139', 2, 0, 4, 738, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (991, 3, 2, 5, 1, 1690, '单体温度140', 2, 0, 4, 739, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (992, 3, 2, 5, 1, 1691, '单体温度141', 2, 0, 4, 740, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (993, 3, 2, 5, 1, 1692, '单体温度142', 2, 0, 4, 741, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (994, 3, 2, 5, 1, 1693, '单体温度143', 2, 0, 4, 742, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (995, 3, 2, 5, 1, 1694, '单体温度144', 2, 0, 4, 743, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (996, 3, 2, 5, 1, 1695, '单体温度145', 2, 0, 4, 744, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (997, 3, 2, 5, 1, 1696, '单体温度146', 2, 0, 4, 745, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (998, 3, 2, 5, 1, 1697, '单体温度147', 2, 0, 4, 746, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (999, 3, 2, 5, 1, 1698, '单体温度148', 2, 0, 4, 747, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1000, 3, 2, 5, 1, 1699, '单体温度149', 2, 0, 4, 748, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1001, 3, 2, 5, 1, 1700, '单体温度150', 2, 0, 4, 749, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1002, 3, 2, 5, 1, 1701, '单体温度151', 2, 0, 4, 750, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1003, 3, 2, 5, 1, 1702, '单体温度152', 2, 0, 4, 751, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1004, 3, 2, 5, 1, 1703, '单体温度153', 2, 0, 4, 752, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1005, 3, 2, 5, 1, 1704, '单体温度154', 2, 0, 4, 753, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1006, 3, 2, 5, 1, 1705, '单体温度155', 2, 0, 4, 754, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1007, 3, 2, 5, 1, 1706, '单体温度156', 2, 0, 4, 755, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1008, 3, 2, 5, 1, 1707, '单体温度157', 2, 0, 4, 756, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1009, 3, 2, 5, 1, 1708, '单体温度158', 2, 0, 4, 757, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1010, 3, 2, 5, 1, 1709, '单体温度159', 2, 0, 4, 758, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1011, 3, 2, 5, 1, 1710, '单体温度160', 2, 0, 4, 759, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1012, 3, 2, 5, 1, 1711, '单体温度161', 2, 0, 4, 760, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1013, 3, 2, 5, 1, 1712, '单体温度162', 2, 0, 4, 761, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1014, 3, 2, 5, 1, 1713, '单体温度163', 2, 0, 4, 762, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1015, 3, 2, 5, 1, 1714, '单体温度164', 2, 0, 4, 763, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1016, 3, 2, 5, 1, 1715, '单体温度165', 2, 0, 4, 764, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1017, 3, 2, 5, 1, 1716, '单体温度166', 2, 0, 4, 765, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1018, 3, 2, 5, 1, 1717, '单体温度167', 2, 0, 4, 766, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1019, 3, 2, 5, 1, 1718, '单体温度168', 2, 0, 4, 767, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1020, 3, 2, 5, 1, 1719, '单体温度169', 2, 0, 4, 768, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1021, 3, 2, 5, 1, 1720, '单体温度170', 2, 0, 4, 769, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1022, 3, 2, 5, 1, 1721, '单体温度171', 2, 0, 4, 770, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1023, 3, 2, 5, 1, 1722, '单体温度172', 2, 0, 4, 771, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1024, 3, 2, 5, 1, 1723, '单体温度173', 2, 0, 4, 772, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1025, 3, 2, 5, 1, 1724, '单体温度174', 2, 0, 4, 773, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1026, 3, 2, 5, 1, 1725, '单体温度175', 2, 0, 4, 774, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1027, 3, 2, 5, 1, 1726, '单体温度176', 2, 0, 4, 775, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1028, 3, 2, 5, 1, 1727, '单体温度177', 2, 0, 4, 776, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1029, 3, 2, 5, 1, 1728, '单体温度178', 2, 0, 4, 777, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1030, 3, 2, 5, 1, 1729, '单体温度179', 2, 0, 4, 778, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1031, 3, 2, 5, 1, 1730, '单体温度180', 2, 0, 4, 779, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1032, 3, 2, 5, 1, 1731, '单体温度181', 2, 0, 4, 780, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1033, 3, 2, 5, 1, 1732, '单体温度182', 2, 0, 4, 781, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1034, 3, 2, 5, 1, 1733, '单体温度183', 2, 0, 4, 782, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1035, 3, 2, 5, 1, 1734, '单体温度184', 2, 0, 4, 783, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1036, 3, 2, 5, 1, 1735, '单体温度185', 2, 0, 4, 784, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1037, 3, 2, 5, 1, 1736, '单体温度186', 2, 0, 4, 785, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1038, 3, 2, 5, 1, 1737, '单体温度187', 2, 0, 4, 786, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1039, 3, 2, 5, 1, 1738, '单体温度188', 2, 0, 4, 787, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1040, 3, 2, 5, 1, 1739, '单体温度189', 2, 0, 4, 788, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1041, 3, 2, 5, 1, 1740, '单体温度190', 2, 0, 4, 789, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1042, 3, 2, 5, 1, 1741, '单体温度191', 2, 0, 4, 790, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1043, 3, 2, 5, 1, 1742, '单体温度192', 2, 0, 4, 791, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1044, 3, 2, 5, 1, 1743, '单体温度193', 2, 0, 4, 792, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1045, 3, 2, 5, 1, 1744, '单体温度194', 2, 0, 4, 793, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1046, 3, 2, 5, 1, 1745, '单体温度195', 2, 0, 4, 794, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1047, 3, 2, 5, 1, 1746, '单体温度196', 2, 0, 4, 795, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1048, 3, 2, 5, 1, 1747, '单体温度197', 2, 0, 4, 796, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1049, 3, 2, 5, 1, 1748, '单体温度198', 2, 0, 4, 797, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1050, 3, 2, 5, 1, 1749, '单体温度199', 2, 0, 4, 798, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1051, 3, 2, 5, 1, 1750, '单体温度200', 2, 0, 4, 799, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "point" VALUES (1052, 3, 2, 5, 1, 1751, '高压下电指令', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "point" VALUES (1053, 4, 2, 1, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1054, 4, 2, 1, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1055, 4, 2, 1, 1, 2, 'A相电压', 2, 0, 3, 97, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1056, 4, 2, 1, 1, 3, 'B相电压', 2, 0, 3, 98, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1057, 4, 2, 1, 1, 4, 'C相电压', 2, 0, 3, 99, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1058, 4, 2, 1, 1, 5, 'AB线电压', 2, 0, 3, 120, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1059, 4, 2, 1, 1, 6, 'BC线电压', 2, 0, 3, 121, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1060, 4, 2, 1, 1, 7, 'CA线电压', 2, 0, 3, 122, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1061, 4, 2, 1, 1, 8, 'A相电流', 3, 0, 3, 100, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "point" VALUES (1062, 4, 2, 1, 1, 9, 'B相电流', 3, 0, 3, 101, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "point" VALUES (1063, 4, 2, 1, 1, 10, 'C相电流', 3, 0, 3, 102, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "point" VALUES (1064, 4, 2, 1, 1, 11, 'A相有功功率', 5, 0, 3, 356, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "point" VALUES (1065, 4, 2, 1, 1, 12, 'B相有功功率', 5, 0, 3, 358, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "point" VALUES (1066, 4, 2, 1, 1, 13, 'C相有功功率', 5, 0, 3, 360, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "point" VALUES (1067, 4, 2, 1, 1, 14, '总有功功率', 5, 0, 3, 362, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "point" VALUES (1068, 4, 2, 1, 1, 15, 'A相无功功率', 5, 0, 3, 364, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1069, 4, 2, 1, 1, 16, 'B相无功功率', 5, 0, 3, 366, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1070, 4, 2, 1, 1, 17, 'C相无功功率', 5, 0, 3, 368, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1071, 4, 2, 1, 1, 18, '总无功功率', 5, 0, 3, 370, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1072, 4, 2, 1, 1, 19, 'A相视在功率', 5, 0, 3, 372, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1073, 4, 2, 1, 1, 20, 'B相视在功率', 5, 0, 3, 374, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1074, 4, 2, 1, 1, 21, 'C相视在功率', 5, 0, 3, 376, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1075, 4, 2, 1, 1, 22, '总视在功率', 5, 0, 3, 378, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1076, 4, 2, 1, 1, 23, 'A相功率因数', 2, 0, 3, 380, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1077, 4, 2, 1, 1, 24, 'B相功率因数', 2, 0, 3, 381, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1078, 4, 2, 1, 1, 25, 'C相功率因数', 2, 0, 3, 382, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1079, 4, 2, 1, 1, 26, '总功率因数', 2, 0, 3, 383, 0, 0, 0, 0.001, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1080, 4, 2, 1, 1, 27, '电网频率', 2, 0, 3, 119, 0, 0, 0, 0.01, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'HZ', 1);
INSERT INTO "point" VALUES (1081, 4, 2, 1, 1, 28, '电压互感比', 2, 0, 3, 141, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1082, 4, 2, 1, 1, 29, '电流互感比', 2, 0, 3, 142, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1083, 4, 2, 1, 1, 30, '正向有功电度', 4, 0, 3, 10, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1084, 4, 2, 1, 1, 31, '反向有功电度', 4, 0, 3, 20, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1085, 4, 2, 1, 1, 32, '正向无功电度', 4, 0, 3, 40, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1086, 4, 2, 1, 1, 33, '反向无功电度', 4, 0, 3, 50, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1087, 4, 2, 1, 1, 34, '尖正向有功电度', 4, 0, 3, 12, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1088, 4, 2, 1, 1, 35, '尖反向有功电度', 4, 0, 3, 22, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1089, 4, 2, 1, 1, 36, '峰正向有功电度', 4, 0, 3, 14, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1090, 4, 2, 1, 1, 37, '峰反向有功电度', 4, 0, 3, 24, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1091, 4, 2, 1, 1, 38, '平正向有功电度', 4, 0, 3, 16, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1092, 4, 2, 1, 1, 39, '平反向有功电度', 4, 0, 3, 26, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1093, 4, 2, 1, 1, 40, '谷正向有功电度', 4, 0, 3, 18, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1094, 4, 2, 1, 1, 41, '谷反向有功电度', 4, 0, 3, 28, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1095, 4, 2, 1, 1, 42, '当月正向有功最大需量', 2, 0, 3, 123, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kw', 1);
INSERT INTO "point" VALUES (1096, 4, 2, 1, 1, 43, '当月反向有功最大需量', 2, 0, 3, 126, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kw', 1);
INSERT INTO "point" VALUES (1097, 5, 2, 2, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1098, 5, 2, 2, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1099, 5, 2, 2, 1, 2, 'A相电压', 2, 0, 3, 97, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1100, 5, 2, 2, 1, 3, 'B相电压', 2, 0, 3, 98, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1101, 5, 2, 2, 1, 4, 'C相电压', 2, 0, 3, 99, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1102, 5, 2, 2, 1, 5, 'AB线电压', 2, 0, 3, 120, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1103, 5, 2, 2, 1, 6, 'BC线电压', 2, 0, 3, 121, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1104, 5, 2, 2, 1, 7, 'CA线电压', 2, 0, 3, 122, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "point" VALUES (1105, 5, 2, 2, 1, 8, 'A相电流', 3, 0, 3, 100, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "point" VALUES (1106, 5, 2, 2, 1, 9, 'B相电流', 3, 0, 3, 101, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "point" VALUES (1107, 5, 2, 2, 1, 10, 'C相电流', 3, 0, 3, 102, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "point" VALUES (1108, 5, 2, 2, 1, 11, 'A相有功功率', 5, 0, 3, 356, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "point" VALUES (1109, 5, 2, 2, 1, 12, 'B相有功功率', 5, 0, 3, 358, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "point" VALUES (1110, 5, 2, 2, 1, 13, 'C相有功功率', 5, 0, 3, 360, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "point" VALUES (1111, 5, 2, 2, 1, 14, '总有功功率', 5, 0, 3, 362, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "point" VALUES (1112, 5, 2, 2, 1, 15, 'A相无功功率', 5, 0, 3, 364, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1113, 5, 2, 2, 1, 16, 'B相无功功率', 5, 0, 3, 366, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1114, 5, 2, 2, 1, 17, 'C相无功功率', 5, 0, 3, 368, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1115, 5, 2, 2, 1, 18, '总无功功率', 5, 0, 3, 370, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1116, 5, 2, 2, 1, 19, 'A相视在功率', 5, 0, 3, 372, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1117, 5, 2, 2, 1, 20, 'B相视在功率', 5, 0, 3, 374, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1118, 5, 2, 2, 1, 21, 'C相视在功率', 5, 0, 3, 376, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1119, 5, 2, 2, 1, 22, '总视在功率', 5, 0, 3, 378, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1120, 5, 2, 2, 1, 23, 'A相功率因数', 2, 0, 3, 380, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1121, 5, 2, 2, 1, 24, 'B相功率因数', 2, 0, 3, 381, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1122, 5, 2, 2, 1, 25, 'C相功率因数', 2, 0, 3, 382, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1123, 5, 2, 2, 1, 26, '总功率因数', 2, 0, 3, 383, 0, 0, 0, 0.001, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1124, 5, 2, 2, 1, 27, '电网频率', 2, 0, 3, 119, 0, 0, 0, 0.01, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'HZ', 1);
INSERT INTO "point" VALUES (1125, 5, 2, 2, 1, 28, '电压互感比', 2, 0, 3, 141, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1126, 5, 2, 2, 1, 29, '电流互感比', 2, 0, 3, 142, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1127, 5, 2, 2, 1, 30, '正向有功电度', 4, 0, 3, 10, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1128, 5, 2, 2, 1, 31, '反向有功电度', 4, 0, 3, 20, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1129, 5, 2, 2, 1, 32, '正向无功电度', 4, 0, 3, 40, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1130, 5, 2, 2, 1, 33, '反向无功电度', 4, 0, 3, 50, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "point" VALUES (1131, 5, 2, 2, 1, 34, '尖正向有功电度', 4, 0, 3, 12, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1132, 5, 2, 2, 1, 35, '尖反向有功电度', 4, 0, 3, 22, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1133, 5, 2, 2, 1, 36, '峰正向有功电度', 4, 0, 3, 14, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1134, 5, 2, 2, 1, 37, '峰反向有功电度', 4, 0, 3, 24, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1135, 5, 2, 2, 1, 38, '平正向有功电度', 4, 0, 3, 16, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1136, 5, 2, 2, 1, 39, '平反向有功电度', 4, 0, 3, 26, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1137, 5, 2, 2, 1, 40, '谷正向有功电度', 4, 0, 3, 18, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1138, 5, 2, 2, 1, 41, '谷反向有功电度', 4, 0, 3, 28, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "point" VALUES (1139, 5, 2, 2, 1, 42, '当月正向有功最大需量', 2, 0, 3, 123, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kw', 1);
INSERT INTO "point" VALUES (1140, 5, 2, 2, 1, 43, '当月反向有功最大需量', 2, 0, 3, 126, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kw', 1);
INSERT INTO "point" VALUES (1141, 6, 2, 10, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1142, 6, 2, 10, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1143, 6, 2, 10, 1, 2, '水浸一级告警', 2, 0, 4, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1144, 6, 2, 10, 1, 3, '水浸二级告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1145, 6, 2, 10, 1, 4, '水浸三级', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1146, 6, 2, 10, 1, 5, '断线告警', 0, 0, 4, 0, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1147, 6, 2, 10, 1, 6, '烟感告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1148, 7, 2, 10, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1149, 7, 2, 10, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1150, 7, 2, 10, 1, 2, '水浸一级告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1151, 7, 2, 10, 1, 3, '水浸二级告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1152, 7, 2, 10, 1, 4, '水浸三级', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1153, 7, 2, 10, 1, 5, '断线告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1154, 7, 2, 10, 1, 6, '烟感告警', 2, 0, 3, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1155, 8, 2, 9, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1156, 8, 2, 9, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1157, 8, 2, 9, 1, 2, '总故障', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1158, 8, 2, 9, 1, 41, '当前温度', 3, 0, 3, 3, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "point" VALUES (1159, 8, 2, 9, 1, 42, '当前湿度', 3, 0, 3, 3, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '%RH', 1);
INSERT INTO "point" VALUES (1160, 8, 2, 9, 2, 71, '制冷温度', 3, 0, 6, 8, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "point" VALUES (1161, 8, 2, 9, 2, 72, '制热温度', 3, 0, 6, 9, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1162, 8, 2, 9, 2, 73, '制冷回差', 3, 0, 6, 10, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "point" VALUES (1163, 8, 2, 9, 2, 74, '制热回差', 0, 0, 0, 0, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "point" VALUES (1164, 8, 2, 9, 2, 75, '运行模式', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1165, 8, 2, 9, 2, 76, '空调开关机', 2, 0, 5, 51, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "point" VALUES (1166, 8, 2, 9, 2, 77, '电芯最高温度', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "point" VALUES (1167, 8, 2, 9, 2, 78, '电芯最低温度', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "point" VALUES (1168, 8, 2, 9, 2, 79, '电芯平均温度', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);

-- ----------------------------
-- Records of protect_algorithm
-- ----------------------------
INSERT INTO "protect_algorithm" VALUES (1, 100.0, 100.0, 100.0, 100.0, 1, 100.0, 80.0, 90.0, 1, 2.0, 10.0, 5, 95, '备注无');

-- ----------------------------
-- Records of protocol
-- ----------------------------
INSERT INTO "protocol" VALUES (1, 1, 0, 1, 'PCS设备', '{"netId": 1, "uId": 1, "ip": "192.168.0.45", "port": 502, "address":"/", "timeout": 3000}', '备注');
INSERT INTO "protocol" VALUES (2, 1, 1, 10, 'MQTT Broker', '{"clientId":"0","username":"szny","password":"szny123456","rootTopic":"company/ems/EMS001/root","periodTopic":"company/ems/EMS001/period","changeTopic":"company/ems/EMS001/change","historyTopic":"company/ems/EMS001/history","controlTopic":"company/ems/EMS001/control","readTopic":"company/ems/EMS001/read","replyControlTopic":"company/ems/EMS001/control/reply","replyReadTopic":"company/ems/EMS001/read/reply","ip":"tcp://yanguang-pre.company.com:1883","qos":"1","port":"1883","isSsl":"0","caCert":"0","x509Cert":"0","keyCert":"0"}', '备注');
INSERT INTO "protocol" VALUES (3, 6, 0, 2, '1#温湿度传感器', '{"uartId":1,"uId": 1,"data":3,"stop":0,"baud":0,"parity":0,"address":"/dev/ttyS9","timeout":3000}', '备注');
INSERT INTO "protocol" VALUES (4, 7, 0, 2, '1#水浸烟感', '{"uartId":1,"uId": 2,"data":3,"stop":0,"baud":0,"parity":0,"address":"/dev/ttyS9","timeout":3000}', NULL);
INSERT INTO "protocol" VALUES (5, 3, 0, 2, '1#PCC电表', '{"uartId":2,"uId": 1,"data":3,"stop":0,"baud":0,"parity":0,"address":"/dev/ttyS5","timeout":3000}', NULL);
INSERT INTO "protocol" VALUES (6, 5, 0, 2, '1#储能表', '{"uartId":2,"uId": 2,"data":3,"stop":0,"baud":0,"parity":0,"address":"/dev/ttyS5","timeout":3000}', NULL);
INSERT INTO "protocol" VALUES (7, 4, 0, 1, '1#BMS', '{"netId": 1, "uId": 1, "ip": "192.168.0.45", "port": 503, "address":"/", "timeout": 3000}', NULL);
INSERT INTO "protocol" VALUES (8, 8, 0, 2, '2#水浸烟感', '{"uartId":1,"uId": 3,"data":3,"stop":0,"baud":0,"parity":0,"address":"/dev/ttyS9","timeout":3000}', NULL);
INSERT INTO "protocol" VALUES (9, 9, 0, 2, '1#空调', '{"uartId":1,"uId": 4,"data":3,"stop":0,"baud":0,"parity":0,"address":"/dev/ttyS9","timeout":3000}', NULL);

-- ----------------------------
-- Records of pv_date_config
-- ----------------------------
INSERT INTO "pv_date_config" VALUES (1, 1, '10-01', '10-31', '1');

-- ----------------------------
-- Records of pv_time_config
-- ----------------------------
INSERT INTO "pv_time_config" VALUES (1, '1月~4月', '1');

-- ----------------------------
-- Records of pv_time_exe
-- ----------------------------
INSERT INTO "pv_time_exe" VALUES (1, 1, 0, 86400, 10.0, '1');

-- ----------------------------
-- Records of sqlite_sequence
-- ----------------------------
INSERT INTO "sqlite_sequence" VALUES ('device', 9);
INSERT INTO "sqlite_sequence" VALUES ('template', 8);
INSERT INTO "sqlite_sequence" VALUES ('template_point', 1165);
INSERT INTO "sqlite_sequence" VALUES ('template_protocol', 2);
INSERT INTO "sqlite_sequence" VALUES ('point', 1165);
INSERT INTO "sqlite_sequence" VALUES ('ems_device_config', 16);
INSERT INTO "sqlite_sequence" VALUES ('protocol', 9);
INSERT INTO "sqlite_sequence" VALUES ('topology', 5);
INSERT INTO "sqlite_sequence" VALUES ('up_dis_config', 0);
INSERT INTO "sqlite_sequence" VALUES ('up_dis_point', 0);
INSERT INTO "sqlite_sequence" VALUES ('work_mode_set', 1);
INSERT INTO "sqlite_sequence" VALUES ('allocate_power', 1);
INSERT INTO "sqlite_sequence" VALUES ('pv_date_config', 1);
INSERT INTO "sqlite_sequence" VALUES ('pv_time_config', 1);
INSERT INTO "sqlite_sequence" VALUES ('pv_time_exe', 1);
INSERT INTO "sqlite_sequence" VALUES ('protect_algorithm', 1);

-- ----------------------------
-- Records of template
-- ----------------------------
INSERT INTO "template" VALUES (1, 0, 1, '-PCS协议', 'PCS协议');
INSERT INTO "template" VALUES (2, 0, 2, '深圳柏特瑞BTR01157 TH-802C温湿度RS485协议', '温湿度协议');
INSERT INTO "template" VALUES (3, 0, 2, 'ESBCM-RS485 MODBUS通讯协议', 'BMS通讯协议');
INSERT INTO "template" VALUES (4, 0, 2, 'PCC-DTSD1352导轨式多功能电能表', 'PCC点电表协议');
INSERT INTO "template" VALUES (5, 0, 2, '储能点-DTSD1352导轨式多功能电能表', '储能点电表协议');
INSERT INTO "template" VALUES (6, 0, 2, 'HW-58D智能水浸传感器通讯协议', '水浸烟感传感器之水浸协议');
INSERT INTO "template" VALUES (7, 0, 2, 'BTR-3302D-智能光电感烟报警通讯协议', '水浸烟感传感器之烟感协议');
INSERT INTO "template" VALUES (8, 0, 2, '上海霆翔空调ZL-696AR+C5-UAC2 RS485通讯协议', '空调水冷机类协议');

-- ----------------------------
-- Records of template_point
-- ----------------------------
INSERT INTO "template_point" VALUES (1, 1, 1, 3, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (2, 1, 1, 3, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (3, 1, 1, 3, 1, 2, '总故障', 0, 0, 2, 84, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (4, 1, 1, 3, 1, 3, '总告警', 0, 0, 2, 85, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (5, 1, 1, 3, 1, 4, 'A相硬件过流', 0, 0, 4, 272, 1, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (6, 1, 1, 3, 1, 5, 'B相硬件过流', 0, 0, 4, 272, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (7, 1, 1, 3, 1, 6, 'C相硬件过流', 0, 0, 4, 272, 3, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (8, 1, 1, 3, 1, 7, 'N相硬件过流', 0, 0, 4, 272, 4, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (9, 1, 1, 3, 1, 8, '单元直压', 0, 0, 4, 272, 7, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (10, 1, 1, 3, 1, 9, '开关电源欠压', 0, 0, 4, 272, 10, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (11, 1, 1, 3, 1, 10, 'A相IGBT故障', 0, 0, 4, 272, 11, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (12, 1, 1, 3, 1, 11, 'B相IGBT故障', 0, 0, 4, 272, 12, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (13, 1, 1, 3, 1, 12, 'C相IGBT故障', 0, 0, 4, 272, 13, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (14, 1, 1, 3, 1, 13, 'N相IGBT故障', 0, 0, 4, 272, 14, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (15, 1, 1, 3, 1, 14, '过温故障', 0, 0, 4, 272, 15, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (16, 1, 1, 3, 1, 15, 'A相输出过流', 0, 0, 4, 273, 1, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (17, 1, 1, 3, 1, 16, 'A相输出速断', 0, 0, 4, 273, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (18, 1, 1, 3, 1, 17, 'B相输出过流', 0, 0, 4, 273, 3, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (19, 1, 1, 3, 1, 18, 'B相输出速断', 0, 0, 4, 273, 4, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (20, 1, 1, 3, 1, 19, 'C相输出过流', 0, 0, 4, 273, 5, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (21, 1, 1, 3, 1, 20, 'C相输出速断', 0, 0, 4, 273, 6, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (22, 1, 1, 3, 1, 21, 'N相输出过流', 0, 0, 4, 273, 7, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (23, 1, 1, 3, 1, 22, 'N相输出速断', 0, 0, 4, 273, 8, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (24, 1, 1, 3, 1, 23, '交流过压', 0, 0, 4, 273, 9, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (25, 1, 1, 3, 1, 24, '交流欠压', 0, 0, 4, 273, 10, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (26, 1, 1, 3, 1, 25, '交流过频', 0, 0, 4, 273, 11, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (27, 1, 1, 3, 1, 26, '交流欠频', 0, 0, 4, 273, 12, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (28, 1, 1, 3, 1, 27, '电压THDU超限', 0, 0, 4, 273, 13, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (29, 1, 1, 3, 1, 28, '系统缺相', 0, 0, 4, 273, 14, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (30, 1, 1, 3, 1, 29, '系统相序错误', 0, 0, 4, 273, 15, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (31, 1, 1, 3, 1, 30, '直流极性反接', 0, 0, 4, 273, 16, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (32, 1, 1, 3, 1, 31, '直流母线欠压', 0, 0, 4, 274, 1, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (33, 1, 1, 3, 1, 32, '直流母线过压', 0, 0, 4, 274, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (34, 1, 1, 3, 1, 33, '系统过频', 0, 0, 4, 274, 3, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (35, 1, 1, 3, 1, 34, '系统欠频', 0, 0, 4, 274, 4, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (36, 1, 1, 3, 1, 35, '直流充电过流', 0, 0, 4, 274, 5, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (37, 1, 1, 3, 1, 36, '直流放电过流', 0, 0, 4, 274, 6, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (38, 1, 1, 3, 1, 37, '孤岛保护', 0, 0, 4, 274, 7, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (39, 1, 1, 3, 1, 38, '交流主接合闸故障', 0, 0, 4, 274, 9, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (40, 1, 1, 3, 1, 39, '交流主接分闸故障', 0, 0, 4, 274, 10, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (41, 1, 1, 3, 1, 40, '交流软启合闸故障', 0, 0, 4, 274, 11, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (42, 1, 1, 3, 1, 41, '交流软启分闸故障', 0, 0, 4, 274, 12, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (43, 1, 1, 3, 1, 42, '直流主接合闸故障', 0, 0, 4, 274, 13, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (44, 1, 1, 3, 1, 43, '直流主接分闸故障', 0, 0, 4, 274, 14, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (45, 1, 1, 3, 1, 44, '直流软启合闸故障', 0, 0, 4, 274, 15, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (46, 1, 1, 3, 1, 45, '直流软启分闸故障', 0, 0, 4, 274, 16, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (47, 1, 1, 3, 1, 46, '铁电参数存储错误', 0, 0, 4, 275, 1, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (48, 1, 1, 3, 1, 47, '直流软起失败', 0, 0, 4, 275, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (49, 1, 1, 3, 1, 48, '起机条件不满足', 0, 0, 4, 275, 5, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (50, 1, 1, 3, 1, 49, '运行中开关故障', 0, 0, 4, 275, 6, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (51, 1, 1, 3, 1, 50, '逆变器启动超时', 0, 0, 4, 275, 7, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (52, 1, 1, 3, 1, 51, '参数下发设置错误', 0, 0, 4, 275, 8, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (53, 1, 1, 3, 1, 52, 'BMS通讯故障', 0, 0, 4, 275, 9, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (54, 1, 1, 3, 1, 53, 'BMS温度异常', 0, 0, 4, 275, 10, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (55, 1, 1, 3, 1, 54, 'BMS跳机', 0, 0, 4, 275, 11, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (56, 1, 1, 3, 1, 55, 'BMS电池告警', 0, 0, 4, 275, 12, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (57, 1, 1, 3, 1, 56, 'DCDC通讯故障', 0, 0, 4, 275, 13, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (58, 1, 1, 3, 1, 57, 'EMS通讯故障', 0, 0, 4, 275, 14, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (59, 1, 1, 3, 1, 58, '急停故障', 0, 0, 4, 275, 15, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (60, 1, 1, 3, 1, 59, '电池软件欠压', 0, 0, 4, 256, 2, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (61, 1, 1, 3, 1, 60, '电池软件过压', 0, 0, 5, 257, 3, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (62, 1, 1, 3, 1, 61, '母线不平衡异常', 0, 0, 6, 258, 4, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (63, 1, 1, 3, 1, 62, '母线半直压过压', 0, 0, 7, 259, 5, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (64, 1, 1, 3, 1, 150, '停机状态', 0, 0, 2, 81, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (65, 1, 1, 3, 1, 151, '待机状态', 0, 0, 2, 82, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (66, 1, 1, 3, 1, 152, '运行状态', 0, 0, 2, 83, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (67, 1, 1, 3, 1, 153, '并网状态', 0, 0, 2, 88, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (68, 1, 1, 3, 1, 154, '离网状态', 0, 0, 2, 89, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (69, 1, 1, 3, 1, 155, 'BMS干节点信号', 0, 0, 2, 94, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (70, 1, 1, 3, 1, 156, '本地/远程', 0, 0, 1, 7, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (71, 1, 1, 3, 1, 157, 'A相电压', 2, 0, 4, 201, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "template_point" VALUES (72, 1, 1, 3, 1, 158, 'B相电压', 2, 0, 4, 202, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "template_point" VALUES (73, 1, 1, 3, 1, 159, 'C相电压', 2, 0, 4, 203, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "template_point" VALUES (74, 1, 1, 3, 1, 160, 'A相电流', 3, 0, 4, 204, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (75, 1, 1, 3, 1, 161, 'B相电流', 3, 0, 4, 205, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (76, 1, 1, 3, 1, 162, 'C相电流', 3, 0, 4, 206, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (77, 1, 1, 3, 1, 163, '电网频率', 2, 0, 4, 207, 0, 0, 0, 1, 0.01, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'Hz', 1);
INSERT INTO "template_point" VALUES (78, 1, 1, 3, 1, 164, 'A相有功功率', 3, 0, 4, 208, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (79, 1, 1, 3, 1, 165, 'B相有功功率', 3, 0, 4, 209, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (80, 1, 1, 3, 1, 166, 'C相有功功率', 3, 0, 4, 210, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (81, 1, 1, 3, 1, 167, '总有功功率', 3, 0, 4, 211, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (82, 1, 1, 3, 1, 168, 'A相无功功率', 3, 0, 4, 212, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "template_point" VALUES (83, 1, 1, 3, 1, 169, 'B相无功功率', 3, 0, 4, 213, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "template_point" VALUES (84, 1, 1, 3, 1, 170, 'C相无功功率', 3, 0, 4, 214, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "template_point" VALUES (85, 1, 1, 3, 1, 171, '总无功功率', 3, 0, 4, 215, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "template_point" VALUES (86, 1, 1, 3, 1, 172, 'A相视在功率', 2, 0, 4, 216, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVA', 1);
INSERT INTO "template_point" VALUES (87, 1, 1, 3, 1, 173, 'B相视在功率', 2, 0, 4, 217, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVA', 1);
INSERT INTO "template_point" VALUES (88, 1, 1, 3, 1, 174, 'C相视在功率', 2, 0, 4, 218, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVA', 1);
INSERT INTO "template_point" VALUES (89, 1, 1, 3, 1, 175, '总视在功率', 2, 0, 4, 219, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVA', 1);
INSERT INTO "template_point" VALUES (90, 1, 1, 3, 1, 176, 'A相功率因数', 2, 0, 4, 220, 0, 0, 0, 1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (91, 1, 1, 3, 1, 177, 'B相功率因数', 2, 0, 4, 221, 0, 0, 0, 1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (92, 1, 1, 3, 1, 178, 'C相功率因数', 2, 0, 4, 222, 0, 0, 0, 1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (93, 1, 1, 3, 1, 179, '总功率因数', 2, 0, 4, 223, 0, 0, 0, 1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (94, 1, 1, 3, 1, 180, 'PCS输入功率', 3, 0, 4, 224, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (95, 1, 1, 3, 1, 181, 'PCS输入电压', 3, 0, 4, 225, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "template_point" VALUES (96, 1, 1, 3, 1, 182, 'PCS输入电流', 3, 0, 4, 226, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (97, 1, 1, 3, 1, 183, '交流累计充电量', 4, 0, 4, 230, 0, 0, 0, 1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kWh', 1);
INSERT INTO "template_point" VALUES (98, 1, 1, 3, 1, 184, '交流累计放电量', 4, 0, 4, 232, 0, 0, 0, 1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kWh', 1);
INSERT INTO "template_point" VALUES (99, 1, 1, 3, 1, 185, '直流累计充电量', 4, 0, 4, 234, 0, 0, 0, 1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kWh', 1);
INSERT INTO "template_point" VALUES (100, 1, 1, 3, 1, 186, '直流累计放电量', 4, 0, 4, 236, 0, 0, 0, 1, 0.001, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kWh', 1);
INSERT INTO "template_point" VALUES (101, 1, 1, 3, 1, 187, '系统时钟-秒', 2, 0, 4, 239, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (102, 1, 1, 3, 1, 188, '系统时钟-分', 2, 0, 4, 240, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (103, 1, 1, 3, 1, 189, '系统时钟-时', 2, 0, 4, 241, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (104, 1, 1, 3, 1, 190, '系统时钟-日', 2, 0, 4, 242, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (105, 1, 1, 3, 1, 191, '系统时钟-月', 2, 0, 4, 243, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (106, 1, 1, 3, 1, 192, '系统时钟-年', 2, 0, 4, 244, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (107, 1, 1, 3, 1, 193, 'PCS故障字1', 2, 0, 4, 272, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (108, 1, 1, 3, 1, 194, 'PCS故障字2', 2, 0, 4, 273, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (109, 1, 1, 3, 1, 195, 'PCS故障字3', 2, 0, 4, 274, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (110, 1, 1, 3, 1, 196, 'PCS故障字4', 2, 0, 4, 275, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (111, 1, 1, 3, 1, 197, 'PCS故障字5', 2, 0, 4, 256, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (112, 1, 1, 3, 1, 198, 'PCS故障字6', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (113, 1, 1, 3, 1, 199, '最大允许充电电流', 2, 0, 4, 281, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (114, 1, 1, 3, 1, 200, '最大允许放电电流', 2, 0, 4, 282, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (115, 1, 1, 3, 1, 201, '最大允许充电功率', 2, 0, 4, 294, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (116, 1, 1, 3, 1, 202, '最大允许放电功率', 2, 0, 4, 295, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (117, 1, 1, 3, 1, 202, 'IGBT温度', 3, 0, 4, 227, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, '℃', 1);
INSERT INTO "template_point" VALUES (118, 1, 1, 3, 2, 250, '远程本地设置', 2, 1, 5, 7, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (119, 1, 1, 3, 2, 251, '运行模式设置', 2, 1, 6, 301, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (120, 1, 1, 3, 2, 252, '并网离网设置', 2, 1, 6, 306, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (121, 1, 1, 3, 2, 253, '设备开机', 0, 0, 1, 2, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (122, 1, 1, 3, 2, 254, '设备停机', 0, 0, 1, 3, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (123, 1, 1, 3, 2, 255, '设备开关机指令', 2, 1, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (124, 1, 1, 3, 2, 256, '远程急停', 0, 0, 1, 4, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (125, 1, 1, 3, 2, 257, '功率因数设置', 2, 1, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (126, 1, 1, 3, 2, 258, '恒压充电电压设置值', 2, 1, 6, 302, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "template_point" VALUES (127, 1, 1, 3, 2, 259, '恒流充电电流设置值', 3, 1, 6, 303, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (128, 1, 1, 3, 2, 260, '恒功率有功功率设置', 3, 1, 6, 304, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (129, 1, 1, 3, 2, 261, '恒功率无功功率设置', 3, 1, 6, 305, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kVar', 1);
INSERT INTO "template_point" VALUES (130, 2, 2, 6, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (131, 2, 2, 6, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, NULL, 1);
INSERT INTO "template_point" VALUES (132, 2, 2, 6, 1, 2, '温度', 3, 0, 4, 0, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "template_point" VALUES (133, 2, 2, 6, 1, 3, '湿度', 2, 0, 4, 1, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 1.0, 0.0, 0.0, 1.0, '%RH', 1);
INSERT INTO "template_point" VALUES (134, 3, 2, 5, 1, 0, '设备启用', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (135, 3, 2, 5, 1, 1, '设备在线', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (136, 3, 2, 5, 1, 2, '与电池簇通信告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (137, 3, 2, 5, 1, 3, '主从通讯告警', 0, 0, 2, 1000, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (138, 3, 2, 5, 1, 4, '总告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (139, 3, 2, 5, 1, 5, '总故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (140, 3, 2, 5, 1, 6, '单体过压告警一级', 0, 0, 2, 22, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (141, 3, 2, 5, 1, 7, '单体欠压告警一级', 0, 0, 2, 25, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (142, 3, 2, 5, 1, 8, '单体过温告警一级', 0, 0, 2, 16, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (143, 3, 2, 5, 1, 9, '单体低温告警一级', 0, 0, 2, 19, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (144, 3, 2, 5, 1, 10, '单体压差告警一级', 0, 0, 2, 28, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (145, 3, 2, 5, 1, 11, '充电过流告警一级', 0, 0, 2, 10, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (146, 3, 2, 5, 1, 12, '放电过流告警一级', 0, 0, 2, 7, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (147, 3, 2, 5, 1, 13, 'SOC过低告警一级', 0, 0, 2, 34, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (148, 3, 2, 5, 1, 14, 'SOC差异过大告警一级', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (149, 3, 2, 5, 1, 15, '绝缘过低告警一级', 0, 0, 2, 13, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (150, 3, 2, 5, 1, 16, '单体过压告警二级', 0, 0, 2, 23, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (151, 3, 2, 5, 1, 17, '单体欠压告警二级', 0, 0, 2, 26, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (152, 3, 2, 5, 1, 18, '单体过温告警二级', 0, 0, 2, 17, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (153, 3, 2, 5, 1, 19, '单体低温告警二级', 0, 0, 2, 20, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (154, 3, 2, 5, 1, 20, '单体压差告警二级', 0, 0, 2, 29, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (155, 3, 2, 5, 1, 21, '充电过流告警二级', 0, 0, 2, 11, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (156, 3, 2, 5, 1, 22, '放电过流告警二级', 0, 0, 2, 8, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (157, 3, 2, 5, 1, 23, 'SOC过低告警二级', 0, 0, 2, 35, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (158, 3, 2, 5, 1, 24, 'SOC差异过大告警二级', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (159, 3, 2, 5, 1, 25, '绝缘过低告警二级', 0, 0, 2, 14, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (160, 3, 2, 5, 1, 26, '单体过压告警三级', 0, 0, 2, 24, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (161, 3, 2, 5, 1, 27, '单体欠压告警三级', 0, 0, 2, 27, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (162, 3, 2, 5, 1, 28, '单体过温告警三级', 0, 0, 2, 18, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (163, 3, 2, 5, 1, 29, '单体低温告警三级', 0, 0, 2, 21, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (164, 3, 2, 5, 1, 30, '单体压差告警三级', 0, 0, 2, 30, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (165, 3, 2, 5, 1, 31, '充电过流告警三级', 0, 0, 2, 12, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (166, 3, 2, 5, 1, 32, '放电过流告警三级', 0, 0, 2, 9, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (167, 3, 2, 5, 1, 33, 'SOC过低告警三级', 0, 0, 2, 36, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (168, 3, 2, 5, 1, 34, 'SOC差异过大告警三级', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (169, 3, 2, 5, 1, 35, '绝缘过低告警三级', 0, 0, 2, 15, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (170, 3, 2, 5, 1, 36, '电芯温度极限告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (171, 3, 2, 5, 1, 37, '电芯电压极限告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (172, 3, 2, 5, 1, 38, '簇间环流1级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (173, 3, 2, 5, 1, 39, '簇间环流2级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (174, 3, 2, 5, 1, 40, '簇间环流3级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (175, 3, 2, 5, 1, 41, '簇间电流差1级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (176, 3, 2, 5, 1, 42, '簇间电流差2级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (177, 3, 2, 5, 1, 43, '簇间电流差3级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (178, 3, 2, 5, 1, 44, '组端过压1级告警', 0, 0, 2, 1, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (179, 3, 2, 5, 1, 45, '组端过压2级告警', 0, 0, 2, 2, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (180, 3, 2, 5, 1, 46, '组端过压3级告警', 0, 0, 2, 3, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (181, 3, 2, 5, 1, 47, '组端欠压1级告警', 0, 0, 2, 4, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (182, 3, 2, 5, 1, 48, '组端欠压2级告警', 0, 0, 2, 5, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (183, 3, 2, 5, 1, 49, '组端欠压3级告警', 0, 0, 2, 6, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (184, 3, 2, 5, 1, 50, '极柱过温1级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (185, 3, 2, 5, 1, 51, '极柱过温2级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (186, 3, 2, 5, 1, 52, '极柱过温3级告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (187, 3, 2, 5, 1, 53, 'AFE温感排线异常', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (188, 3, 2, 5, 1, 54, 'AFE电压排线异常', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (189, 3, 2, 5, 1, 55, 'AFE故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (190, 3, 2, 5, 1, 56, '高压异常', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (191, 3, 2, 5, 1, 57, '预充告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (192, 3, 2, 5, 1, 58, '开路故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (193, 3, 2, 5, 1, 59, '继电器粘连告警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (194, 3, 2, 5, 1, 60, '电池极限故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (195, 3, 2, 5, 1, 61, '熔丝故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (196, 3, 2, 5, 1, 62, '断路器故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (197, 3, 2, 5, 1, 63, '空调故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (198, 3, 2, 5, 1, 64, '消防喷洒', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (199, 3, 2, 5, 1, 65, '消防设备故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (200, 3, 2, 5, 1, 66, '消防火警', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (201, 3, 2, 5, 1, 67, '急停故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (202, 3, 2, 5, 1, 68, 'MSD故障', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (203, 3, 2, 5, 1, 69, '控制指令超时', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (204, 3, 2, 5, 1, 101, 'SOC', 2, 0, 4, 3, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, '%', 1);
INSERT INTO "template_point" VALUES (205, 3, 2, 5, 1, 102, 'SOH', 2, 0, 4, 4, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, '%', 1);
INSERT INTO "template_point" VALUES (206, 3, 2, 5, 1, 103, '总电压', 2, 0, 4, 1, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'V', 1);
INSERT INTO "template_point" VALUES (207, 3, 2, 5, 1, 104, '总电流', 3, 0, 4, 2, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (208, 3, 2, 5, 1, 105, '系统SOH', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, '%', 1);
INSERT INTO "template_point" VALUES (209, 3, 2, 5, 1, 106, '最大单体电压', 2, 0, 4, 21, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'mv', 1);
INSERT INTO "template_point" VALUES (210, 3, 2, 5, 1, 107, '最小单体电压', 2, 0, 4, 24, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'mv', 1);
INSERT INTO "template_point" VALUES (211, 3, 2, 5, 1, 108, '平均单体电压', 2, 0, 4, 20, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'mv', 1);
INSERT INTO "template_point" VALUES (212, 3, 2, 5, 1, 109, '最高单体温度', 2, 0, 4, 12, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (213, 3, 2, 5, 1, 110, '最低单体温度', 2, 0, 4, 15, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (214, 3, 2, 5, 1, 111, '平均单体温度', 2, 0, 4, 18, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (215, 3, 2, 5, 1, 112, '充电限制电流', 2, 0, 4, 150, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (216, 3, 2, 5, 1, 113, '放电限制电流', 2, 0, 4, 151, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'A', 1);
INSERT INTO "template_point" VALUES (217, 3, 2, 5, 1, 114, '充电限制功率', 2, 0, 4, 152, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (218, 3, 2, 5, 1, 115, '放电限制功率', 2, 0, 4, 153, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, 'kW', 1);
INSERT INTO "template_point" VALUES (219, 3, 2, 5, 1, 151, 'PACK均衡状态字1', 2, 0, 4, 7000, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (220, 3, 2, 5, 1, 152, 'PACK均衡状态字2', 2, 0, 4, 7001, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (221, 3, 2, 5, 1, 153, 'PACK均衡状态字3', 2, 0, 4, 7002, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (222, 3, 2, 5, 1, 154, 'PACK均衡状态字4', 2, 0, 4, 7003, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (223, 3, 2, 5, 1, 155, 'PACK均衡状态字5', 2, 0, 4, 7004, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (224, 3, 2, 5, 1, 156, 'PACK均衡状态字6', 2, 0, 4, 7005, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (225, 3, 2, 5, 1, 157, 'PACK均衡状态字7', 2, 0, 4, 7006, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (226, 3, 2, 5, 1, 158, 'PACK均衡状态字8', 2, 0, 4, 7007, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (227, 3, 2, 5, 1, 159, 'PACK均衡状态字9', 2, 0, 4, 7008, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (228, 3, 2, 5, 1, 160, 'PACK均衡状态字10', 2, 0, 4, 7009, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (229, 3, 2, 5, 1, 161, 'PACK均衡状态字11', 2, 0, 4, 7010, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (230, 3, 2, 5, 1, 162, 'PACK均衡状态字12', 2, 0, 4, 7011, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (231, 3, 2, 5, 1, 163, 'PACK均衡状态字13', 2, 0, 4, 7012, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (232, 3, 2, 5, 1, 164, 'PACK均衡状态字14', 2, 0, 4, 7013, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (233, 3, 2, 5, 1, 165, 'PACK均衡状态字15', 2, 0, 4, 7014, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (234, 3, 2, 5, 1, 166, 'PACK均衡状态字16', 2, 0, 4, 7015, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (235, 3, 2, 5, 1, 167, 'PACK均衡状态字17', 2, 0, 4, 7016, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (236, 3, 2, 5, 1, 168, 'PACK均衡状态字18', 2, 0, 4, 7017, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (237, 3, 2, 5, 1, 169, 'PACK均衡状态字19', 2, 0, 4, 7018, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (238, 3, 2, 5, 1, 170, 'PACK均衡状态字20', 2, 0, 4, 7019, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (239, 3, 2, 5, 1, 171, 'PACK均衡状态字21', 2, 0, 4, 7020, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (240, 3, 2, 5, 1, 172, 'PACK均衡状态字22', 2, 0, 4, 7021, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (241, 3, 2, 5, 1, 173, 'PACK均衡状态字23', 2, 0, 4, 7022, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (242, 3, 2, 5, 1, 174, 'PACK均衡状态字24', 2, 0, 4, 7023, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (243, 3, 2, 5, 1, 175, 'PACK均衡状态字25', 2, 0, 4, 7024, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (244, 3, 2, 5, 1, 176, 'PACK均衡状态字26', 2, 0, 4, 7025, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (245, 3, 2, 5, 1, 177, 'PACK均衡状态字27', 2, 0, 4, 7026, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (246, 3, 2, 5, 1, 178, 'PACK均衡状态字28', 2, 0, 4, 7027, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (247, 3, 2, 5, 1, 179, 'PACK均衡状态字29', 2, 0, 4, 7028, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (248, 3, 2, 5, 1, 180, 'PACK均衡状态字30', 2, 0, 4, 7029, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (249, 3, 2, 5, 1, 201, '单体电压1', 2, 0, 4, 1000, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (250, 3, 2, 5, 1, 202, '单体电压2', 2, 0, 4, 1001, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (251, 3, 2, 5, 1, 203, '单体电压3', 2, 0, 4, 1002, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (252, 3, 2, 5, 1, 204, '单体电压4', 2, 0, 4, 1003, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (253, 3, 2, 5, 1, 205, '单体电压5', 2, 0, 4, 1004, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (254, 3, 2, 5, 1, 206, '单体电压6', 2, 0, 4, 1005, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (255, 3, 2, 5, 1, 207, '单体电压7', 2, 0, 4, 1006, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (256, 3, 2, 5, 1, 208, '单体电压8', 2, 0, 4, 1007, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (257, 3, 2, 5, 1, 209, '单体电压9', 2, 0, 4, 1008, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (258, 3, 2, 5, 1, 210, '单体电压10', 2, 0, 4, 1009, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (259, 3, 2, 5, 1, 211, '单体电压11', 2, 0, 4, 1010, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (260, 3, 2, 5, 1, 212, '单体电压12', 2, 0, 4, 1011, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (261, 3, 2, 5, 1, 213, '单体电压13', 2, 0, 4, 1012, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (262, 3, 2, 5, 1, 214, '单体电压14', 2, 0, 4, 1013, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (263, 3, 2, 5, 1, 215, '单体电压15', 2, 0, 4, 1014, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (264, 3, 2, 5, 1, 216, '单体电压16', 2, 0, 4, 1015, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (265, 3, 2, 5, 1, 217, '单体电压17', 2, 0, 4, 1016, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (266, 3, 2, 5, 1, 218, '单体电压18', 2, 0, 4, 1017, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (267, 3, 2, 5, 1, 219, '单体电压19', 2, 0, 4, 1018, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (268, 3, 2, 5, 1, 220, '单体电压20', 2, 0, 4, 1019, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (269, 3, 2, 5, 1, 221, '单体电压21', 2, 0, 4, 1020, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (270, 3, 2, 5, 1, 222, '单体电压22', 2, 0, 4, 1021, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (271, 3, 2, 5, 1, 223, '单体电压23', 2, 0, 4, 1022, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (272, 3, 2, 5, 1, 224, '单体电压24', 2, 0, 4, 1023, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (273, 3, 2, 5, 1, 225, '单体电压25', 2, 0, 4, 1024, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (274, 3, 2, 5, 1, 226, '单体电压26', 2, 0, 4, 1025, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (275, 3, 2, 5, 1, 227, '单体电压27', 2, 0, 4, 1026, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (276, 3, 2, 5, 1, 228, '单体电压28', 2, 0, 4, 1027, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (277, 3, 2, 5, 1, 229, '单体电压29', 2, 0, 4, 1028, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (278, 3, 2, 5, 1, 230, '单体电压30', 2, 0, 4, 1029, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (279, 3, 2, 5, 1, 231, '单体电压31', 2, 0, 4, 1030, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (280, 3, 2, 5, 1, 232, '单体电压32', 2, 0, 4, 1031, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (281, 3, 2, 5, 1, 233, '单体电压33', 2, 0, 4, 1032, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (282, 3, 2, 5, 1, 234, '单体电压34', 2, 0, 4, 1033, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (283, 3, 2, 5, 1, 235, '单体电压35', 2, 0, 4, 1034, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (284, 3, 2, 5, 1, 236, '单体电压36', 2, 0, 4, 1035, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (285, 3, 2, 5, 1, 237, '单体电压37', 2, 0, 4, 1036, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (286, 3, 2, 5, 1, 238, '单体电压38', 2, 0, 4, 1037, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (287, 3, 2, 5, 1, 239, '单体电压39', 2, 0, 4, 1038, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (288, 3, 2, 5, 1, 240, '单体电压40', 2, 0, 4, 1039, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (289, 3, 2, 5, 1, 241, '单体电压41', 2, 0, 4, 1040, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (290, 3, 2, 5, 1, 242, '单体电压42', 2, 0, 4, 1041, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (291, 3, 2, 5, 1, 243, '单体电压43', 2, 0, 4, 1042, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (292, 3, 2, 5, 1, 244, '单体电压44', 2, 0, 4, 1043, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (293, 3, 2, 5, 1, 245, '单体电压45', 2, 0, 4, 1044, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (294, 3, 2, 5, 1, 246, '单体电压46', 2, 0, 4, 1045, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (295, 3, 2, 5, 1, 247, '单体电压47', 2, 0, 4, 1046, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (296, 3, 2, 5, 1, 248, '单体电压48', 2, 0, 4, 1047, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (297, 3, 2, 5, 1, 249, '单体电压49', 2, 0, 4, 1048, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (298, 3, 2, 5, 1, 250, '单体电压50', 2, 0, 4, 1049, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (299, 3, 2, 5, 1, 251, '单体电压51', 2, 0, 4, 1050, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (300, 3, 2, 5, 1, 252, '单体电压52', 2, 0, 4, 1051, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (301, 3, 2, 5, 1, 253, '单体电压53', 2, 0, 4, 1052, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (302, 3, 2, 5, 1, 254, '单体电压54', 2, 0, 4, 1053, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (303, 3, 2, 5, 1, 255, '单体电压55', 2, 0, 4, 1054, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (304, 3, 2, 5, 1, 256, '单体电压56', 2, 0, 4, 1055, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (305, 3, 2, 5, 1, 257, '单体电压57', 2, 0, 4, 1056, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (306, 3, 2, 5, 1, 258, '单体电压58', 2, 0, 4, 1057, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (307, 3, 2, 5, 1, 259, '单体电压59', 2, 0, 4, 1058, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (308, 3, 2, 5, 1, 260, '单体电压60', 2, 0, 4, 1059, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (309, 3, 2, 5, 1, 261, '单体电压61', 2, 0, 4, 1060, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (310, 3, 2, 5, 1, 262, '单体电压62', 2, 0, 4, 1061, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (311, 3, 2, 5, 1, 263, '单体电压63', 2, 0, 4, 1062, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (312, 3, 2, 5, 1, 264, '单体电压64', 2, 0, 4, 1063, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (313, 3, 2, 5, 1, 265, '单体电压65', 2, 0, 4, 1064, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (314, 3, 2, 5, 1, 266, '单体电压66', 2, 0, 4, 1065, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (315, 3, 2, 5, 1, 267, '单体电压67', 2, 0, 4, 1066, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (316, 3, 2, 5, 1, 268, '单体电压68', 2, 0, 4, 1067, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (317, 3, 2, 5, 1, 269, '单体电压69', 2, 0, 4, 1068, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (318, 3, 2, 5, 1, 270, '单体电压70', 2, 0, 4, 1069, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (319, 3, 2, 5, 1, 271, '单体电压71', 2, 0, 4, 1070, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (320, 3, 2, 5, 1, 272, '单体电压72', 2, 0, 4, 1071, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (321, 3, 2, 5, 1, 273, '单体电压73', 2, 0, 4, 1072, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (322, 3, 2, 5, 1, 274, '单体电压74', 2, 0, 4, 1073, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (323, 3, 2, 5, 1, 275, '单体电压75', 2, 0, 4, 1074, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (324, 3, 2, 5, 1, 276, '单体电压76', 2, 0, 4, 1075, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (325, 3, 2, 5, 1, 277, '单体电压77', 2, 0, 4, 1076, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (326, 3, 2, 5, 1, 278, '单体电压78', 2, 0, 4, 1077, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (327, 3, 2, 5, 1, 279, '单体电压79', 2, 0, 4, 1078, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (328, 3, 2, 5, 1, 280, '单体电压80', 2, 0, 4, 1079, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (329, 3, 2, 5, 1, 281, '单体电压81', 2, 0, 4, 1080, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (330, 3, 2, 5, 1, 282, '单体电压82', 2, 0, 4, 1081, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (331, 3, 2, 5, 1, 283, '单体电压83', 2, 0, 4, 1082, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (332, 3, 2, 5, 1, 284, '单体电压84', 2, 0, 4, 1083, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (333, 3, 2, 5, 1, 285, '单体电压85', 2, 0, 4, 1084, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (334, 3, 2, 5, 1, 286, '单体电压86', 2, 0, 4, 1085, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (335, 3, 2, 5, 1, 287, '单体电压87', 2, 0, 4, 1086, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (336, 3, 2, 5, 1, 288, '单体电压88', 2, 0, 4, 1087, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (337, 3, 2, 5, 1, 289, '单体电压89', 2, 0, 4, 1088, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (338, 3, 2, 5, 1, 290, '单体电压90', 2, 0, 4, 1089, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (339, 3, 2, 5, 1, 291, '单体电压91', 2, 0, 4, 1090, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (340, 3, 2, 5, 1, 292, '单体电压92', 2, 0, 4, 1091, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (341, 3, 2, 5, 1, 293, '单体电压93', 2, 0, 4, 1092, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (342, 3, 2, 5, 1, 294, '单体电压94', 2, 0, 4, 1093, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (343, 3, 2, 5, 1, 295, '单体电压95', 2, 0, 4, 1094, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (344, 3, 2, 5, 1, 296, '单体电压96', 2, 0, 4, 1095, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (345, 3, 2, 5, 1, 297, '单体电压97', 2, 0, 4, 1096, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (346, 3, 2, 5, 1, 298, '单体电压98', 2, 0, 4, 1097, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (347, 3, 2, 5, 1, 299, '单体电压99', 2, 0, 4, 1098, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (348, 3, 2, 5, 1, 300, '单体电压100', 2, 0, 4, 1099, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (349, 3, 2, 5, 1, 301, '单体电压101', 2, 0, 4, 1100, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (350, 3, 2, 5, 1, 302, '单体电压102', 2, 0, 4, 1101, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (351, 3, 2, 5, 1, 303, '单体电压103', 2, 0, 4, 1102, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (352, 3, 2, 5, 1, 304, '单体电压104', 2, 0, 4, 1103, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (353, 3, 2, 5, 1, 305, '单体电压105', 2, 0, 4, 1104, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (354, 3, 2, 5, 1, 306, '单体电压106', 2, 0, 4, 1105, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (355, 3, 2, 5, 1, 307, '单体电压107', 2, 0, 4, 1106, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (356, 3, 2, 5, 1, 308, '单体电压108', 2, 0, 4, 1107, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (357, 3, 2, 5, 1, 309, '单体电压109', 2, 0, 4, 1108, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (358, 3, 2, 5, 1, 310, '单体电压110', 2, 0, 4, 1109, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (359, 3, 2, 5, 1, 311, '单体电压111', 2, 0, 4, 1110, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (360, 3, 2, 5, 1, 312, '单体电压112', 2, 0, 4, 1111, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (361, 3, 2, 5, 1, 313, '单体电压113', 2, 0, 4, 1112, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (362, 3, 2, 5, 1, 314, '单体电压114', 2, 0, 4, 1113, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (363, 3, 2, 5, 1, 315, '单体电压115', 2, 0, 4, 1114, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (364, 3, 2, 5, 1, 316, '单体电压116', 2, 0, 4, 1115, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (365, 3, 2, 5, 1, 317, '单体电压117', 2, 0, 4, 1116, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (366, 3, 2, 5, 1, 318, '单体电压118', 2, 0, 4, 1117, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (367, 3, 2, 5, 1, 319, '单体电压119', 2, 0, 4, 1118, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (368, 3, 2, 5, 1, 320, '单体电压120', 2, 0, 4, 1119, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (369, 3, 2, 5, 1, 321, '单体电压121', 2, 0, 4, 1120, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (370, 3, 2, 5, 1, 322, '单体电压122', 2, 0, 4, 1121, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (371, 3, 2, 5, 1, 323, '单体电压123', 2, 0, 4, 1122, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (372, 3, 2, 5, 1, 324, '单体电压124', 2, 0, 4, 1123, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (373, 3, 2, 5, 1, 325, '单体电压125', 2, 0, 4, 1124, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (374, 3, 2, 5, 1, 326, '单体电压126', 2, 0, 4, 1125, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (375, 3, 2, 5, 1, 327, '单体电压127', 2, 0, 4, 1126, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (376, 3, 2, 5, 1, 328, '单体电压128', 2, 0, 4, 1127, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (377, 3, 2, 5, 1, 329, '单体电压129', 2, 0, 4, 1128, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (378, 3, 2, 5, 1, 330, '单体电压130', 2, 0, 4, 1129, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (379, 3, 2, 5, 1, 331, '单体电压131', 2, 0, 4, 1130, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (380, 3, 2, 5, 1, 332, '单体电压132', 2, 0, 4, 1131, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (381, 3, 2, 5, 1, 333, '单体电压133', 2, 0, 4, 1132, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (382, 3, 2, 5, 1, 334, '单体电压134', 2, 0, 4, 1133, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (383, 3, 2, 5, 1, 335, '单体电压135', 2, 0, 4, 1134, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (384, 3, 2, 5, 1, 336, '单体电压136', 2, 0, 4, 1135, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (385, 3, 2, 5, 1, 337, '单体电压137', 2, 0, 4, 1136, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (386, 3, 2, 5, 1, 338, '单体电压138', 2, 0, 4, 1137, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (387, 3, 2, 5, 1, 339, '单体电压139', 2, 0, 4, 1138, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (388, 3, 2, 5, 1, 340, '单体电压140', 2, 0, 4, 1139, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (389, 3, 2, 5, 1, 341, '单体电压141', 2, 0, 4, 1140, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (390, 3, 2, 5, 1, 342, '单体电压142', 2, 0, 4, 1141, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (391, 3, 2, 5, 1, 343, '单体电压143', 2, 0, 4, 1142, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (392, 3, 2, 5, 1, 344, '单体电压144', 2, 0, 4, 1143, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (393, 3, 2, 5, 1, 345, '单体电压145', 2, 0, 4, 1144, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (394, 3, 2, 5, 1, 346, '单体电压146', 2, 0, 4, 1145, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (395, 3, 2, 5, 1, 347, '单体电压147', 2, 0, 4, 1146, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (396, 3, 2, 5, 1, 348, '单体电压148', 2, 0, 4, 1147, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (397, 3, 2, 5, 1, 349, '单体电压149', 2, 0, 4, 1148, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (398, 3, 2, 5, 1, 350, '单体电压150', 2, 0, 4, 1149, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (399, 3, 2, 5, 1, 351, '单体电压151', 2, 0, 4, 1150, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (400, 3, 2, 5, 1, 352, '单体电压152', 2, 0, 4, 1151, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (401, 3, 2, 5, 1, 353, '单体电压153', 2, 0, 4, 1152, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (402, 3, 2, 5, 1, 354, '单体电压154', 2, 0, 4, 1153, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (403, 3, 2, 5, 1, 355, '单体电压155', 2, 0, 4, 1154, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (404, 3, 2, 5, 1, 356, '单体电压156', 2, 0, 4, 1155, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (405, 3, 2, 5, 1, 357, '单体电压157', 2, 0, 4, 1156, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (406, 3, 2, 5, 1, 358, '单体电压158', 2, 0, 4, 1157, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (407, 3, 2, 5, 1, 359, '单体电压159', 2, 0, 4, 1158, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (408, 3, 2, 5, 1, 360, '单体电压160', 2, 0, 4, 1159, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (409, 3, 2, 5, 1, 361, '单体电压161', 2, 0, 4, 1160, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (410, 3, 2, 5, 1, 362, '单体电压162', 2, 0, 4, 1161, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (411, 3, 2, 5, 1, 363, '单体电压163', 2, 0, 4, 1162, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (412, 3, 2, 5, 1, 364, '单体电压164', 2, 0, 4, 1163, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (413, 3, 2, 5, 1, 365, '单体电压165', 2, 0, 4, 1164, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (414, 3, 2, 5, 1, 366, '单体电压166', 2, 0, 4, 1165, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (415, 3, 2, 5, 1, 367, '单体电压167', 2, 0, 4, 1166, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (416, 3, 2, 5, 1, 368, '单体电压168', 2, 0, 4, 1167, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (417, 3, 2, 5, 1, 369, '单体电压169', 2, 0, 4, 1168, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (418, 3, 2, 5, 1, 370, '单体电压170', 2, 0, 4, 1169, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (419, 3, 2, 5, 1, 371, '单体电压171', 2, 0, 4, 1170, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (420, 3, 2, 5, 1, 372, '单体电压172', 2, 0, 4, 1171, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (421, 3, 2, 5, 1, 373, '单体电压173', 2, 0, 4, 1172, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (422, 3, 2, 5, 1, 374, '单体电压174', 2, 0, 4, 1173, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (423, 3, 2, 5, 1, 375, '单体电压175', 2, 0, 4, 1174, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (424, 3, 2, 5, 1, 376, '单体电压176', 2, 0, 4, 1175, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (425, 3, 2, 5, 1, 377, '单体电压177', 2, 0, 4, 1176, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (426, 3, 2, 5, 1, 378, '单体电压178', 2, 0, 4, 1177, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (427, 3, 2, 5, 1, 379, '单体电压179', 2, 0, 4, 1178, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (428, 3, 2, 5, 1, 380, '单体电压180', 2, 0, 4, 1179, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (429, 3, 2, 5, 1, 381, '单体电压181', 2, 0, 4, 1180, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (430, 3, 2, 5, 1, 382, '单体电压182', 2, 0, 4, 1181, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (431, 3, 2, 5, 1, 383, '单体电压183', 2, 0, 4, 1182, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (432, 3, 2, 5, 1, 384, '单体电压184', 2, 0, 4, 1183, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (433, 3, 2, 5, 1, 385, '单体电压185', 2, 0, 4, 1184, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (434, 3, 2, 5, 1, 386, '单体电压186', 2, 0, 4, 1185, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (435, 3, 2, 5, 1, 387, '单体电压187', 2, 0, 4, 1186, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (436, 3, 2, 5, 1, 388, '单体电压188', 2, 0, 4, 1187, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (437, 3, 2, 5, 1, 389, '单体电压189', 2, 0, 4, 1188, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (438, 3, 2, 5, 1, 390, '单体电压190', 2, 0, 4, 1189, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (439, 3, 2, 5, 1, 391, '单体电压191', 2, 0, 4, 1190, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (440, 3, 2, 5, 1, 392, '单体电压192', 2, 0, 4, 1191, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (441, 3, 2, 5, 1, 393, '单体电压193', 2, 0, 4, 1192, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (442, 3, 2, 5, 1, 394, '单体电压194', 2, 0, 4, 1193, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (443, 3, 2, 5, 1, 395, '单体电压195', 2, 0, 4, 1194, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (444, 3, 2, 5, 1, 396, '单体电压196', 2, 0, 4, 1195, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (445, 3, 2, 5, 1, 397, '单体电压197', 2, 0, 4, 1196, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (446, 3, 2, 5, 1, 398, '单体电压198', 2, 0, 4, 1197, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (447, 3, 2, 5, 1, 399, '单体电压199', 2, 0, 4, 1198, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (448, 3, 2, 5, 1, 400, '单体电压200', 2, 0, 4, 1199, 0, 0, 0, 0.01, 1.0, 0.0, 1, 5000, 1.0, 0.0, 1.0, 4.5, 'V', 1);
INSERT INTO "template_point" VALUES (449, 3, 2, 5, 1, 651, '单体SOC1', 2, 0, 4, 3000, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (450, 3, 2, 5, 1, 652, '单体SOC2', 2, 0, 4, 3001, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (451, 3, 2, 5, 1, 653, '单体SOC3', 2, 0, 4, 3002, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (452, 3, 2, 5, 1, 654, '单体SOC4', 2, 0, 4, 3003, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (453, 3, 2, 5, 1, 655, '单体SOC5', 2, 0, 4, 3004, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (454, 3, 2, 5, 1, 656, '单体SOC6', 2, 0, 4, 3005, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (455, 3, 2, 5, 1, 657, '单体SOC7', 2, 0, 4, 3006, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (456, 3, 2, 5, 1, 658, '单体SOC8', 2, 0, 4, 3007, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (457, 3, 2, 5, 1, 659, '单体SOC9', 2, 0, 4, 3008, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (458, 3, 2, 5, 1, 660, '单体SOC10', 2, 0, 4, 3009, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (459, 3, 2, 5, 1, 661, '单体SOC11', 2, 0, 4, 3010, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (460, 3, 2, 5, 1, 662, '单体SOC12', 2, 0, 4, 3011, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (461, 3, 2, 5, 1, 663, '单体SOC13', 2, 0, 4, 3012, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (462, 3, 2, 5, 1, 664, '单体SOC14', 2, 0, 4, 3013, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (463, 3, 2, 5, 1, 665, '单体SOC15', 2, 0, 4, 3014, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (464, 3, 2, 5, 1, 666, '单体SOC16', 2, 0, 4, 3015, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (465, 3, 2, 5, 1, 667, '单体SOC17', 2, 0, 4, 3016, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (466, 3, 2, 5, 1, 668, '单体SOC18', 2, 0, 4, 3017, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (467, 3, 2, 5, 1, 669, '单体SOC19', 2, 0, 4, 3018, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (468, 3, 2, 5, 1, 670, '单体SOC20', 2, 0, 4, 3019, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (469, 3, 2, 5, 1, 671, '单体SOC21', 2, 0, 4, 3020, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (470, 3, 2, 5, 1, 672, '单体SOC22', 2, 0, 4, 3021, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (471, 3, 2, 5, 1, 673, '单体SOC23', 2, 0, 4, 3022, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (472, 3, 2, 5, 1, 674, '单体SOC24', 2, 0, 4, 3023, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (473, 3, 2, 5, 1, 675, '单体SOC25', 2, 0, 4, 3024, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (474, 3, 2, 5, 1, 676, '单体SOC26', 2, 0, 4, 3025, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (475, 3, 2, 5, 1, 677, '单体SOC27', 2, 0, 4, 3026, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (476, 3, 2, 5, 1, 678, '单体SOC28', 2, 0, 4, 3027, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (477, 3, 2, 5, 1, 679, '单体SOC29', 2, 0, 4, 3028, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (478, 3, 2, 5, 1, 680, '单体SOC30', 2, 0, 4, 3029, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (479, 3, 2, 5, 1, 681, '单体SOC31', 2, 0, 4, 3030, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (480, 3, 2, 5, 1, 682, '单体SOC32', 2, 0, 4, 3031, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (481, 3, 2, 5, 1, 683, '单体SOC33', 2, 0, 4, 3032, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (482, 3, 2, 5, 1, 684, '单体SOC34', 2, 0, 4, 3033, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (483, 3, 2, 5, 1, 685, '单体SOC35', 2, 0, 4, 3034, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (484, 3, 2, 5, 1, 686, '单体SOC36', 2, 0, 4, 3035, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (485, 3, 2, 5, 1, 687, '单体SOC37', 2, 0, 4, 3036, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (486, 3, 2, 5, 1, 688, '单体SOC38', 2, 0, 4, 3037, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (487, 3, 2, 5, 1, 689, '单体SOC39', 2, 0, 4, 3038, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (488, 3, 2, 5, 1, 690, '单体SOC40', 2, 0, 4, 3039, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (489, 3, 2, 5, 1, 691, '单体SOC41', 2, 0, 4, 3040, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (490, 3, 2, 5, 1, 692, '单体SOC42', 2, 0, 4, 3041, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (491, 3, 2, 5, 1, 693, '单体SOC43', 2, 0, 4, 3042, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (492, 3, 2, 5, 1, 694, '单体SOC44', 2, 0, 4, 3043, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (493, 3, 2, 5, 1, 695, '单体SOC45', 2, 0, 4, 3044, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (494, 3, 2, 5, 1, 696, '单体SOC46', 2, 0, 4, 3045, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (495, 3, 2, 5, 1, 697, '单体SOC47', 2, 0, 4, 3046, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (496, 3, 2, 5, 1, 698, '单体SOC48', 2, 0, 4, 3047, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (497, 3, 2, 5, 1, 699, '单体SOC49', 2, 0, 4, 3048, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (498, 3, 2, 5, 1, 700, '单体SOC50', 2, 0, 4, 3049, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (499, 3, 2, 5, 1, 701, '单体SOC51', 2, 0, 4, 3050, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (500, 3, 2, 5, 1, 702, '单体SOC52', 2, 0, 4, 3051, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (501, 3, 2, 5, 1, 703, '单体SOC53', 2, 0, 4, 3052, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (502, 3, 2, 5, 1, 704, '单体SOC54', 2, 0, 4, 3053, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (503, 3, 2, 5, 1, 705, '单体SOC55', 2, 0, 4, 3054, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (504, 3, 2, 5, 1, 706, '单体SOC56', 2, 0, 4, 3055, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (505, 3, 2, 5, 1, 707, '单体SOC57', 2, 0, 4, 3056, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (506, 3, 2, 5, 1, 708, '单体SOC58', 2, 0, 4, 3057, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (507, 3, 2, 5, 1, 709, '单体SOC59', 2, 0, 4, 3058, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (508, 3, 2, 5, 1, 710, '单体SOC60', 2, 0, 4, 3059, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (509, 3, 2, 5, 1, 711, '单体SOC61', 2, 0, 4, 3060, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (510, 3, 2, 5, 1, 712, '单体SOC62', 2, 0, 4, 3061, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (511, 3, 2, 5, 1, 713, '单体SOC63', 2, 0, 4, 3062, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (512, 3, 2, 5, 1, 714, '单体SOC64', 2, 0, 4, 3063, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (513, 3, 2, 5, 1, 715, '单体SOC65', 2, 0, 4, 3064, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (514, 3, 2, 5, 1, 716, '单体SOC66', 2, 0, 4, 3065, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (515, 3, 2, 5, 1, 717, '单体SOC67', 2, 0, 4, 3066, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (516, 3, 2, 5, 1, 718, '单体SOC68', 2, 0, 4, 3067, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (517, 3, 2, 5, 1, 719, '单体SOC69', 2, 0, 4, 3068, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (518, 3, 2, 5, 1, 720, '单体SOC70', 2, 0, 4, 3069, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (519, 3, 2, 5, 1, 721, '单体SOC71', 2, 0, 4, 3070, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (520, 3, 2, 5, 1, 722, '单体SOC72', 2, 0, 4, 3071, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (521, 3, 2, 5, 1, 723, '单体SOC73', 2, 0, 4, 3072, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (522, 3, 2, 5, 1, 724, '单体SOC74', 2, 0, 4, 3073, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (523, 3, 2, 5, 1, 725, '单体SOC75', 2, 0, 4, 3074, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (524, 3, 2, 5, 1, 726, '单体SOC76', 2, 0, 4, 3075, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (525, 3, 2, 5, 1, 727, '单体SOC77', 2, 0, 4, 3076, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (526, 3, 2, 5, 1, 728, '单体SOC78', 2, 0, 4, 3077, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (527, 3, 2, 5, 1, 729, '单体SOC79', 2, 0, 4, 3078, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (528, 3, 2, 5, 1, 730, '单体SOC80', 2, 0, 4, 3079, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (529, 3, 2, 5, 1, 731, '单体SOC81', 2, 0, 4, 3080, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (530, 3, 2, 5, 1, 732, '单体SOC82', 2, 0, 4, 3081, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (531, 3, 2, 5, 1, 733, '单体SOC83', 2, 0, 4, 3082, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (532, 3, 2, 5, 1, 734, '单体SOC84', 2, 0, 4, 3083, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (533, 3, 2, 5, 1, 735, '单体SOC85', 2, 0, 4, 3084, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (534, 3, 2, 5, 1, 736, '单体SOC86', 2, 0, 4, 3085, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (535, 3, 2, 5, 1, 737, '单体SOC87', 2, 0, 4, 3086, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (536, 3, 2, 5, 1, 738, '单体SOC88', 2, 0, 4, 3087, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (537, 3, 2, 5, 1, 739, '单体SOC89', 2, 0, 4, 3088, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (538, 3, 2, 5, 1, 740, '单体SOC90', 2, 0, 4, 3089, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (539, 3, 2, 5, 1, 741, '单体SOC91', 2, 0, 4, 3090, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (540, 3, 2, 5, 1, 742, '单体SOC92', 2, 0, 4, 3091, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (541, 3, 2, 5, 1, 743, '单体SOC93', 2, 0, 4, 3092, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (542, 3, 2, 5, 1, 744, '单体SOC94', 2, 0, 4, 3093, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (543, 3, 2, 5, 1, 745, '单体SOC95', 2, 0, 4, 3094, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (544, 3, 2, 5, 1, 746, '单体SOC96', 2, 0, 4, 3095, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (545, 3, 2, 5, 1, 747, '单体SOC97', 2, 0, 4, 3096, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (546, 3, 2, 5, 1, 748, '单体SOC98', 2, 0, 4, 3097, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (547, 3, 2, 5, 1, 749, '单体SOC99', 2, 0, 4, 3098, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (548, 3, 2, 5, 1, 750, '单体SOC100', 2, 0, 4, 3099, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (549, 3, 2, 5, 1, 751, '单体SOC101', 2, 0, 4, 3100, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (550, 3, 2, 5, 1, 752, '单体SOC102', 2, 0, 4, 3101, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (551, 3, 2, 5, 1, 753, '单体SOC103', 2, 0, 4, 3102, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (552, 3, 2, 5, 1, 754, '单体SOC104', 2, 0, 4, 3103, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (553, 3, 2, 5, 1, 755, '单体SOC105', 2, 0, 4, 3104, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (554, 3, 2, 5, 1, 756, '单体SOC106', 2, 0, 4, 3105, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (555, 3, 2, 5, 1, 757, '单体SOC107', 2, 0, 4, 3106, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (556, 3, 2, 5, 1, 758, '单体SOC108', 2, 0, 4, 3107, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (557, 3, 2, 5, 1, 759, '单体SOC109', 2, 0, 4, 3108, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (558, 3, 2, 5, 1, 760, '单体SOC110', 2, 0, 4, 3109, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (559, 3, 2, 5, 1, 761, '单体SOC111', 2, 0, 4, 3110, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (560, 3, 2, 5, 1, 762, '单体SOC112', 2, 0, 4, 3111, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (561, 3, 2, 5, 1, 763, '单体SOC113', 2, 0, 4, 3112, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (562, 3, 2, 5, 1, 764, '单体SOC114', 2, 0, 4, 3113, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (563, 3, 2, 5, 1, 765, '单体SOC115', 2, 0, 4, 3114, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (564, 3, 2, 5, 1, 766, '单体SOC116', 2, 0, 4, 3115, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (565, 3, 2, 5, 1, 767, '单体SOC117', 2, 0, 4, 3116, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (566, 3, 2, 5, 1, 768, '单体SOC118', 2, 0, 4, 3117, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (567, 3, 2, 5, 1, 769, '单体SOC119', 2, 0, 4, 3118, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (568, 3, 2, 5, 1, 770, '单体SOC120', 2, 0, 4, 3119, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (569, 3, 2, 5, 1, 771, '单体SOC121', 2, 0, 4, 3120, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (570, 3, 2, 5, 1, 772, '单体SOC122', 2, 0, 4, 3121, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (571, 3, 2, 5, 1, 773, '单体SOC123', 2, 0, 4, 3122, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (572, 3, 2, 5, 1, 774, '单体SOC124', 2, 0, 4, 3123, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (573, 3, 2, 5, 1, 775, '单体SOC125', 2, 0, 4, 3124, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (574, 3, 2, 5, 1, 776, '单体SOC126', 2, 0, 4, 3125, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (575, 3, 2, 5, 1, 777, '单体SOC127', 2, 0, 4, 3126, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (576, 3, 2, 5, 1, 778, '单体SOC128', 2, 0, 4, 3127, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (577, 3, 2, 5, 1, 779, '单体SOC129', 2, 0, 4, 3128, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (578, 3, 2, 5, 1, 780, '单体SOC130', 2, 0, 4, 3129, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (579, 3, 2, 5, 1, 781, '单体SOC131', 2, 0, 4, 3130, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (580, 3, 2, 5, 1, 782, '单体SOC132', 2, 0, 4, 3131, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (581, 3, 2, 5, 1, 783, '单体SOC133', 2, 0, 4, 3132, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (582, 3, 2, 5, 1, 784, '单体SOC134', 2, 0, 4, 3133, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (583, 3, 2, 5, 1, 785, '单体SOC135', 2, 0, 4, 3134, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (584, 3, 2, 5, 1, 786, '单体SOC136', 2, 0, 4, 3135, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (585, 3, 2, 5, 1, 787, '单体SOC137', 2, 0, 4, 3136, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (586, 3, 2, 5, 1, 788, '单体SOC138', 2, 0, 4, 3137, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (587, 3, 2, 5, 1, 789, '单体SOC139', 2, 0, 4, 3138, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (588, 3, 2, 5, 1, 790, '单体SOC140', 2, 0, 4, 3139, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (589, 3, 2, 5, 1, 791, '单体SOC141', 2, 0, 4, 3140, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (590, 3, 2, 5, 1, 792, '单体SOC142', 2, 0, 4, 3141, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (591, 3, 2, 5, 1, 793, '单体SOC143', 2, 0, 4, 3142, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (592, 3, 2, 5, 1, 794, '单体SOC144', 2, 0, 4, 3143, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (593, 3, 2, 5, 1, 795, '单体SOC145', 2, 0, 4, 3144, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (594, 3, 2, 5, 1, 796, '单体SOC146', 2, 0, 4, 3145, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (595, 3, 2, 5, 1, 797, '单体SOC147', 2, 0, 4, 3146, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (596, 3, 2, 5, 1, 798, '单体SOC148', 2, 0, 4, 3147, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (597, 3, 2, 5, 1, 799, '单体SOC149', 2, 0, 4, 3148, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (598, 3, 2, 5, 1, 800, '单体SOC150', 2, 0, 4, 3149, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (599, 3, 2, 5, 1, 801, '单体SOC151', 2, 0, 4, 3150, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (600, 3, 2, 5, 1, 802, '单体SOC152', 2, 0, 4, 3151, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (601, 3, 2, 5, 1, 803, '单体SOC153', 2, 0, 4, 3152, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (602, 3, 2, 5, 1, 804, '单体SOC154', 2, 0, 4, 3153, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (603, 3, 2, 5, 1, 805, '单体SOC155', 2, 0, 4, 3154, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (604, 3, 2, 5, 1, 806, '单体SOC156', 2, 0, 4, 3155, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (605, 3, 2, 5, 1, 807, '单体SOC157', 2, 0, 4, 3156, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (606, 3, 2, 5, 1, 808, '单体SOC158', 2, 0, 4, 3157, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (607, 3, 2, 5, 1, 809, '单体SOC159', 2, 0, 4, 3158, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (608, 3, 2, 5, 1, 810, '单体SOC160', 2, 0, 4, 3159, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (609, 3, 2, 5, 1, 811, '单体SOC161', 2, 0, 4, 3160, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (610, 3, 2, 5, 1, 812, '单体SOC162', 2, 0, 4, 3161, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (611, 3, 2, 5, 1, 813, '单体SOC163', 2, 0, 4, 3162, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (612, 3, 2, 5, 1, 814, '单体SOC164', 2, 0, 4, 3163, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (613, 3, 2, 5, 1, 815, '单体SOC165', 2, 0, 4, 3164, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (614, 3, 2, 5, 1, 816, '单体SOC166', 2, 0, 4, 3165, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (615, 3, 2, 5, 1, 817, '单体SOC167', 2, 0, 4, 3166, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (616, 3, 2, 5, 1, 818, '单体SOC168', 2, 0, 4, 3167, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (617, 3, 2, 5, 1, 819, '单体SOC169', 2, 0, 4, 3168, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (618, 3, 2, 5, 1, 820, '单体SOC170', 2, 0, 4, 3169, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (619, 3, 2, 5, 1, 821, '单体SOC171', 2, 0, 4, 3170, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (620, 3, 2, 5, 1, 822, '单体SOC172', 2, 0, 4, 3171, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (621, 3, 2, 5, 1, 823, '单体SOC173', 2, 0, 4, 3172, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (622, 3, 2, 5, 1, 824, '单体SOC174', 2, 0, 4, 3173, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (623, 3, 2, 5, 1, 825, '单体SOC175', 2, 0, 4, 3174, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (624, 3, 2, 5, 1, 826, '单体SOC176', 2, 0, 4, 3175, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (625, 3, 2, 5, 1, 827, '单体SOC177', 2, 0, 4, 3176, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (626, 3, 2, 5, 1, 828, '单体SOC178', 2, 0, 4, 3177, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (627, 3, 2, 5, 1, 829, '单体SOC179', 2, 0, 4, 3178, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (628, 3, 2, 5, 1, 830, '单体SOC180', 2, 0, 4, 3179, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (629, 3, 2, 5, 1, 831, '单体SOC181', 2, 0, 4, 3180, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (630, 3, 2, 5, 1, 832, '单体SOC182', 2, 0, 4, 3181, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (631, 3, 2, 5, 1, 833, '单体SOC183', 2, 0, 4, 3182, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (632, 3, 2, 5, 1, 834, '单体SOC184', 2, 0, 4, 3183, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (633, 3, 2, 5, 1, 835, '单体SOC185', 2, 0, 4, 3184, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (634, 3, 2, 5, 1, 836, '单体SOC186', 2, 0, 4, 3185, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (635, 3, 2, 5, 1, 837, '单体SOC187', 2, 0, 4, 3186, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (636, 3, 2, 5, 1, 838, '单体SOC188', 2, 0, 4, 3187, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (637, 3, 2, 5, 1, 839, '单体SOC189', 2, 0, 4, 3188, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (638, 3, 2, 5, 1, 840, '单体SOC190', 2, 0, 4, 3189, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (639, 3, 2, 5, 1, 841, '单体SOC191', 2, 0, 4, 3190, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (640, 3, 2, 5, 1, 842, '单体SOC192', 2, 0, 4, 3191, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (641, 3, 2, 5, 1, 843, '单体SOC193', 2, 0, 4, 3192, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (642, 3, 2, 5, 1, 844, '单体SOC194', 2, 0, 4, 3193, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (643, 3, 2, 5, 1, 845, '单体SOC195', 2, 0, 4, 3194, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (644, 3, 2, 5, 1, 846, '单体SOC196', 2, 0, 4, 3195, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (645, 3, 2, 5, 1, 847, '单体SOC197', 2, 0, 4, 3196, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (646, 3, 2, 5, 1, 848, '单体SOC198', 2, 0, 4, 3197, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (647, 3, 2, 5, 1, 849, '单体SOC199', 2, 0, 4, 3198, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (648, 3, 2, 5, 1, 850, '单体SOC200', 2, 0, 4, 3199, 0, 0, 0, 0.1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (649, 3, 2, 5, 1, 1101, '单体SOH1', 2, 0, 4, 4000, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (650, 3, 2, 5, 1, 1102, '单体SOH2', 2, 0, 4, 4001, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (651, 3, 2, 5, 1, 1103, '单体SOH3', 2, 0, 4, 4002, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (652, 3, 2, 5, 1, 1104, '单体SOH4', 2, 0, 4, 4003, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (653, 3, 2, 5, 1, 1105, '单体SOH5', 2, 0, 4, 4004, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (654, 3, 2, 5, 1, 1106, '单体SOH6', 2, 0, 4, 4005, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (655, 3, 2, 5, 1, 1107, '单体SOH7', 2, 0, 4, 4006, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (656, 3, 2, 5, 1, 1108, '单体SOH8', 2, 0, 4, 4007, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (657, 3, 2, 5, 1, 1109, '单体SOH9', 2, 0, 4, 4008, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (658, 3, 2, 5, 1, 1110, '单体SOH10', 2, 0, 4, 4009, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (659, 3, 2, 5, 1, 1111, '单体SOH11', 2, 0, 4, 4010, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (660, 3, 2, 5, 1, 1112, '单体SOH12', 2, 0, 4, 4011, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (661, 3, 2, 5, 1, 1113, '单体SOH13', 2, 0, 4, 4012, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (662, 3, 2, 5, 1, 1114, '单体SOH14', 2, 0, 4, 4013, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (663, 3, 2, 5, 1, 1115, '单体SOH15', 2, 0, 4, 4014, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (664, 3, 2, 5, 1, 1116, '单体SOH16', 2, 0, 4, 4015, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (665, 3, 2, 5, 1, 1117, '单体SOH17', 2, 0, 4, 4016, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (666, 3, 2, 5, 1, 1118, '单体SOH18', 2, 0, 4, 4017, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (667, 3, 2, 5, 1, 1119, '单体SOH19', 2, 0, 4, 4018, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (668, 3, 2, 5, 1, 1120, '单体SOH20', 2, 0, 4, 4019, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (669, 3, 2, 5, 1, 1121, '单体SOH21', 2, 0, 4, 4020, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (670, 3, 2, 5, 1, 1122, '单体SOH22', 2, 0, 4, 4021, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (671, 3, 2, 5, 1, 1123, '单体SOH23', 2, 0, 4, 4022, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (672, 3, 2, 5, 1, 1124, '单体SOH24', 2, 0, 4, 4023, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (673, 3, 2, 5, 1, 1125, '单体SOH25', 2, 0, 4, 4024, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (674, 3, 2, 5, 1, 1126, '单体SOH26', 2, 0, 4, 4025, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (675, 3, 2, 5, 1, 1127, '单体SOH27', 2, 0, 4, 4026, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (676, 3, 2, 5, 1, 1128, '单体SOH28', 2, 0, 4, 4027, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (677, 3, 2, 5, 1, 1129, '单体SOH29', 2, 0, 4, 4028, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (678, 3, 2, 5, 1, 1130, '单体SOH30', 2, 0, 4, 4029, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (679, 3, 2, 5, 1, 1131, '单体SOH31', 2, 0, 4, 4030, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (680, 3, 2, 5, 1, 1132, '单体SOH32', 2, 0, 4, 4031, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (681, 3, 2, 5, 1, 1133, '单体SOH33', 2, 0, 4, 4032, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (682, 3, 2, 5, 1, 1134, '单体SOH34', 2, 0, 4, 4033, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (683, 3, 2, 5, 1, 1135, '单体SOH35', 2, 0, 4, 4034, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (684, 3, 2, 5, 1, 1136, '单体SOH36', 2, 0, 4, 4035, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (685, 3, 2, 5, 1, 1137, '单体SOH37', 2, 0, 4, 4036, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (686, 3, 2, 5, 1, 1138, '单体SOH38', 2, 0, 4, 4037, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (687, 3, 2, 5, 1, 1139, '单体SOH39', 2, 0, 4, 4038, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (688, 3, 2, 5, 1, 1140, '单体SOH40', 2, 0, 4, 4039, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (689, 3, 2, 5, 1, 1141, '单体SOH41', 2, 0, 4, 4040, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (690, 3, 2, 5, 1, 1142, '单体SOH42', 2, 0, 4, 4041, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (691, 3, 2, 5, 1, 1143, '单体SOH43', 2, 0, 4, 4042, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (692, 3, 2, 5, 1, 1144, '单体SOH44', 2, 0, 4, 4043, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (693, 3, 2, 5, 1, 1145, '单体SOH45', 2, 0, 4, 4044, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (694, 3, 2, 5, 1, 1146, '单体SOH46', 2, 0, 4, 4045, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (695, 3, 2, 5, 1, 1147, '单体SOH47', 2, 0, 4, 4046, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (696, 3, 2, 5, 1, 1148, '单体SOH48', 2, 0, 4, 4047, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (697, 3, 2, 5, 1, 1149, '单体SOH49', 2, 0, 4, 4048, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (698, 3, 2, 5, 1, 1150, '单体SOH50', 2, 0, 4, 4049, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (699, 3, 2, 5, 1, 1151, '单体SOH51', 2, 0, 4, 4050, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (700, 3, 2, 5, 1, 1152, '单体SOH52', 2, 0, 4, 4051, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (701, 3, 2, 5, 1, 1153, '单体SOH53', 2, 0, 4, 4052, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (702, 3, 2, 5, 1, 1154, '单体SOH54', 2, 0, 4, 4053, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (703, 3, 2, 5, 1, 1155, '单体SOH55', 2, 0, 4, 4054, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (704, 3, 2, 5, 1, 1156, '单体SOH56', 2, 0, 4, 4055, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (705, 3, 2, 5, 1, 1157, '单体SOH57', 2, 0, 4, 4056, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (706, 3, 2, 5, 1, 1158, '单体SOH58', 2, 0, 4, 4057, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (707, 3, 2, 5, 1, 1159, '单体SOH59', 2, 0, 4, 4058, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (708, 3, 2, 5, 1, 1160, '单体SOH60', 2, 0, 4, 4059, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (709, 3, 2, 5, 1, 1161, '单体SOH61', 2, 0, 4, 4060, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (710, 3, 2, 5, 1, 1162, '单体SOH62', 2, 0, 4, 4061, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (711, 3, 2, 5, 1, 1163, '单体SOH63', 2, 0, 4, 4062, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (712, 3, 2, 5, 1, 1164, '单体SOH64', 2, 0, 4, 4063, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (713, 3, 2, 5, 1, 1165, '单体SOH65', 2, 0, 4, 4064, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (714, 3, 2, 5, 1, 1166, '单体SOH66', 2, 0, 4, 4065, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (715, 3, 2, 5, 1, 1167, '单体SOH67', 2, 0, 4, 4066, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (716, 3, 2, 5, 1, 1168, '单体SOH68', 2, 0, 4, 4067, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (717, 3, 2, 5, 1, 1169, '单体SOH69', 2, 0, 4, 4068, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (718, 3, 2, 5, 1, 1170, '单体SOH70', 2, 0, 4, 4069, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (719, 3, 2, 5, 1, 1171, '单体SOH71', 2, 0, 4, 4070, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (720, 3, 2, 5, 1, 1172, '单体SOH72', 2, 0, 4, 4071, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (721, 3, 2, 5, 1, 1173, '单体SOH73', 2, 0, 4, 4072, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (722, 3, 2, 5, 1, 1174, '单体SOH74', 2, 0, 4, 4073, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (723, 3, 2, 5, 1, 1175, '单体SOH75', 2, 0, 4, 4074, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (724, 3, 2, 5, 1, 1176, '单体SOH76', 2, 0, 4, 4075, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (725, 3, 2, 5, 1, 1177, '单体SOH77', 2, 0, 4, 4076, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (726, 3, 2, 5, 1, 1178, '单体SOH78', 2, 0, 4, 4077, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (727, 3, 2, 5, 1, 1179, '单体SOH79', 2, 0, 4, 4078, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (728, 3, 2, 5, 1, 1180, '单体SOH80', 2, 0, 4, 4079, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (729, 3, 2, 5, 1, 1181, '单体SOH81', 2, 0, 4, 4080, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (730, 3, 2, 5, 1, 1182, '单体SOH82', 2, 0, 4, 4081, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (731, 3, 2, 5, 1, 1183, '单体SOH83', 2, 0, 4, 4082, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (732, 3, 2, 5, 1, 1184, '单体SOH84', 2, 0, 4, 4083, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (733, 3, 2, 5, 1, 1185, '单体SOH85', 2, 0, 4, 4084, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (734, 3, 2, 5, 1, 1186, '单体SOH86', 2, 0, 4, 4085, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (735, 3, 2, 5, 1, 1187, '单体SOH87', 2, 0, 4, 4086, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (736, 3, 2, 5, 1, 1188, '单体SOH88', 2, 0, 4, 4087, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (737, 3, 2, 5, 1, 1189, '单体SOH89', 2, 0, 4, 4088, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (738, 3, 2, 5, 1, 1190, '单体SOH90', 2, 0, 4, 4089, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (739, 3, 2, 5, 1, 1191, '单体SOH91', 2, 0, 4, 4090, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (740, 3, 2, 5, 1, 1192, '单体SOH92', 2, 0, 4, 4091, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (741, 3, 2, 5, 1, 1193, '单体SOH93', 2, 0, 4, 4092, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (742, 3, 2, 5, 1, 1194, '单体SOH94', 2, 0, 4, 4093, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (743, 3, 2, 5, 1, 1195, '单体SOH95', 2, 0, 4, 4094, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (744, 3, 2, 5, 1, 1196, '单体SOH96', 2, 0, 4, 4095, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (745, 3, 2, 5, 1, 1197, '单体SOH97', 2, 0, 4, 4096, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (746, 3, 2, 5, 1, 1198, '单体SOH98', 2, 0, 4, 4097, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (747, 3, 2, 5, 1, 1199, '单体SOH99', 2, 0, 4, 4098, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (748, 3, 2, 5, 1, 1200, '单体SOH100', 2, 0, 4, 4099, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (749, 3, 2, 5, 1, 1201, '单体SOH101', 2, 0, 4, 4100, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (750, 3, 2, 5, 1, 1202, '单体SOH102', 2, 0, 4, 4101, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (751, 3, 2, 5, 1, 1203, '单体SOH103', 2, 0, 4, 4102, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (752, 3, 2, 5, 1, 1204, '单体SOH104', 2, 0, 4, 4103, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (753, 3, 2, 5, 1, 1205, '单体SOH105', 2, 0, 4, 4104, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (754, 3, 2, 5, 1, 1206, '单体SOH106', 2, 0, 4, 4105, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (755, 3, 2, 5, 1, 1207, '单体SOH107', 2, 0, 4, 4106, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (756, 3, 2, 5, 1, 1208, '单体SOH108', 2, 0, 4, 4107, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (757, 3, 2, 5, 1, 1209, '单体SOH109', 2, 0, 4, 4108, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (758, 3, 2, 5, 1, 1210, '单体SOH110', 2, 0, 4, 4109, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (759, 3, 2, 5, 1, 1211, '单体SOH111', 2, 0, 4, 4110, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (760, 3, 2, 5, 1, 1212, '单体SOH112', 2, 0, 4, 4111, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (761, 3, 2, 5, 1, 1213, '单体SOH113', 2, 0, 4, 4112, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (762, 3, 2, 5, 1, 1214, '单体SOH114', 2, 0, 4, 4113, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (763, 3, 2, 5, 1, 1215, '单体SOH115', 2, 0, 4, 4114, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (764, 3, 2, 5, 1, 1216, '单体SOH116', 2, 0, 4, 4115, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (765, 3, 2, 5, 1, 1217, '单体SOH117', 2, 0, 4, 4116, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (766, 3, 2, 5, 1, 1218, '单体SOH118', 2, 0, 4, 4117, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (767, 3, 2, 5, 1, 1219, '单体SOH119', 2, 0, 4, 4118, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (768, 3, 2, 5, 1, 1220, '单体SOH120', 2, 0, 4, 4119, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (769, 3, 2, 5, 1, 1221, '单体SOH121', 2, 0, 4, 4120, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (770, 3, 2, 5, 1, 1222, '单体SOH122', 2, 0, 4, 4121, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (771, 3, 2, 5, 1, 1223, '单体SOH123', 2, 0, 4, 4122, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (772, 3, 2, 5, 1, 1224, '单体SOH124', 2, 0, 4, 4123, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (773, 3, 2, 5, 1, 1225, '单体SOH125', 2, 0, 4, 4124, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (774, 3, 2, 5, 1, 1226, '单体SOH126', 2, 0, 4, 4125, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (775, 3, 2, 5, 1, 1227, '单体SOH127', 2, 0, 4, 4126, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (776, 3, 2, 5, 1, 1228, '单体SOH128', 2, 0, 4, 4127, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (777, 3, 2, 5, 1, 1229, '单体SOH129', 2, 0, 4, 4128, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (778, 3, 2, 5, 1, 1230, '单体SOH130', 2, 0, 4, 4129, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (779, 3, 2, 5, 1, 1231, '单体SOH131', 2, 0, 4, 4130, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (780, 3, 2, 5, 1, 1232, '单体SOH132', 2, 0, 4, 4131, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (781, 3, 2, 5, 1, 1233, '单体SOH133', 2, 0, 4, 4132, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (782, 3, 2, 5, 1, 1234, '单体SOH134', 2, 0, 4, 4133, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (783, 3, 2, 5, 1, 1235, '单体SOH135', 2, 0, 4, 4134, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (784, 3, 2, 5, 1, 1236, '单体SOH136', 2, 0, 4, 4135, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (785, 3, 2, 5, 1, 1237, '单体SOH137', 2, 0, 4, 4136, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (786, 3, 2, 5, 1, 1238, '单体SOH138', 2, 0, 4, 4137, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (787, 3, 2, 5, 1, 1239, '单体SOH139', 2, 0, 4, 4138, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (788, 3, 2, 5, 1, 1240, '单体SOH140', 2, 0, 4, 4139, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (789, 3, 2, 5, 1, 1241, '单体SOH141', 2, 0, 4, 4140, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (790, 3, 2, 5, 1, 1242, '单体SOH142', 2, 0, 4, 4141, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (791, 3, 2, 5, 1, 1243, '单体SOH143', 2, 0, 4, 4142, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (792, 3, 2, 5, 1, 1244, '单体SOH144', 2, 0, 4, 4143, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (793, 3, 2, 5, 1, 1245, '单体SOH145', 2, 0, 4, 4144, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (794, 3, 2, 5, 1, 1246, '单体SOH146', 2, 0, 4, 4145, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (795, 3, 2, 5, 1, 1247, '单体SOH147', 2, 0, 4, 4146, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (796, 3, 2, 5, 1, 1248, '单体SOH148', 2, 0, 4, 4147, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (797, 3, 2, 5, 1, 1249, '单体SOH149', 2, 0, 4, 4148, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (798, 3, 2, 5, 1, 1250, '单体SOH150', 2, 0, 4, 4149, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (799, 3, 2, 5, 1, 1251, '单体SOH151', 2, 0, 4, 4150, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (800, 3, 2, 5, 1, 1252, '单体SOH152', 2, 0, 4, 4151, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (801, 3, 2, 5, 1, 1253, '单体SOH153', 2, 0, 4, 4152, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (802, 3, 2, 5, 1, 1254, '单体SOH154', 2, 0, 4, 4153, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (803, 3, 2, 5, 1, 1255, '单体SOH155', 2, 0, 4, 4154, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (804, 3, 2, 5, 1, 1256, '单体SOH156', 2, 0, 4, 4155, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (805, 3, 2, 5, 1, 1257, '单体SOH157', 2, 0, 4, 4156, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (806, 3, 2, 5, 1, 1258, '单体SOH158', 2, 0, 4, 4157, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (807, 3, 2, 5, 1, 1259, '单体SOH159', 2, 0, 4, 4158, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (808, 3, 2, 5, 1, 1260, '单体SOH160', 2, 0, 4, 4159, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (809, 3, 2, 5, 1, 1261, '单体SOH161', 2, 0, 4, 4160, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (810, 3, 2, 5, 1, 1262, '单体SOH162', 2, 0, 4, 4161, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (811, 3, 2, 5, 1, 1263, '单体SOH163', 2, 0, 4, 4162, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (812, 3, 2, 5, 1, 1264, '单体SOH164', 2, 0, 4, 4163, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (813, 3, 2, 5, 1, 1265, '单体SOH165', 2, 0, 4, 4164, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (814, 3, 2, 5, 1, 1266, '单体SOH166', 2, 0, 4, 4165, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (815, 3, 2, 5, 1, 1267, '单体SOH167', 2, 0, 4, 4166, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (816, 3, 2, 5, 1, 1268, '单体SOH168', 2, 0, 4, 4167, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (817, 3, 2, 5, 1, 1269, '单体SOH169', 2, 0, 4, 4168, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (818, 3, 2, 5, 1, 1270, '单体SOH170', 2, 0, 4, 4169, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (819, 3, 2, 5, 1, 1271, '单体SOH171', 2, 0, 4, 4170, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (820, 3, 2, 5, 1, 1272, '单体SOH172', 2, 0, 4, 4171, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (821, 3, 2, 5, 1, 1273, '单体SOH173', 2, 0, 4, 4172, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (822, 3, 2, 5, 1, 1274, '单体SOH174', 2, 0, 4, 4173, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (823, 3, 2, 5, 1, 1275, '单体SOH175', 2, 0, 4, 4174, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (824, 3, 2, 5, 1, 1276, '单体SOH176', 2, 0, 4, 4175, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (825, 3, 2, 5, 1, 1277, '单体SOH177', 2, 0, 4, 4176, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (826, 3, 2, 5, 1, 1278, '单体SOH178', 2, 0, 4, 4177, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (827, 3, 2, 5, 1, 1279, '单体SOH179', 2, 0, 4, 4178, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (828, 3, 2, 5, 1, 1280, '单体SOH180', 2, 0, 4, 4179, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (829, 3, 2, 5, 1, 1281, '单体SOH181', 2, 0, 4, 4180, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (830, 3, 2, 5, 1, 1282, '单体SOH182', 2, 0, 4, 4181, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (831, 3, 2, 5, 1, 1283, '单体SOH183', 2, 0, 4, 4182, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (832, 3, 2, 5, 1, 1284, '单体SOH184', 2, 0, 4, 4183, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (833, 3, 2, 5, 1, 1285, '单体SOH185', 2, 0, 4, 4184, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (834, 3, 2, 5, 1, 1286, '单体SOH186', 2, 0, 4, 4185, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (835, 3, 2, 5, 1, 1287, '单体SOH187', 2, 0, 4, 4186, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (836, 3, 2, 5, 1, 1288, '单体SOH188', 2, 0, 4, 4187, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (837, 3, 2, 5, 1, 1289, '单体SOH189', 2, 0, 4, 4188, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (838, 3, 2, 5, 1, 1290, '单体SOH190', 2, 0, 4, 4189, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (839, 3, 2, 5, 1, 1291, '单体SOH191', 2, 0, 4, 4190, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (840, 3, 2, 5, 1, 1292, '单体SOH192', 2, 0, 4, 4191, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (841, 3, 2, 5, 1, 1293, '单体SOH193', 2, 0, 4, 4192, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (842, 3, 2, 5, 1, 1294, '单体SOH194', 2, 0, 4, 4193, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (843, 3, 2, 5, 1, 1295, '单体SOH195', 2, 0, 4, 4194, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (844, 3, 2, 5, 1, 1296, '单体SOH196', 2, 0, 4, 4195, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (845, 3, 2, 5, 1, 1297, '单体SOH197', 2, 0, 4, 4196, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (846, 3, 2, 5, 1, 1298, '单体SOH198', 2, 0, 4, 4197, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (847, 3, 2, 5, 1, 1299, '单体SOH199', 2, 0, 4, 4198, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (848, 3, 2, 5, 1, 1300, '单体SOH200', 2, 0, 4, 4199, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, 0.0, 100.0, '%', 1);
INSERT INTO "template_point" VALUES (849, 3, 2, 5, 1, 1551, '单体温度1', 2, 0, 4, 2000, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (850, 3, 2, 5, 1, 1552, '单体温度2', 2, 0, 4, 2001, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (851, 3, 2, 5, 1, 1553, '单体温度3', 2, 0, 4, 2002, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (852, 3, 2, 5, 1, 1554, '单体温度4', 2, 0, 4, 2003, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (853, 3, 2, 5, 1, 1555, '单体温度5', 2, 0, 4, 2004, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (854, 3, 2, 5, 1, 1556, '单体温度6', 2, 0, 4, 2005, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (855, 3, 2, 5, 1, 1557, '单体温度7', 2, 0, 4, 2006, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (856, 3, 2, 5, 1, 1558, '单体温度8', 2, 0, 4, 2007, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (857, 3, 2, 5, 1, 1559, '单体温度9', 2, 0, 4, 2008, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (858, 3, 2, 5, 1, 1560, '单体温度10', 2, 0, 4, 2009, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (859, 3, 2, 5, 1, 1561, '单体温度11', 2, 0, 4, 2010, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (860, 3, 2, 5, 1, 1562, '单体温度12', 2, 0, 4, 2011, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (861, 3, 2, 5, 1, 1563, '单体温度13', 2, 0, 4, 2012, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (862, 3, 2, 5, 1, 1564, '单体温度14', 2, 0, 4, 2013, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (863, 3, 2, 5, 1, 1565, '单体温度15', 2, 0, 4, 2014, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (864, 3, 2, 5, 1, 1566, '单体温度16', 2, 0, 4, 2015, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (865, 3, 2, 5, 1, 1567, '单体温度17', 2, 0, 4, 2016, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (866, 3, 2, 5, 1, 1568, '单体温度18', 2, 0, 4, 2017, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (867, 3, 2, 5, 1, 1569, '单体温度19', 2, 0, 4, 2018, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (868, 3, 2, 5, 1, 1570, '单体温度20', 2, 0, 4, 2019, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (869, 3, 2, 5, 1, 1571, '单体温度21', 2, 0, 4, 2020, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (870, 3, 2, 5, 1, 1572, '单体温度22', 2, 0, 4, 2021, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (871, 3, 2, 5, 1, 1573, '单体温度23', 2, 0, 4, 2022, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (872, 3, 2, 5, 1, 1574, '单体温度24', 2, 0, 4, 2023, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (873, 3, 2, 5, 1, 1575, '单体温度25', 2, 0, 4, 2024, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (874, 3, 2, 5, 1, 1576, '单体温度26', 2, 0, 4, 2025, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (875, 3, 2, 5, 1, 1577, '单体温度27', 2, 0, 4, 2026, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (876, 3, 2, 5, 1, 1578, '单体温度28', 2, 0, 4, 2027, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (877, 3, 2, 5, 1, 1579, '单体温度29', 2, 0, 4, 2028, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (878, 3, 2, 5, 1, 1580, '单体温度30', 2, 0, 4, 2029, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (879, 3, 2, 5, 1, 1581, '单体温度31', 2, 0, 4, 2030, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (880, 3, 2, 5, 1, 1582, '单体温度32', 2, 0, 4, 2031, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (881, 3, 2, 5, 1, 1583, '单体温度33', 2, 0, 4, 2032, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (882, 3, 2, 5, 1, 1584, '单体温度34', 2, 0, 4, 2033, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (883, 3, 2, 5, 1, 1585, '单体温度35', 2, 0, 4, 2034, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (884, 3, 2, 5, 1, 1586, '单体温度36', 2, 0, 4, 2035, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (885, 3, 2, 5, 1, 1587, '单体温度37', 2, 0, 4, 2036, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (886, 3, 2, 5, 1, 1588, '单体温度38', 2, 0, 4, 2037, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (887, 3, 2, 5, 1, 1589, '单体温度39', 2, 0, 4, 2038, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (888, 3, 2, 5, 1, 1590, '单体温度40', 2, 0, 4, 2039, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (889, 3, 2, 5, 1, 1591, '单体温度41', 2, 0, 4, 2040, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (890, 3, 2, 5, 1, 1592, '单体温度42', 2, 0, 4, 2041, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (891, 3, 2, 5, 1, 1593, '单体温度43', 2, 0, 4, 2042, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (892, 3, 2, 5, 1, 1594, '单体温度44', 2, 0, 4, 2043, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (893, 3, 2, 5, 1, 1595, '单体温度45', 2, 0, 4, 2044, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (894, 3, 2, 5, 1, 1596, '单体温度46', 2, 0, 4, 2045, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (895, 3, 2, 5, 1, 1597, '单体温度47', 2, 0, 4, 2046, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (896, 3, 2, 5, 1, 1598, '单体温度48', 2, 0, 4, 2047, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (897, 3, 2, 5, 1, 1599, '单体温度49', 2, 0, 4, 2048, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (898, 3, 2, 5, 1, 1600, '单体温度50', 2, 0, 4, 2049, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (899, 3, 2, 5, 1, 1601, '单体温度51', 2, 0, 4, 2050, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (900, 3, 2, 5, 1, 1602, '单体温度52', 2, 0, 4, 2051, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (901, 3, 2, 5, 1, 1603, '单体温度53', 2, 0, 4, 2052, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (902, 3, 2, 5, 1, 1604, '单体温度54', 2, 0, 4, 2053, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (903, 3, 2, 5, 1, 1605, '单体温度55', 2, 0, 4, 2054, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (904, 3, 2, 5, 1, 1606, '单体温度56', 2, 0, 4, 2055, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (905, 3, 2, 5, 1, 1607, '单体温度57', 2, 0, 4, 2056, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (906, 3, 2, 5, 1, 1608, '单体温度58', 2, 0, 4, 2057, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (907, 3, 2, 5, 1, 1609, '单体温度59', 2, 0, 4, 2058, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (908, 3, 2, 5, 1, 1610, '单体温度60', 2, 0, 4, 2059, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (909, 3, 2, 5, 1, 1611, '单体温度61', 2, 0, 4, 2060, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (910, 3, 2, 5, 1, 1612, '单体温度62', 2, 0, 4, 2061, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (911, 3, 2, 5, 1, 1613, '单体温度63', 2, 0, 4, 2062, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (912, 3, 2, 5, 1, 1614, '单体温度64', 2, 0, 4, 2063, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (913, 3, 2, 5, 1, 1615, '单体温度65', 2, 0, 4, 2064, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (914, 3, 2, 5, 1, 1616, '单体温度66', 2, 0, 4, 2065, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (915, 3, 2, 5, 1, 1617, '单体温度67', 2, 0, 4, 2066, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (916, 3, 2, 5, 1, 1618, '单体温度68', 2, 0, 4, 2067, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (917, 3, 2, 5, 1, 1619, '单体温度69', 2, 0, 4, 2068, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (918, 3, 2, 5, 1, 1620, '单体温度70', 2, 0, 4, 2069, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (919, 3, 2, 5, 1, 1621, '单体温度71', 2, 0, 4, 2070, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (920, 3, 2, 5, 1, 1622, '单体温度72', 2, 0, 4, 2071, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (921, 3, 2, 5, 1, 1623, '单体温度73', 2, 0, 4, 2072, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (922, 3, 2, 5, 1, 1624, '单体温度74', 2, 0, 4, 2073, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (923, 3, 2, 5, 1, 1625, '单体温度75', 2, 0, 4, 2074, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (924, 3, 2, 5, 1, 1626, '单体温度76', 2, 0, 4, 2075, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (925, 3, 2, 5, 1, 1627, '单体温度77', 2, 0, 4, 2076, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (926, 3, 2, 5, 1, 1628, '单体温度78', 2, 0, 4, 2077, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (927, 3, 2, 5, 1, 1629, '单体温度79', 2, 0, 4, 2078, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (928, 3, 2, 5, 1, 1630, '单体温度80', 2, 0, 4, 2079, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (929, 3, 2, 5, 1, 1631, '单体温度81', 2, 0, 4, 2080, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (930, 3, 2, 5, 1, 1632, '单体温度82', 2, 0, 4, 2081, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (931, 3, 2, 5, 1, 1633, '单体温度83', 2, 0, 4, 2082, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (932, 3, 2, 5, 1, 1634, '单体温度84', 2, 0, 4, 2083, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (933, 3, 2, 5, 1, 1635, '单体温度85', 2, 0, 4, 2084, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (934, 3, 2, 5, 1, 1636, '单体温度86', 2, 0, 4, 2085, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (935, 3, 2, 5, 1, 1637, '单体温度87', 2, 0, 4, 2086, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (936, 3, 2, 5, 1, 1638, '单体温度88', 2, 0, 4, 2087, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (937, 3, 2, 5, 1, 1639, '单体温度89', 2, 0, 4, 2088, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (938, 3, 2, 5, 1, 1640, '单体温度90', 2, 0, 4, 2089, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (939, 3, 2, 5, 1, 1641, '单体温度91', 2, 0, 4, 2090, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (940, 3, 2, 5, 1, 1642, '单体温度92', 2, 0, 4, 2091, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (941, 3, 2, 5, 1, 1643, '单体温度93', 2, 0, 4, 2092, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (942, 3, 2, 5, 1, 1644, '单体温度94', 2, 0, 4, 2093, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (943, 3, 2, 5, 1, 1645, '单体温度95', 2, 0, 4, 2094, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (944, 3, 2, 5, 1, 1646, '单体温度96', 2, 0, 4, 2095, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (945, 3, 2, 5, 1, 1647, '单体温度97', 2, 0, 4, 2096, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (946, 3, 2, 5, 1, 1648, '单体温度98', 2, 0, 4, 2097, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (947, 3, 2, 5, 1, 1649, '单体温度99', 2, 0, 4, 2098, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (948, 3, 2, 5, 1, 1650, '单体温度100', 2, 0, 4, 2099, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (949, 3, 2, 5, 1, 1651, '单体温度101', 2, 0, 4, 2100, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (950, 3, 2, 5, 1, 1652, '单体温度102', 2, 0, 4, 2101, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (951, 3, 2, 5, 1, 1653, '单体温度103', 2, 0, 4, 2102, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (952, 3, 2, 5, 1, 1654, '单体温度104', 2, 0, 4, 2103, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (953, 3, 2, 5, 1, 1655, '单体温度105', 2, 0, 4, 2104, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (954, 3, 2, 5, 1, 1656, '单体温度106', 2, 0, 4, 2105, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (955, 3, 2, 5, 1, 1657, '单体温度107', 2, 0, 4, 2106, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (956, 3, 2, 5, 1, 1658, '单体温度108', 2, 0, 4, 2107, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (957, 3, 2, 5, 1, 1659, '单体温度109', 2, 0, 4, 2108, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (958, 3, 2, 5, 1, 1660, '单体温度110', 2, 0, 4, 2109, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (959, 3, 2, 5, 1, 1661, '单体温度111', 2, 0, 4, 2110, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (960, 3, 2, 5, 1, 1662, '单体温度112', 2, 0, 4, 2111, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (961, 3, 2, 5, 1, 1663, '单体温度113', 2, 0, 4, 2112, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (962, 3, 2, 5, 1, 1664, '单体温度114', 2, 0, 4, 2113, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (963, 3, 2, 5, 1, 1665, '单体温度115', 2, 0, 4, 2114, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (964, 3, 2, 5, 1, 1666, '单体温度116', 2, 0, 4, 2115, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (965, 3, 2, 5, 1, 1667, '单体温度117', 2, 0, 4, 2116, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (966, 3, 2, 5, 1, 1668, '单体温度118', 2, 0, 4, 2117, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (967, 3, 2, 5, 1, 1669, '单体温度119', 2, 0, 4, 2118, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (968, 3, 2, 5, 1, 1670, '单体温度120', 2, 0, 4, 2119, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (969, 3, 2, 5, 1, 1671, '单体温度121', 2, 0, 4, 2120, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (970, 3, 2, 5, 1, 1672, '单体温度122', 2, 0, 4, 2121, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (971, 3, 2, 5, 1, 1673, '单体温度123', 2, 0, 4, 2122, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (972, 3, 2, 5, 1, 1674, '单体温度124', 2, 0, 4, 2123, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (973, 3, 2, 5, 1, 1675, '单体温度125', 2, 0, 4, 2124, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (974, 3, 2, 5, 1, 1676, '单体温度126', 2, 0, 4, 2125, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (975, 3, 2, 5, 1, 1677, '单体温度127', 2, 0, 4, 2126, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (976, 3, 2, 5, 1, 1678, '单体温度128', 2, 0, 4, 2127, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (977, 3, 2, 5, 1, 1679, '单体温度129', 2, 0, 4, 2128, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (978, 3, 2, 5, 1, 1680, '单体温度130', 2, 0, 4, 2129, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (979, 3, 2, 5, 1, 1681, '单体温度131', 2, 0, 4, 2130, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (980, 3, 2, 5, 1, 1682, '单体温度132', 2, 0, 4, 2131, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (981, 3, 2, 5, 1, 1683, '单体温度133', 2, 0, 4, 2132, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (982, 3, 2, 5, 1, 1684, '单体温度134', 2, 0, 4, 2133, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (983, 3, 2, 5, 1, 1685, '单体温度135', 2, 0, 4, 2134, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (984, 3, 2, 5, 1, 1686, '单体温度136', 2, 0, 4, 2135, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (985, 3, 2, 5, 1, 1687, '单体温度137', 2, 0, 4, 2136, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (986, 3, 2, 5, 1, 1688, '单体温度138', 2, 0, 4, 2137, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (987, 3, 2, 5, 1, 1689, '单体温度139', 2, 0, 4, 2138, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (988, 3, 2, 5, 1, 1690, '单体温度140', 2, 0, 4, 2139, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (989, 3, 2, 5, 1, 1691, '单体温度141', 2, 0, 4, 2140, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (990, 3, 2, 5, 1, 1692, '单体温度142', 2, 0, 4, 2141, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (991, 3, 2, 5, 1, 1693, '单体温度143', 2, 0, 4, 2142, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (992, 3, 2, 5, 1, 1694, '单体温度144', 2, 0, 4, 2143, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (993, 3, 2, 5, 1, 1695, '单体温度145', 2, 0, 4, 2144, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (994, 3, 2, 5, 1, 1696, '单体温度146', 2, 0, 4, 2145, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (995, 3, 2, 5, 1, 1697, '单体温度147', 2, 0, 4, 2146, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (996, 3, 2, 5, 1, 1698, '单体温度148', 2, 0, 4, 2147, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (997, 3, 2, 5, 1, 1699, '单体温度149', 2, 0, 4, 2148, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (998, 3, 2, 5, 1, 1700, '单体温度150', 2, 0, 4, 2149, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (999, 3, 2, 5, 1, 1701, '单体温度151', 2, 0, 4, 2150, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1000, 3, 2, 5, 1, 1702, '单体温度152', 2, 0, 4, 2151, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1001, 3, 2, 5, 1, 1703, '单体温度153', 2, 0, 4, 2152, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1002, 3, 2, 5, 1, 1704, '单体温度154', 2, 0, 4, 2153, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1003, 3, 2, 5, 1, 1705, '单体温度155', 2, 0, 4, 2154, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1004, 3, 2, 5, 1, 1706, '单体温度156', 2, 0, 4, 2155, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1005, 3, 2, 5, 1, 1707, '单体温度157', 2, 0, 4, 2156, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1006, 3, 2, 5, 1, 1708, '单体温度158', 2, 0, 4, 2157, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1007, 3, 2, 5, 1, 1709, '单体温度159', 2, 0, 4, 2158, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1008, 3, 2, 5, 1, 1710, '单体温度160', 2, 0, 4, 2159, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1009, 3, 2, 5, 1, 1711, '单体温度161', 2, 0, 4, 2160, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1010, 3, 2, 5, 1, 1712, '单体温度162', 2, 0, 4, 2161, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1011, 3, 2, 5, 1, 1713, '单体温度163', 2, 0, 4, 2162, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1012, 3, 2, 5, 1, 1714, '单体温度164', 2, 0, 4, 2163, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1013, 3, 2, 5, 1, 1715, '单体温度165', 2, 0, 4, 2164, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1014, 3, 2, 5, 1, 1716, '单体温度166', 2, 0, 4, 2165, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1015, 3, 2, 5, 1, 1717, '单体温度167', 2, 0, 4, 2166, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1016, 3, 2, 5, 1, 1718, '单体温度168', 2, 0, 4, 2167, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1017, 3, 2, 5, 1, 1719, '单体温度169', 2, 0, 4, 2168, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1018, 3, 2, 5, 1, 1720, '单体温度170', 2, 0, 4, 2169, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1019, 3, 2, 5, 1, 1721, '单体温度171', 2, 0, 4, 2170, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1020, 3, 2, 5, 1, 1722, '单体温度172', 2, 0, 4, 2171, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1021, 3, 2, 5, 1, 1723, '单体温度173', 2, 0, 4, 2172, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1022, 3, 2, 5, 1, 1724, '单体温度174', 2, 0, 4, 2173, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1023, 3, 2, 5, 1, 1725, '单体温度175', 2, 0, 4, 2174, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1024, 3, 2, 5, 1, 1726, '单体温度176', 2, 0, 4, 2175, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1025, 3, 2, 5, 1, 1727, '单体温度177', 2, 0, 4, 2176, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1026, 3, 2, 5, 1, 1728, '单体温度178', 2, 0, 4, 2177, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1027, 3, 2, 5, 1, 1729, '单体温度179', 2, 0, 4, 2178, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1028, 3, 2, 5, 1, 1730, '单体温度180', 2, 0, 4, 2179, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1029, 3, 2, 5, 1, 1731, '单体温度181', 2, 0, 4, 2180, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1030, 3, 2, 5, 1, 1732, '单体温度182', 2, 0, 4, 2181, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1031, 3, 2, 5, 1, 1733, '单体温度183', 2, 0, 4, 2182, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1032, 3, 2, 5, 1, 1734, '单体温度184', 2, 0, 4, 2183, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1033, 3, 2, 5, 1, 1735, '单体温度185', 2, 0, 4, 2184, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1034, 3, 2, 5, 1, 1736, '单体温度186', 2, 0, 4, 2185, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1035, 3, 2, 5, 1, 1737, '单体温度187', 2, 0, 4, 2186, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1036, 3, 2, 5, 1, 1738, '单体温度188', 2, 0, 4, 2187, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1037, 3, 2, 5, 1, 1739, '单体温度189', 2, 0, 4, 2188, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1038, 3, 2, 5, 1, 1740, '单体温度190', 2, 0, 4, 2189, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1039, 3, 2, 5, 1, 1741, '单体温度191', 2, 0, 4, 2190, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1040, 3, 2, 5, 1, 1742, '单体温度192', 2, 0, 4, 2191, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1041, 3, 2, 5, 1, 1743, '单体温度193', 2, 0, 4, 2192, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1042, 3, 2, 5, 1, 1744, '单体温度194', 2, 0, 4, 2193, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1043, 3, 2, 5, 1, 1745, '单体温度195', 2, 0, 4, 2194, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1044, 3, 2, 5, 1, 1746, '单体温度196', 2, 0, 4, 2195, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1045, 3, 2, 5, 1, 1747, '单体温度197', 2, 0, 4, 2196, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1046, 3, 2, 5, 1, 1748, '单体温度198', 2, 0, 4, 2197, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1047, 3, 2, 5, 1, 1749, '单体温度199', 2, 0, 4, 2198, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1048, 3, 2, 5, 1, 1750, '单体温度200', 2, 0, 4, 2199, 0, 0, 0, 0.1, 1.0, -40.0, 1, 5000, 1.0, 0.0, -40.0, 120.0, '℃', 1);
INSERT INTO "template_point" VALUES (1049, 3, 2, 5, 1, 1751, '高压下电指令', 2, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 1.0, 0.0, NULL, NULL, NULL, 1);
INSERT INTO "template_point" VALUES (1050, 4, 2, 1, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1051, 4, 2, 1, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1052, 4, 2, 1, 1, 2, 'A相电压', 2, 0, 3, 97, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1053, 4, 2, 1, 1, 3, 'B相电压', 2, 0, 3, 98, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1054, 4, 2, 1, 1, 4, 'C相电压', 2, 0, 3, 99, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1055, 4, 2, 1, 1, 5, 'AB线电压', 2, 0, 3, 120, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1056, 4, 2, 1, 1, 6, 'BC线电压', 2, 0, 3, 121, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1057, 4, 2, 1, 1, 7, 'CA线电压', 2, 0, 3, 122, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1058, 4, 2, 1, 1, 8, 'A相电流', 3, 0, 3, 100, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "template_point" VALUES (1059, 4, 2, 1, 1, 9, 'B相电流', 3, 0, 3, 101, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "template_point" VALUES (1060, 4, 2, 1, 1, 10, 'C相电流', 3, 0, 3, 102, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "template_point" VALUES (1061, 4, 2, 1, 1, 11, 'A相有功功率', 5, 0, 3, 356, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "template_point" VALUES (1062, 4, 2, 1, 1, 12, 'B相有功功率', 5, 0, 3, 358, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "template_point" VALUES (1063, 4, 2, 1, 1, 13, 'C相有功功率', 5, 0, 3, 360, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "template_point" VALUES (1064, 4, 2, 1, 1, 14, '总有功功率', 5, 0, 3, 362, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "template_point" VALUES (1065, 4, 2, 1, 1, 15, 'A相无功功率', 5, 0, 3, 364, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1066, 4, 2, 1, 1, 16, 'B相无功功率', 5, 0, 3, 366, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1067, 4, 2, 1, 1, 17, 'C相无功功率', 5, 0, 3, 368, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1068, 4, 2, 1, 1, 18, '总无功功率', 5, 0, 3, 370, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1069, 4, 2, 1, 1, 19, 'A相视在功率', 5, 0, 3, 372, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1070, 4, 2, 1, 1, 20, 'B相视在功率', 5, 0, 3, 374, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1071, 4, 2, 1, 1, 21, 'C相视在功率', 5, 0, 3, 376, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1072, 4, 2, 1, 1, 22, '总视在功率', 5, 0, 3, 378, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1073, 4, 2, 1, 1, 23, 'A相功率因数', 2, 0, 3, 380, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1074, 4, 2, 1, 1, 24, 'B相功率因数', 2, 0, 3, 381, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1075, 4, 2, 1, 1, 25, 'C相功率因数', 2, 0, 3, 382, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1076, 4, 2, 1, 1, 26, '总功率因数', 2, 0, 3, 383, 0, 0, 0, 0.001, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1077, 4, 2, 1, 1, 27, '电网频率', 2, 0, 3, 119, 0, 0, 0, 0.01, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'HZ', 1);
INSERT INTO "template_point" VALUES (1078, 4, 2, 1, 1, 28, '电压互感比', 2, 0, 3, 141, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1079, 4, 2, 1, 1, 29, '电流互感比', 2, 0, 3, 142, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1080, 4, 2, 1, 1, 30, '正向有功电度', 4, 0, 3, 10, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1081, 4, 2, 1, 1, 31, '反向有功电度', 4, 0, 3, 20, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1082, 4, 2, 1, 1, 32, '正向无功电度', 4, 0, 3, 40, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1083, 4, 2, 1, 1, 33, '反向无功电度', 4, 0, 3, 50, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1084, 4, 2, 1, 1, 34, '尖正向有功电度', 4, 0, 3, 12, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1085, 4, 2, 1, 1, 35, '尖反向有功电度', 4, 0, 3, 22, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1086, 4, 2, 1, 1, 36, '峰正向有功电度', 4, 0, 3, 14, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1087, 4, 2, 1, 1, 37, '峰反向有功电度', 4, 0, 3, 24, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1088, 4, 2, 1, 1, 38, '平正向有功电度', 4, 0, 3, 16, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1089, 4, 2, 1, 1, 39, '平反向有功电度', 4, 0, 3, 26, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1090, 4, 2, 1, 1, 40, '谷正向有功电度', 4, 0, 3, 18, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1091, 4, 2, 1, 1, 41, '谷反向有功电度', 4, 0, 3, 28, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1092, 4, 2, 1, 1, 42, '当月正向有功最大需量', 2, 0, 3, 123, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kw', 1);
INSERT INTO "template_point" VALUES (1093, 4, 2, 1, 1, 43, '当月反向有功最大需量', 2, 0, 3, 126, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kw', 1);
INSERT INTO "template_point" VALUES (1094, 5, 2, 2, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1095, 5, 2, 2, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1096, 5, 2, 2, 1, 2, 'A相电压', 2, 0, 3, 97, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1097, 5, 2, 2, 1, 3, 'B相电压', 2, 0, 3, 98, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1098, 5, 2, 2, 1, 4, 'C相电压', 2, 0, 3, 99, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1099, 5, 2, 2, 1, 5, 'AB线电压', 2, 0, 3, 120, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1100, 5, 2, 2, 1, 6, 'BC线电压', 2, 0, 3, 121, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1101, 5, 2, 2, 1, 7, 'CA线电压', 2, 0, 3, 122, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'V', 1);
INSERT INTO "template_point" VALUES (1102, 5, 2, 2, 1, 8, 'A相电流', 3, 0, 3, 100, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "template_point" VALUES (1103, 5, 2, 2, 1, 9, 'B相电流', 3, 0, 3, 101, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "template_point" VALUES (1104, 5, 2, 2, 1, 10, 'C相电流', 3, 0, 3, 102, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'A', 1);
INSERT INTO "template_point" VALUES (1105, 5, 2, 2, 1, 11, 'A相有功功率', 5, 0, 3, 356, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "template_point" VALUES (1106, 5, 2, 2, 1, 12, 'B相有功功率', 5, 0, 3, 358, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "template_point" VALUES (1107, 5, 2, 2, 1, 13, 'C相有功功率', 5, 0, 3, 360, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "template_point" VALUES (1108, 5, 2, 2, 1, 14, '总有功功率', 5, 0, 3, 362, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kW', 1);
INSERT INTO "template_point" VALUES (1109, 5, 2, 2, 1, 15, 'A相无功功率', 5, 0, 3, 364, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1110, 5, 2, 2, 1, 16, 'B相无功功率', 5, 0, 3, 366, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1111, 5, 2, 2, 1, 17, 'C相无功功率', 5, 0, 3, 368, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1112, 5, 2, 2, 1, 18, '总无功功率', 5, 0, 3, 370, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1113, 5, 2, 2, 1, 19, 'A相视在功率', 5, 0, 3, 372, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1114, 5, 2, 2, 1, 20, 'B相视在功率', 5, 0, 3, 374, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1115, 5, 2, 2, 1, 21, 'C相视在功率', 5, 0, 3, 376, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1116, 5, 2, 2, 1, 22, '总视在功率', 5, 0, 3, 378, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1117, 5, 2, 2, 1, 23, 'A相功率因数', 2, 0, 3, 380, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1118, 5, 2, 2, 1, 24, 'B相功率因数', 2, 0, 3, 381, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1119, 5, 2, 2, 1, 25, 'C相功率因数', 2, 0, 3, 382, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1120, 5, 2, 2, 1, 26, '总功率因数', 2, 0, 3, 383, 0, 0, 0, 0.001, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1121, 5, 2, 2, 1, 27, '电网频率', 2, 0, 3, 119, 0, 0, 0, 0.01, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'HZ', 1);
INSERT INTO "template_point" VALUES (1122, 5, 2, 2, 1, 28, '电压互感比', 2, 0, 3, 141, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1123, 5, 2, 2, 1, 29, '电流互感比', 2, 0, 3, 142, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1124, 5, 2, 2, 1, 30, '正向有功电度', 4, 0, 3, 10, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1125, 5, 2, 2, 1, 31, '反向有功电度', 4, 0, 3, 20, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1126, 5, 2, 2, 1, 32, '正向无功电度', 4, 0, 3, 40, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1127, 5, 2, 2, 1, 33, '反向无功电度', 4, 0, 3, 50, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kVar', 1);
INSERT INTO "template_point" VALUES (1128, 5, 2, 2, 1, 34, '尖正向有功电度', 4, 0, 3, 12, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1129, 5, 2, 2, 1, 35, '尖反向有功电度', 4, 0, 3, 22, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1130, 5, 2, 2, 1, 36, '峰正向有功电度', 4, 0, 3, 14, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1131, 5, 2, 2, 1, 37, '峰反向有功电度', 4, 0, 3, 24, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1132, 5, 2, 2, 1, 38, '平正向有功电度', 4, 0, 3, 16, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1133, 5, 2, 2, 1, 39, '平反向有功电度', 4, 0, 3, 26, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1134, 5, 2, 2, 1, 40, '谷正向有功电度', 4, 0, 3, 18, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1135, 5, 2, 2, 1, 41, '谷反向有功电度', 4, 0, 3, 28, 0, 0, 0, 0.1, 0.01, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kwh', 1);
INSERT INTO "template_point" VALUES (1136, 5, 2, 2, 1, 42, '当月正向有功最大需量', 2, 0, 3, 123, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kw', 1);
INSERT INTO "template_point" VALUES (1137, 5, 2, 2, 1, 43, '当月反向有功最大需量', 2, 0, 3, 126, 0, 0, 0, 0.1, 0.001, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, 'kw', 1);
INSERT INTO "template_point" VALUES (1138, 6, 2, 10, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1139, 6, 2, 10, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1140, 6, 2, 10, 1, 2, '水浸一级告警', 2, 0, 4, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1141, 6, 2, 10, 1, 3, '水浸二级告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1142, 6, 2, 10, 1, 4, '水浸三级', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1143, 6, 2, 10, 1, 5, '断线告警', 0, 0, 4, 0, 2, 1, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1144, 6, 2, 10, 1, 6, '烟感告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1145, 7, 2, 10, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1146, 7, 2, 10, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1147, 7, 2, 10, 1, 2, '水浸一级告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1148, 7, 2, 10, 1, 3, '水浸二级告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1149, 7, 2, 10, 1, 4, '水浸三级', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1150, 7, 2, 10, 1, 5, '断线告警', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1151, 7, 2, 10, 1, 6, '烟感告警', 2, 0, 3, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1152, 8, 2, 9, 1, 0, '是否启用', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1153, 8, 2, 9, 1, 1, '是否在线', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1154, 8, 2, 9, 1, 2, '总故障', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1155, 8, 2, 9, 1, 41, '当前温度', 3, 0, 3, 3, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "template_point" VALUES (1156, 8, 2, 9, 1, 42, '当前湿度', 3, 0, 3, 3, 0, 0, 0, 0.1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '%RH', 1);
INSERT INTO "template_point" VALUES (1157, 8, 2, 9, 2, 71, '制冷温度', 3, 0, 6, 8, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "template_point" VALUES (1158, 8, 2, 9, 2, 72, '制热温度', 3, 0, 6, 9, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1159, 8, 2, 9, 2, 73, '制冷回差', 3, 0, 6, 10, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "template_point" VALUES (1160, 8, 2, 9, 2, 74, '制热回差', 0, 0, 0, 0, 0, 0, 0, 1, 0.1, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "template_point" VALUES (1161, 8, 2, 9, 2, 75, '运行模式', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1162, 8, 2, 9, 2, 76, '空调开关机', 2, 0, 5, 51, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '/', 1);
INSERT INTO "template_point" VALUES (1163, 8, 2, 9, 2, 77, '电芯最高温度', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "template_point" VALUES (1164, 8, 2, 9, 2, 78, '电芯最低温度', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);
INSERT INTO "template_point" VALUES (1165, 8, 2, 9, 2, 79, '电芯平均温度', 0, 0, 0, 0, 0, 0, 0, 1, 1.0, 0.0, 1, 5000, 0.5, 0.0, 0.0, 1.0, '℃', 1);

-- ----------------------------
-- Records of template_protocol
-- ----------------------------
INSERT INTO "template_protocol" VALUES (1, 1, 0, 1, '0', '{"netId": 1, "ip": "192.168.3.25", "port": 2645, "address":1, "timeout": 3000}', '备注');
INSERT INTO "template_protocol" VALUES (2, 2, 0, 2, '1', '{"uartId":1,"data":3,"stop":0,"baud":0,"parity":0,"address":1, "timeout": 3000}', '备注');

-- ----------------------------
-- Records of topology
-- ----------------------------
INSERT INTO "topology" VALUES (0, -999, 1, '1#EMS', 0, 0, '0');
INSERT INTO "topology" VALUES (1, 0, 1, '1#PCS', 1, 1, '1');
INSERT INTO "topology" VALUES (2, 0, 1, '1#BMS', 2, 2, '2');
INSERT INTO "topology" VALUES (3, 0, 1, '1#METER', 3, 3, '3');
INSERT INTO "topology" VALUES (4, 2, 1, '1#BCU', 4, 4, '4');
INSERT INTO "topology" VALUES (5, 0, 1, '2#METER', 5, 5, '5');

-- ----------------------------
-- Records of up_dis_point
-- ----------------------------
INSERT INTO "up_dis_point" VALUES (0, 0, '0', '0', 0, '0');

-- ----------------------------
-- Records of work_mode_set
-- ----------------------------
INSERT INTO "work_mode_set" VALUES (1, 3, '1');

-- ----------------------------
-- Records of debug_algorithm
-- ----------------------------
INSERT INTO "debug_algorithm" VALUES (1, 0, 0, 0, 1, '调试算法配置');

-- ----------------------------
-- Auto increment value for allocate_power
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 1 WHERE name = 'allocate_power';

-- ----------------------------
-- Auto increment value for device
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 9 WHERE name = 'device';

-- ----------------------------
-- Auto increment value for ems_device_config
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 16 WHERE name = 'ems_device_config';

-- ----------------------------
-- Auto increment value for point
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 1165 WHERE name = 'point';

-- ----------------------------
-- Auto increment value for protect_algorithm
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 1 WHERE name = 'protect_algorithm';

-- ----------------------------
-- Auto increment value for protocol
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 9 WHERE name = 'protocol';

-- ----------------------------
-- Auto increment value for pv_date_config
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 1 WHERE name = 'pv_date_config';

-- ----------------------------
-- Auto increment value for pv_time_config
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 1 WHERE name = 'pv_time_config';

-- ----------------------------
-- Auto increment value for pv_time_exe
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 1 WHERE name = 'pv_time_exe';

-- ----------------------------
-- Auto increment value for template
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 8 WHERE name = 'template';

-- ----------------------------
-- Auto increment value for template_point
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 1165 WHERE name = 'template_point';

-- ----------------------------
-- Auto increment value for template_protocol
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 2 WHERE name = 'template_protocol';

-- ----------------------------
-- Auto increment value for topology
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 5 WHERE name = 'topology';

-- ----------------------------
-- Auto increment value for work_mode_set
-- ----------------------------
UPDATE "sqlite_sequence" SET seq = 1 WHERE name = 'work_mode_set';
